#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   This file is part of IntiPunku
#   Copyright (C) 2007 Mark Baas <mark.baas123@gmail.com>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import inti.intigtk as gtk
import gobject, pango
import os, os.path, time

import inti.utils as utils
import tools
import urllib

from inti.constants import *

# Localization
import gettext
_ = gettext.gettext

import mimetypes

class SunBrowser(gtk.HPaned):
    __gsignals__ = {
        'album-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,)),
        }
    def __init__(self, callback, imindex, dindex, tagindex, win, config):
        gtk.HPaned.__init__(self)
        gobject.GObject.__init__(self)

        self.imindex = imindex
        self.dindex = dindex
        self.tagindex = tagindex
        self.album = ""
        self.callback = callback
        self.collections = []
        self.config = config

        # default widget settings
        self.set_position(200)


        # Treeview: folders
        self.tv = gtk.TreeView()
        self.tm = gtk.TreeStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
        self.tv.set_model(self.tm)

        render_textp = gtk.CellRendererText()
        render_text = gtk.CellRendererText()
        render_text.set_property('editable', True)
        render_text.connect('edited', self.cell_edited_cb)
        self.render = render_text

        column = gtk.TreeViewColumn()
        column.pack_start(render_text, expand=True)
        column.add_attribute(render_text, "text", 1)
        column.set_title(_("Albums"))

        self.tv.append_column(column)
        self.tv.show()
        self.tv.set_headers_visible(True)
        self.tv.set_name("tvbrowser")
        self.tv.connect("button-release-event", self.album_popup)
        self.tv.enable_model_drag_dest([('text/plain', 0, 0)],
                  gtk.gdk.ACTION_DEFAULT | gtk.gdk.ACTION_COPY)
        self.tv.connect("drag-data-received", self.albums_drag_received)
        self.selection = self.tv.get_selection()
        self.selection.connect("changed", self.selection_cb)
        self.selection.set_select_function(self.selection_func, data=None, full=False)

        sw = gtk.ScrolledWindow()
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        sw.add(self.tv)
        
        # Photo viewer
        self.viewer = SunAlbum(self.imindex, self.dindex, win, config, callback=callback)

        # Combo box
        combobox = gtk.combo_box_new_text()
        combobox.insert_text(0, _("Folders"))
        combobox.insert_text(1, _("Tags"))
        combobox.set_active(0)
        combobox.connect("changed", self.combobox_cb)
        self.viewtype = 0


        vbox = gtk.VBox()
        vbox.pack_start(combobox, fill=False, expand=False)
        vbox.pack_end(sw)
        self.pack1(vbox)
        self.pack2(self.viewer)
        #self.show_all()


        gobject.idle_add(self.init)


    def init(self):
        self.load_albums()
        i = self.tm.get_iter((0,0))
        self.selection.select_iter(i)
        self.viewer.checksize_cb()

    def combobox_cb(self, widget, *args):
        self.selection.unselect_all()
        self.viewtype = widget.get_active()
        self.tm.clear()
        if self.viewtype == 1:
            self.render.set_property('editable', False)
            self.load_tags()
        else:
            self.render.set_property('editable', True)
            self.load_folders()
         

    def load_albums(self, select=""):

        if self.viewtype == 1:
            self.render.set_property('editable', False)
            self.load_tags(select=select)
        else:
            self.render.set_property('editable', True)
            self.load_folders(select=select)

    def load_folders(self, select=""):
        self.viewer.imindex = self.imindex
        # Loading collection
        collections = []
        coldict = {}
        ims = []
        for im in self.imindex:
            c = os.path.basename(os.path.dirname(im))
            if not c in collections:
                coldict[c] = []
                collections.append(c)

        for im in self.imindex:
            c = os.path.basename(os.path.dirname(im))
            date = self.dindex[im]
            coldict[c].append(date)

        for album in coldict:
            coldict[album].sort()
            

        # Sort on date
        if self.config["albums_sort_date"]:
            collections.sort(cmp=lambda x,y: cmp(coldict[y][-1],coldict[x][-1]))
        else:
            collections.sort()
            iterp = self.tm.append(None)
            self.tm.set_value(iterp, 1, _("Albums"))
        years = []

        for c in collections:
            if self.config["albums_sort_date"]:
                year = coldict[c][-1].year
                if not year in years:
                    iterp = self.tm.append(None)
                    self.tm.set_value(iterp, 1, year)
                    years.append(year)
            iter = self.tm.append(iterp)
            self.tm.set_value(iter, 1, c)
                
            if c == select:
                self.selection.select_iter(iter)
        self.collections = collections
        self.tv.expand_all()
        
        # Change icon
        self.viewer.browserim.set_from_pixbuf(utils.load_icon("stock_folder",48))
        self.viewer.progress_icon = "stock_folder"


    def load_tags(self, select=""):

        self.viewer.imindex = self.imindex
        self.viewer.tagindex = self.tagindex
        # Loading collection
        collections = []
        for im in self.tagindex:
            for tag in self.tagindex[im]:
                if not tag in collections:  
                    collections.append(tag)    

        for c in collections:
            iter = self.tm.append(None)
            self.tm.set_value(iter, 1, c)
            if c == select:
                self.selection.select_iter(iter)

        self.tv.expand_all()
        # Change icon
        self.viewer.browserim.set_from_pixbuf(utils.load_icon("tag2",48))
        self.viewer.progress_icon = "tag2"

    def selection_cb(self, widget):
        tm, iter = widget.get_selected()
        if not iter.__class__ == gtk.TreeIter:
            return 
        colvalue = tm.get_value(iter, 1)
        self.album = colvalue
        self.viewer.update(colvalue, vt=self.viewtype)
        self.emit("album-changed", self.viewer.box_load)
        self.cellpath = tm.get_path(iter)

    def selection_func(self, path, *args):
        if len(path) < 2:
            if self.viewtype == 1:
                return True
            else:
                return False
        return True

    def get_selection(self, select_all=False):
        selection = []
        if select_all == False:
            sel = self.viewer.box_selection
        else:
            sel = self.viewer.box_load
        for box in sel:
            f = box.get_children()[0].get_name()
            if not os.path.splitext(f)[1].lower() in EXT_VID:
                
                selection.append(f)
        return selection

    def set_selection(self, selection):
        for s in selection:
            for box in self.viewer.box_load:
                if box.get_children()[0].get_name() == s:
                    self.viewer.select_boxes_b1(box)
                

    def album_popup(self, widget, event):
        if event.button == 3 and self.viewtype == 0:
            self.callback("album_popup")

    def set_editing(self):
        self.tv.set_cursor_on_cell(self.cellpath, self.tv.get_column(0), self.render, True)

    def cell_edited_cb(self, widget, pos, new):
        if self.album != new:
            self.callback("rename_album", new)
            self.refresh_albums(new)

    def refresh_albums(self, select=""):
        self.tm.clear()
        self.load_albums(select)

    def select_all(self):
        self.viewer.select_box_all()
        
    def albums_drag_received(self, treeview, context, x, y, selection, info, timestamp):
        self.viewer.dragging = False
        # Drag drop only works with folder view
        if self.viewtype == 1: return
        
        path, pos = self.tv.get_dest_row_at_pos(x, y)
        if pos == gtk.TREE_VIEW_DROP_INTO_OR_BEFORE or pos == gtk.TREE_VIEW_DROP_INTO_OR_AFTER:
            model = self.tv.get_model()
            iter = model.get_iter(path)
            v = model.get_value(iter, 1)
            # Search for the full path of the folder
            fpath = None
            for im in self.imindex:
                c = os.path.basename(os.path.dirname(im))
                if v == c:
                    fpath = os.path.dirname(im)
                    break     
            self.callback("dragdest", fpath, selection.data)
    

class SunAlbum(gtk.VBox):
    def __init__(self, imindex, dindex, win, config, tagindex=None, col=None, callback=None, thumbsize=148):
        gtk.VBox.__init__(self)
        self.title = col
        self.album = col
        self.imindex = imindex
        self.dindex = dindex
        self.box_selection = []
        self.box_load = []
        self.modifier = None
        self.win = win
        self.callback = callback
        self.viewtype = 0
        self.tagindex = tagindex
        self.config = config
        self.progress_icon = "stock_folder"
        self.dragging = False
        self.thumbsize = thumbsize + 12

        # Set up title, date, location field
        hbox_h = gtk.HBox()
        vbox_h = gtk.VBox()
        vbox_h.set_spacing(2)
        vbox_h.set_border_width(2)
        self.label_title = gtk.Label()
        hbox_title = gtk.HBox()
        hbox_title.pack_end(self.label_title, fill=False, expand=False)
        #pixbuf = utils.load_icon("folder",128)
        self.browserim = gtk.Image()
        gim = self.browserim
        hbox_title.pack_start(gim,fill=False,expand=False)
        self.label_year = gtk.Label("")
        for l in [hbox_title, self.label_year]:
            align = gtk.Alignment(xalign=0)
            align.add(l)
            vbox_h.pack_start(align, fill=False, expand=False)
        hbox_h.pack_start(vbox_h, True, True)
        
        # Here we can put stuff later, like buttons
        self.titlespace = gtk.VBox()
        hbox_h.pack_start(self.titlespace, True, True)


        # Set up scrollwindow + table
        scrollwin = gtk.ScrolledWindow()
        scrollwin.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrollwin.show()
        self.scrollwin = scrollwin

        table = self.make_table()

        box = gtk.EventBox()
        color = gtk.gdk.color_parse("white")
        box.modify_bg(gtk.STATE_NORMAL, color)
        box.add(table)
        scrollwin.add_with_viewport(box)
        self.table = table

        # file label
        self.label_file = gtk.Label()
        self.label_file.set_ellipsize(pango.ELLIPSIZE_MIDDLE)
        align_file = gtk.Alignment(xalign=0.0, yalign=0.0)
        align_file.add(self.label_file)

        self.pack_start(hbox_h, expand=False, fill=False)
        self.pack_start(scrollwin)
        self.pack_end(align_file, expand=False, fill=False)

        self.connect_events()

    def make_table(self):
        table = gtk.Table()
        table.set_property("column-spacing", 1)
        table.set_property("row-spacing", 1)

        table.show()
        return table

    def connect_events(self):
        self.connect("size-allocate", self.checksize_cb)
        #gobject.timeout_add(2000, self.checksize_cb)
        self.win.connect("key-press-event", self.set_modifier)
        self.win.connect("key-release-event", self.unset_modifier)

    def update(self, col, vt=0):
        self.title = col
        self.album = col
        self.viewtype = vt        

        # Set up (new) title, location and year
        self.label_title.set_markup("<span size='20000'>%s</span>" % self.title.replace("&","&amp;"))
        self.label_year.set_markup("<i>%s</i>" % self.get_date())

        # Load (new) thumbnails
        self.load_thumbs()

    def load_thumbs(self):  
        col = self.album
        # Counting the size
        self.width = self.scrollwin.get_allocation().width
        c = int(self.width/self.thumbsize)

        # clear table
        box_selection = []
        for b in self.box_selection:
            try: 
                box_selection.append(b.get_children()[0].get_name())
            except: continue
        self.unselect_box_all()
        for box in self.box_load:
            for child in box.get_children()[0].get_children()[0]:
                child.destroy()
            box.destroy()
        self.box_load = []

        # loading thumbs into table  
        self.cnt_col = 0
        self.cnt_row = 0      
        lfiles = []
        
        for im in self.dindex:
            if self.viewtype == 0:
                colf = im.split(os.sep)[-2]
                if colf == col:
                    lfiles.append((self.dindex[im],im))
            else:
                if col in self.tagindex[im]:
                    lfiles.append((self.dindex[im],im))
        if self.config["browser_sort_date"]:
            lfiles.sort()
        else:
            lfiles.sort(cmp=lambda x,y: cmp(x[1],y[1]))         

        # Load the first 25 locking with progress bar, the rest when we are idle.. this method is convenient and very fast 

        pr = utils.SunProgress(_("Loading album"), icon=self.progress_icon)
        pr.start(len(lfiles[:25]))
            
        for date, im in lfiles[:25]:
            self.add_thumb(date, im, box_selection, c)
            pr.plus()
        pr.destroy()


        for date, im in lfiles[25:]:
            
            while gtk.events_pending(): 
		        gtk.main_iteration()
            if self.album != col: break
            self.add_thumb(date, im, box_selection, c)
        self.callback("album-changed")

    def add_thumb(self, date, im, box_selection, c):
        if im.endswith("intiplaceholder.png"):
            return
    
        color = self.get_style().base[gtk.STATE_NORMAL]
        
        # Create gtk.Image from thumb
        thumb_file = self.imindex[im]
        thumb = gtk.Image()
        #thumb.set_size_request(163, 126)
        # Load pixbuf when exposing, faster method
        thumb.connect("expose-event", self.thumb_expose, im)
        #thumb.set_from_pixbuf(gtk.gdk.pixbuf_new_from_file(thumb_file))
        thumb.show()
 
        thumbbox = gtk.EventBox()
        thumbbox.add(thumb)
        thumbbox.modify_bg(gtk.STATE_NORMAL, color)
        thumbbox.show()
        thumbbox.connect("drag_data_get", self.drag_thumb, im)
        thumbbox.connect("drag_begin", self.drag_begin)
        thumbbox.connect("drag_end", self.drag_end)
        thumbbox.drag_source_set(gtk.gdk.BUTTON1_MASK, [("text/plain", 0, 80)], gtk.gdk.ACTION_COPY)
        

        
        # Date label             
        labelbox = gtk.EventBox()
        label = gtk.Label()
        label.set_markup("<i>%s</i>" % self.dindex[im].strftime("%d %b %Y %H:%M"))
        if self.config["browser_show_date"]:
            label.show()
        else:
            label.hide()
        labelbox.add(label)
        labelbox.show()
        labelbox.modify_bg(gtk.STATE_NORMAL, color)

        # Filename label
        labelbox2 = gtk.EventBox()
        label = gtk.Label()
        label.set_ellipsize(pango.ELLIPSIZE_MIDDLE)
        label.set_markup("<b>%s</b>" % os.path.basename(os.path.splitext(im)[0]).replace("&", "&amp;"))
        if self.config["browser_show_filename"]:
            label.show()
        else:
            label.hide()
        labelbox2.add(label)
        labelbox2.show()
        labelbox2.modify_bg(gtk.STATE_NORMAL, color)


        # vbox for thumb and label
        vbox = gtk.VBox()
        vbox.pack_start(thumbbox, fill=False, expand=False)
        vbox.pack_start(labelbox, fill=False, expand=False)
        vbox.pack_start(labelbox2, fill=False, expand=False)
        vbox.show()

        # Create eventbox
        box1 = gtk.EventBox()
        box1.add(vbox)
        box1.connect("button-release-event", self.select_boxes)
        box1.connect("button-press-event", self.select_boxes)
        box1.set_name(im)
        box1.set_border_width(2)
        box1.show()
        
        
        
        # Create border eventbox
        box = gtk.EventBox()
        #box.set_size_request(165, 128)
        #box.connect("expose-event", self.box_expose, im)
        
        box.modify_bg(gtk.STATE_NORMAL, color)
        box.add(box1)
        box.show()
        self.box_load.append(box)

        # fix selection
        if im in box_selection:
            self.select_box(box)
        cnt_row = self.cnt_row
        cnt_col = self.cnt_col
        self.table.attach(box, cnt_row, cnt_row+1, cnt_col, cnt_col+1, xoptions=gtk.SHRINK, yoptions=gtk.SHRINK)

        self.cnt_row += 1
        if self.cnt_row == c:
            self.cnt_row = 0
            self.cnt_col += 1
        
        
    def add_thumb_simple(self, f):
        # This function can be used for adding new files
        # It simplifies add_thumb, to add thumb even when lacking information 
        
        # Counting the size
        self.width = self.scrollwin.get_allocation().width
        c = int(self.width/self.thumbsize)

        date = self.dindex[f]
        self.add_thumb(date, f, [], c)
        
    def remove_thumb_simple(self, f):
        # Same as add_thumb_simple, but remove
        box_load = []
        for box in self.box_load:
            if box.get_children()[0].get_name() == f:
                box.destroy()
            else:
                box_load.append(box)
        self.box_load = box_load
  

    def sort_thumbs(self):  
        col = self.album
        # Counting the size
        self.width = self.scrollwin.get_allocation().width
        c = int(self.width/self.thumbsize)

        # clear table
        for box in self.box_load:
            self.table.remove(box)

        # loading thumbs into table  
        cnt_col = 0
        cnt_row = 0      
        lfiles = []
        
        for im in self.dindex:
            if self.viewtype == 0:
                colf = im.split(os.sep)[-2]
                if colf == col:
                    lfiles.append((self.dindex[im],im))
            else:
                if col in self.tagindex[im]:
                    lfiles.append((self.dindex[im],im))
        lfiles.sort()          
        box_load = [] 
        for date, im in lfiles:
            box = None
            for box0 in self.box_load:
                box1 = box0.get_children()[0]
                if box1.get_name() == im:
                    box = box0
                    break
            if not box: return
            self.table.attach(box, cnt_row, cnt_row+1, cnt_col, cnt_col+1, xoptions=gtk.SHRINK, yoptions=gtk.SHRINK)
            box_load.append(box)
            cnt_row += 1
            if cnt_row == c:
                cnt_row = 0
                cnt_col += 1
        self.box_load = box_load
       
    def thumb_expose(self, thumb, event, im):
        if not thumb.get_name() == "exposed":
            thumb_file = self.imindex[im]
            if not os.path.exists(thumb_file):
                pixbuf = utils.load_icon("intipunku")
                pixbuf = pixbuf.scale_simple(148,148,gtk.gdk.INTERP_BILINEAR)
            else:    
                pixbuf = gtk.gdk.pixbuf_new_from_file(thumb_file)
            thumb.set_from_pixbuf(pixbuf)
            thumb.set_name("exposed")
            thumb.set_size_request(pixbuf.get_width(),pixbuf.get_height())

    def checksize_cb(self, *args):
        if self.album != None:

            if self.scrollwin.get_allocation().width != self.width:
                self.sort_thumbs()
        return True

    def select_boxes(self, box1, event):
        box = box1.get_parent()
       
        if MAC and self.modifier == "leftb" and event.button == 1:
            event.button = 3

        if event.button == 1 and event.type == gtk.gdk.BUTTON_RELEASE:  
            if self.dragging:
                return       
            self.select_boxes_b1(box, self.modifier)

        if event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS:
            self.callback("edit", box1.get_name())

        if event.button == 2:
            self.select_box_all()
            self.callback("slideshow")
        
        if event.button == 3:
            if not box in self.box_selection:
                self.unselect_box_all()
                self.select_box(box)  
                self.start_box = box
            self.callback("button3", event.time, self.unset_modifier)
   
    def select_boxes_b1(self, box, modifier=None):
        if modifier == None:
            self.unselect_box_all()
            self.select_box(box)  
            self.start_box = box
    
        if modifier == "ctrl":
            if box in self.box_selection:
                self.unselect_box(box)
            else:
                self.select_box(box)
                self.start_box = box  

        if modifier == "shift":
            box_first = self.start_box
            box_last = box
            self.unselect_box_all()
            pos1 = self.box_load.index(box_first)
            pos2 = self.box_load.index(box_last)
            if pos1 < pos2:
                load = self.box_load[pos1:pos2+1]
            elif pos1 > pos2:
                load = self.box_load[pos2:pos1+1]
            for b in load:
                self.select_box(b)

        # update file label
        w = self.get_allocation().width
        h = self.label_file.get_allocation().height
        self.label_file.set_size_request(w,h)
        n = len(self.box_selection)
        if n > 1:
            self.label_file.set_markup("<b> %s %s </b>" % (n, _("photos selected")))
        else:
            self.label_file.set_markup("<b> %s %s </b>" % (n, _("photo selected")))


    def select_box(self, box):
        self.select_ebox(box, True)
        self.box_selection.append(box)
        
        self.callback("phototray", "add", self.imindex[box.get_children()[0].get_name()])

    def unselect_box(self, box):
        self.select_ebox(box, False)
        self.box_selection.remove(box)       
        self.callback("phototray", "remove", self.imindex[box.get_children()[0].get_name()])

    def unselect_box_all(self):
        for box in self.box_selection:
            self.select_ebox(box, False)
        self.box_selection = []
        self.callback("phototray", "clear")

    def select_box_all(self):
        self.unselect_box_all()
        for box in self.box_load:
            self.select_box(box)

    def set_modifier(self, widget, event):
        # In OSX the HIG is different, shift doesn't not exist
        if MAC:
            key = event.keyval
            if key in [65507,65508]:
                self.modifier = "leftb" # Ctrl key emulates 
            elif key in [65513,65511,65505,65512,65506]:
                self.modifier = "ctrl"
        else:
            if event.keyval == 65508 or event.keyval == 65507:
                self.modifier = "ctrl"
    
            if event.keyval == 65505 or event.keyval == 65506:
                self.modifier = "shift"

    
    def unset_modifier(self, widget, event):
        self.modifier = None


    def update_thumb(self, f):
        for box in self.box_load:
            box1 = box.get_children()[0]
            if box1.get_name() == f:
                thumb_file = self.imindex[f]
                gim = box1.get_children()[0].get_children()[0].get_children()[0]
                gim.set_from_pixbuf(gtk.gdk.pixbuf_new_from_file(thumb_file))

    def update_date(self, f):
        for box in self.box_load:
            box1 = box.get_children()[0]
            if box1.get_name() == f:
                thumb_file = self.imindex[f]
                label = box1.get_children()[0].get_children()[1].get_children()[0]
                label.set_markup("<i>%s</i>" % self.dindex[f].strftime("%d %b %Y %H:%M"))

    def get_date(self):
        dates = []
        for im in self.imindex:
            c = os.path.basename(os.path.dirname(im))
            if c == self.album:
                dates.append(self.dindex[im])
        dates.sort()
        try:
            s = dates[0].strftime("%a, %d %b %Y" ).capitalize() + " - " + dates[-1].strftime("%a, %d %b %Y").capitalize()
        except: s = ""
        return s
        

    #This function is responsible for the graphical representation of the selection
    def select_ebox(self, box, flag):
        if flag: color = self.get_style().base[gtk.STATE_SELECTED]
        else: color = box.get_style().base[gtk.STATE_NORMAL]

        if MAC:
            box1 = box.get_children()[0]
            gim = box1.get_children()[0].get_children()[0].get_children()[0]
            f = box1.get_name()
            if flag:
                pixbuf = tools.pixbuf_draw_rect(self.imindex[f],color.to_string())
            else:
                pixbuf = gtk.gdk.pixbuf_new_from_file(self.imindex[f])
            gim.set_from_pixbuf(pixbuf)
        else:
            box.modify_bg(gtk.STATE_NORMAL, color)
     
    def drag_begin(self, widget, context):
        self.dragging = True
        
    def drag_end(self, widget, context):
        self.dragging = False
            
    def drag_thumb(self, widget, context, selection, targetType, eventTime, im):
         
        selection.set(selection.target, 8, "file://"+urllib.quote(im))
    
class SunMiniBrowser(gtk.HBox):
    def __init__(self, imindex, callback, i_method=gtk.gdk.INTERP_BILINEAR):
        gtk.HBox.__init__(self)
        self.callback = callback
        self.i_method = i_method

        ebox = gtk.EventBox()
        ebox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("black"))
        ebox2 = gtk.EventBox()
        ebox2.set_border_width(1)
        ebox.add(ebox2)
    

        self.hbox = gtk.HBox()
        
        ebox2.add(self.hbox)
        button_back = gtk.Button(stock=gtk.STOCK_GO_BACK)
        button_for = gtk.Button(stock=gtk.STOCK_GO_FORWARD)
        button_back.connect("clicked", self.move, -1)
        button_for.connect("clicked", self.move, 1)
        self.pack_start(button_back, fill=False, expand=False)
        self.pack_start(ebox, fill=False, expand=False)
        self.pack_start(button_for, fill=False, expand=False)
        self.imindex = imindex
        self.im_select = None

        
    def init(self, editor=None, browser=None):
        if editor:
            editor.connect("image-changed", self.image_changed_cb)
        if browser:
            browser.connect("album-changed", self.album_changed_cb)
        self.show_all()

    def album_changed_cb(self, widget, box_load):
        self.im_load = []
        for box in box_load:
            box1 = box.get_children()[0]
            self.im_load.append(box1.get_name())
        if self.im_select: self.image_changed_cb(None, self.im_select)
        

    def image_changed_cb(self, widget, im):
        self.im_select = im
        try:
            n = self.im_load.index(im)
        except: return
        n1 = n - 2
        n2 = n + 3
        load_before = []
        if n1 < 0:
            for i in range(n1*-1):
                load_before.append(None)
            n1 = 0
        load_after = []
        if n2 > len(self.im_load)-1:
            for i in range(n2-len(self.im_load)):
                load_after.append(None)
            n2 = len(self.im_load)
        #print self.im
        load_full = load_before + self.im_load[n1:n2] + load_after
        gobject.idle_add(self.load, load_full)

    def load(self, ims):
        for box in self.hbox.get_children():
            box.destroy()
        #im1, im2, im3, im4, im5 = thumbs
        for im in ims:
            box = gtk.EventBox()
            color1 = self.get_style().base[gtk.STATE_SELECTED]
            color2 = gtk.gdk.color_parse("white")
            if self.im_select == im:
                color = color1
            else: color = color2
            box.modify_bg(gtk.STATE_NORMAL, color)
            box1 = gtk.EventBox()
            box1.modify_bg(gtk.STATE_NORMAL, color2)
            box1.set_border_width(5)
            box.add(box1)
            box1.set_size_request(50,50)
            if im:   
                if MAC and self.im_select == im:
                    pixbuf = tools.pixbuf_draw_rect(self.imindex[im],color=color1.to_string(),border=20)
                else:
                    pixbuf = gtk.gdk.pixbuf_new_from_file(self.imindex[im])
                pixbuf = pixbuf.scale_simple(50, 50, self.i_method)
                gim = gtk.image_new_from_pixbuf(pixbuf)
                gim.show()
                box1.add(gim)
                box.set_name(im)
                box.connect("button-press-event", self.box_clicked, im)
            self.hbox.pack_start(box, fill=False, expand=False)
        self.show_all()

    def box_clicked(self, widget, event, im):
        if event.button == 1:
            self.callback("savedraw")
            self.callback("edit", im)

    def move(self, widget, d):
        n = self.im_load.index(self.im_select)
        if n+d >= 0 and n+d < len(self.im_load):
            im = self.im_load[n+d]
            self.callback("savedraw")
            self.callback("edit", im)

    def update_thumb(self, im):
        for box in self.hbox.get_children():
            name = box.get_name()
            if name == im:
                box1 = box.get_children()[0]
                gim = box1.get_children()[0]
                pixbuf = gtk.gdk.pixbuf_new_from_file(self.imindex[name])
                pixbuf = pixbuf.scale_simple(50, 50, self.i_method)
                #gim = gtk.image_new_from_pixbuf(pixbuf)
                gim.set_from_pixbuf(pixbuf)
                
                   
        


def cb(*args):
    print args

if __name__ == "__main__":
    sb = SunBrowser(cb)
    win = gtk.Window()
    win.add(sb)
    win.show()
    
    gtk.main()
