#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   This file is part of IntiPunku
#   Copyright (C) 2007 Mark Baas <mark.baas123@gmail.com>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os, os.path
from distutils import dir_util
import dialogs
import inti.intigtk as gtk
import gobject
from inti.constants import *
from inti import intiimport
from inti import utils

# Localization
import gettext, locale
_ = gettext.gettext


HOME = os.path.expanduser("~")

class SunConfig(dict):
    def __init__(self, confdir):
        self.confdir = confdir
        self.cfile = self.confdir + os.sep + "settings.conf"
        self["photo_folder"] = IMGDIR
        self["web_exports"] = ""
        self["version"] = "0.1"
        self["import_original"] = False
        self["import_delete"] = False
        self["import_subfolders"] = False
        self["import_convert"] = True
        self["browser_show_date"] = True
        self["browser_show_filename"] = True
        self["first_time"] = True
        self["slide_auto"] = True
        self["slide_timeout"] = 10
        self["slide_hide"] = True
        self["default_import"] = False
        self["default_viewer"] = False
        self["print_filename"] = True
        self["print_date"] = True
        self["google_token"] = 0
        self["import_raw"] = False
        self["raw_format"] = "tiff"
        self["notify_updates"] = True
        self["email"] = ""
        self["albums_sort_date"] = True
        self["browser_sort_date"] = True
        self["scan_res"] = 250
        self["scan_mode"] = "Color"
        self["scan_brightness"] = 0
        self["scan_contrast"] = 0
        self["editor_lossless"] = False
        self["editor_autosave"] = False
                          
        self.integers = ["version", "slide_timeout", "picasa_token", "scan_res", "scan_brightness", "scan_contrast"]

        if not os.path.exists(self.confdir):
            dir_util.mkpath(self.confdir)

        if os.path.exists(self.cfile):
            self.load()
        else:
            self.save()         

    def init(self):

        if not os.path.exists(self["photo_folder"]):
            dialog = dialogs.IntiSettings(self)
            r = dialog.run()
            if r == gtk.RESPONSE_ACCEPT:
                dialog.save_settings()
            dialog.destroy()


    def load(self):
        f = open(self.cfile)
        lines = f.readlines()
        for l in lines:
            if not "=" in l:
                print "Error in line %s of config %s" % (lines.index(l), self.cfile)
            else:
                a = l.strip().split("=")
                if a[1] == "False":
                    b = False
                elif a[1] == "True":
                    b = True
                elif a[1] == "None":
                    b = None
                elif a[0] in self.integers:
                    b = float(a[1])
                else:
                    b = a[1]
                self[a[0]] = b
        

    def save(self):
        config = ""
        l = list(self)
        l.sort()
        for setting in l:
            config += "%s=%s\n" % (setting, self[setting])
        f = open(self.cfile, "w")    
        f.write(config)
        f.close()
             
        return True


    def create_firsttime_page(self, title):
        vbox = gtk.VBox()
        vbox.pack_start(gtk.Label(title), fill=False, expand=False)

        if title == _("Introduction"):
            text = first_intro
            
        else:
            text = ""
        label = gtk.Label()
        label.set_markup(text)
        label.set_line_wrap(True)
        label.set_size_request(300,300)
        vbox.pack_start(label)
        return vbox


class DialogFirst(gtk.Assistant):
    def __init__(self, conf):
        gtk.Assistant.__init__(self)
        
        self.loop = None
        self.response = gtk.RESPONSE_CANCEL

        self.set_size_request(500,500)
        self.set_title(_("IntiPunku Startup Wizard"))     
        self.set_property("skip-taskbar-hint", False)
        self.set_icon_from_file(DATADIR + os.sep + "logo-48x48.png")
        self.set_border_width(10)
        self.set_position(gtk.WIN_POS_CENTER)
        
        self.connect("cancel", self.cb_cancel)
        self.connect("apply", self.cb_apply)
        self.connect("delete-event", self.cb_cancel)
        self.connect("prepare", self.cb_prepare)
        self.setting_page = dialogs.PageSettings(conf)
        
        pages = [(_("Introduction"),first_intro,None, gtk.ASSISTANT_PAGE_INTRO),
                 (_("Configure Intipunku"), first_settings, self.setting_page, gtk.ASSISTANT_PAGE_CONTENT),
                 (_("Import photos"), first_import, self.vbox_import(), gtk.ASSISTANT_PAGE_CONFIRM),
                ]

        for page,text,vbox2,t in pages:
            vbox = gtk.VBox()
            vbox.set_border_width(10)
            
            label = gtk.Label()
            label.set_markup(text)
            label.set_line_wrap(True)
            label.set_size_request(400,10)
            vbox.pack_start(label)

            if not vbox2 == None:
                vbox.pack_start(vbox2, fill=False, expand=False)

            self.append_page(vbox)
            self.set_page_title(vbox, page)
            self.set_page_type(vbox, t)
            self.set_page_complete(vbox, True)
            self.set_page_header_image(vbox, utils.load_icon("intipunku", size=48))
        
        

    def vbox_import(self):
        vbox = gtk.VBox()
        r = 0
        for page, label in page_labels:
            im = gtk.image_new_from_pixbuf(utils.load_icon(page, size=22))
            button = gtk.Button(label=_("Import from") + " " + label)
            button.set_image(im)
            button.connect("clicked", self.import_cb, r)
            r += 1
            vbox.pack_start(button)
        return vbox

    def import_cb(self, widget, r):
        self.albumdir = self.setting_page.config["photo_folder"]
        albums = []
        for album in os.listdir(self.albumdir):      
            if os.path.isdir(self.albumdir + os.sep + album):
                albums.append(album)

        import_t = intiimport.pages[page_labels[r][0]](self, albums, self.setting_page.config, callback=self.handle_import)
        dialog = dialogs.ImportDialog()
        dialog.set_page(import_t)
        dialog.set_transient_for(self)
        r = dialog.run()
        if r == gtk.RESPONSE_OK:
            import_t.doImport()
        dialog.destroy()
        page = self.get_nth_page(2)
        self.set_page_complete(page, self.valid_albumdir(self.setting_page.config["photo_folder"]))

    def handle_import(self, name, *args):
        pass

    def run(self):
        self.show_all()
        self.loop = gobject.MainLoop()
        self.loop.run()
        return self.response
        
    def cb_cancel(self, widget, *args):
        self.loop.quit()
       
        
    def cb_apply(self, widget):
        self.setting_page.config["first_time"] = False
        self.response = gtk.RESPONSE_APPLY
        self.loop.quit()

    def cb_prepare(self, a, page):
        label = page.get_children()[0]
        
        # Change label on page 2 to make him import stuff if albumdir has no photos
        if self.get_nth_page(2) == page:
            # Save settings from page 1
            self.setting_page.save_settings()

            if self.valid_albumdir(self.setting_page.config["photo_folder"]):
                label.set_text(first_import1)
            else:
                label.set_text(first_import2)
                self.set_page_complete(page, False)
                
    def valid_albumdir(self, d):
        albums = os.listdir(d)
        photos = []
        for album in albums:   
            if os.path.isdir(d + os.sep +album):  
                
                for f in os.listdir(d + os.sep + album):
                    ext = os.path.splitext(f)[1].lower()
                    if ext in EXT:
                        photos.append(f)
        if len(photos) > 0:
            return True
        else:
            return False
                    
