#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   This file is part of IntiPunku
#   Copyright (C) 2007 Mark Baas <mark.baas123@gmail.com>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os,sys, os.path

WIN = False
MAC = False
KDE = False

if sys.platform.startswith("win"):
    WIN = True
elif sys.platform.startswith("darwin"):
    MAC = True
if "plasma " in os.popen("ps aux").read():
    KDE = True
    
if WIN:
    import ctypes
    from win32com.shell import shell, shellcon
    
VERSION = 0.52 # 0.5 beta 4
    

# Localization
import gettext
_ = gettext.gettext

# Universal conf path
if WIN:
    CONFDIR = os.environ["APPDATA"] + os.sep + "Intipunku"
    IMGDIR = shell.SHGetPathFromIDList(shell.SHGetFolderLocation (0,shellcon.CSIDL_MYPICTURES, 0, 0))
elif MAC:
    CONFDIR = os.path.expanduser("~") + os.sep + ".config/intipunku"
    IMGDIR = os.path.expanduser("~") + os.sep + "Pictures"
else:
    CONFDIR = os.path.expanduser("~") + os.sep + ".config/intipunku"
    IMGDIR = os.path.expanduser("~") + os.sep + _("Images")

# Supported extensions, gif and png work almost, however it is unstable
EXT = ".jpg .tiff .png .bmp .gif".split()
# Supported for tag reading/writing
EXT_EXIV2 = ".jpg .tiff .jp2 .png".split()
# EXT_PIL unsupported all that can be opened by PIL, but exiv2 cannot write
EXT_PIL = ".bmp .gif .ico .png .ppm .tiff .xpm .psd".split()
# Movie Decs
EXT_VID = ".avi .mov".split()
def load_video():
    try: 
        from inti import vlcwidget
        for t in EXT_VID:
            if not t in EXT: EXT.append(t)
        return True
    except:
        return False
VIDEO_ENABLED = load_video()
try:del gst
except: pass
try: del pygst
except: pass

    
# EXT_PIL unsupported all that can be opened by dcraw but exiv2 cannot write
EXT_RAW = ".cr2 .crw .nef .raw .raf .x3f .pef .dng .kdc .arw .mrw .orf .erf .3fr".split()
RAW_IMPORT = ["tiff", "jpg"]


# Essential directories
if os.environ.has_key("INTIDATA"):    
    DATA = os.environ["INTIDATA"]
else:
    DATA = ""
DATADIR = DATA
ICONDIR = DATA + os.sep + "icons"
PLUGINS_SEARCH_PATH = [DATA + os.sep + "plugins", CONFDIR + os.sep + "plugins"]

# Import stuff
if sys.platform.startswith("win32"): # Camera under windows is not possible
    page_labels = (("picasaweb", _("PicasaWeb Album")), ("folder", _("folder")))
else:
    page_labels = (("camera", _("camera")), ("picasaweb", _("PicasaWeb Album")), ("folder", _("folder")), ("scanner", _("scanner")))
RESPONSE_CAMERA = 10
RESPONSE_FOLDER = 11
RESPONSE_PICASA = 12
RESPONSE_SCAN = 14

# Skip folders
SKIPDIRS = [".*", "Originals", "Archived"]
SKIPFILES = [".*"]

# Facebook API
FB_API_KEY = "da8f3acf4b75247cafdf93a1e6bcd160"
FB_API_SECRET = "9ebe9afe8eff1bb69291938335e7a7c8"

# Export
export_labels = (("picasaweb",_("PicasaWeb Album")),("flickr", _("Flickr Web Album")), ("html",_("HTML album")),("slideswf",_("Flash slideshow")),("folder", _("Folder")))


# Print layout
PRINT_LAYOUT = ((1,3),
                (2,3),
                (3,4),
                (4,6),
               )


# First time wizard text
first_intro = _("""
Welcome and thank you for installing IntiPunku.
Before proceding it is important that you know something about the program and its funcionality.

Inti Puncu is the name of the sun gate of the ancient civilization of Tiahuanacu in Bolivia.
IntiPunku will try to shed more sunshine over your photos!

Upon start up the program will scan your (to be) asigned photo folder. All folders in the photo folder will seen as albums and all photos inside (excluding subfolders) as its content. 
After scanning you can can browse photos in the library view, you can view and edit photos by switching to the editor. Alternatively, you can use the slideshow to view your photos full screen.
""")

first_settings = _("""
The photo folder field is required. By default it is set to %s.

Be free to configure anything else, it can all be changed later.
""" % (IMGDIR))

first_import1 = _("""
All is ready. If you wish to import any photos in addition to the ones in the photo folder, click one of the buttons below.

The indexing process can take some time, so your files will not all directly show up.
""")
first_import2 = _("""
Your photo folder seems to be empty. In order to proceed you have to import photos. Click the one of the buttons below.

The indexing process can take some time, so your files will not all directly show up.
""")
first_import = first_import1

# Languages for manual language change
langs = {
_("English"): "C",
_("Spanish"): "es",
_("Dutch"): "nl",
_("Italiano") : "it",
_("Catalonian") : "ca",
_("Russian") : "ru",
_("French"): "fr",
}
c = {}
c["langs"] = langs

# Win specific stuff
if WIN:
    VSEP = ";"
    EXEXT = ".exe"
else: 
    VSEP = ":"
    EXEXT = ""

# Flickr api
FLICKR_API_KEY = "83156abb11589626c6af0e673073e614"
FLICKR_API_SECRET = "8b5a324bb2e68955"

# Default icon sizes per system
if MAC:
    SYS_ICONSIZE=18
else:
    SYS_ICONSIZE=22
