#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   This file is part of IntiPunku
#   Copyright (C) 2007 Mark Baas <mark.baas123@gmail.com>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import inti.intigtk as gtk
import gettext, gobject, os
from datetime import datetime
import time
_ = gettext.gettext
from threading import Thread
import utils
from constants import *
from inti import optional

class SunDialog(gtk.Dialog):
    def __init__(self):
        gtk.Dialog.__init__(self)
        self.set_destroy_with_parent(True)
        self.set_modal(True)
        self.set_property("skip-taskbar-hint", True)
        self.set_position(gtk.WIN_POS_CENTER_ALWAYS)
        self.set_icon_from_file(DATADIR + os.sep + "logo-48x48.png")
        self.set_title("Intipunku")

    def set_icon_name(self, name):
        self.set_icon(utils.load_icon(name))

class SunWindow(gtk.Dialog):
    def __init__(self):
        gtk.Dialog.__init__(self)
        self.set_destroy_with_parent(True)
        self.set_modal(True)
        self.set_property("skip-taskbar-hint", True)
        self.set_position(gtk.WIN_POS_CENTER_ALWAYS)
        self.set_icon_from_file(DATADIR + os.sep + "logo-48x48.png")



class PageSettings(gtk.VBox):
    def __init__(self, conf):
        gtk.VBox.__init__(self)
    
        self.config = conf

        self.set_spacing(5)
        notebook = gtk.Notebook()
        self.pack_start(notebook)

        # Frame1
        #frame = gtk.Frame(_("General"))
        vbox_gen = gtk.VBox()
        vbox_gen.set_spacing(5)
        vbox_gen.set_border_width(7)
        #frame.add(vbox_gen)
        notebook.append_page(vbox_gen, tab_label=gtk.Label(_("General")))
        #self.pack_start(frame, fill=False, expand=False)

        # Frame2
        #frame = gtk.Frame(_("Library"))
        vbox_lib = gtk.VBox()
        vbox_lib.set_spacing(5)
        vbox_lib.set_border_width(7)
        #frame.add(vbox_lib)
        notebook.append_page(vbox_lib, tab_label=gtk.Label(_("Library")))
        #self.pack_start(frame, fill=False, expand=False) 
        
        # Frame3
        #frame = gtk.Frame(_("Editor"))
        vbox_editor = gtk.VBox()
        vbox_editor.set_spacing(5)
        vbox_editor.set_border_width(7)
        #frame.add(vbox_editor)
        notebook.append_page(vbox_editor, tab_label=gtk.Label(_("Editor")))
        #self.pack_start(frame, fill=False, expand=False) 
        
        # Frame4
        #frame = gtk.Frame(_("Printing"))
        vbox_print = gtk.VBox()
        vbox_print.set_spacing(5)
        vbox_print.set_border_width(7)
        #frame.add(vbox_print)
        notebook.append_page(vbox_print, tab_label=gtk.Label(_("Printing")))
        #self.pack_start(frame, fill=False, expand=False)  

        self.buttons_file = []
        self.buttons_check = []
        self.buttons_spin = []
        self.combos = []
               
        
        # General prefs
        for a,b in [(_("Photo folder"),"photo_folder")]:
            label = gtk.Label(str=a+":")
            entry = gtk.Entry()
            button = gtk.FileChooserButton(_("Select folder"))
            button.set_name(b)
            button.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
            self.buttons_file.append(button)
            hbox = gtk.HBox()
            hbox.set_spacing(5)
            hbox.pack_start(label, fill=False, expand=False)               
            hbox.pack_start(button)        
            vbox_gen.pack_start(hbox, False, False)
        l = [
            #(_("Use IntiPunku Viewer as default image viewer") + " " + _("Gnome/Windows only"), "default_viewer"),
            (_("Use IntiPunku Import as default camera importer")+ " (" + _("Gnome only") + ")", "default_import"),
            (_("Notify when new updates are available"), "notify_updates"),
            ]
        for a,b in l:
            if sys.platform.startswith("win"):
                if b == "default_import":
                    continue
            button = gtk.CheckButton(label=a)
            button.set_name(b)
            vbox_gen.pack_start(button, False, False)
            self.buttons_check.append(button)     
        
        button = gtk.CheckButton(label=_("Automatically import raw photos to")+" ")
        button.set_name("import_raw")
        combo = gtk.combo_box_new_text()
        combo.set_name("raw_format")
        hbox = gtk.HBox()
        hbox.pack_start(button, False, False)
        hbox.pack_end(combo, False, False)
        vbox_gen.pack_start(hbox, False, False)
        self.combos.append(combo)
        self.buttons_check.append(button)

        # Viewer prefs  
        for a,b in [(_("Sort albums on date"),"albums_sort_date"),(_("Sort photos on date"),"browser_sort_date"), (_("Show date label"),"browser_show_date"), (_("Show filename label"), "browser_show_filename")]:
            button = gtk.CheckButton(label=a)
            button.set_name(b)
            vbox_lib.pack_start(button, False, False)
            self.buttons_check.append(button)
                                  
        # Print prefs
        for a,b in [(_("Print filename label"), "print_filename"), (_("Print date label"), "print_date")]:
            button = gtk.CheckButton(label=a)
            button.set_name(b)
            vbox_print.pack_start(button, False, False)
            self.buttons_check.append(button)
        
        # Editor prefs
        for a,b in [(_("Automatically save changes"), "editor_autosave"),(_("Use lossless rotation \n(slower; requires jpegtran, see components)"),"editor_lossless")]:
            button = gtk.CheckButton(label=a)
            button.set_name(b)
            vbox_editor.pack_start(button, False, False)
            self.buttons_check.append(button)            
        
            
        self.load_settings()

    def load_settings(self):
        for button in self.buttons_file:
            if os.path.exists(os.path.abspath(self.config[button.get_name()])):
                button.set_current_folder(os.path.abspath(self.config[button.get_name()]))

        for button in self.buttons_check:
            button.set_active(self.config[button.get_name()])
            
        for button in self.buttons_spin:
            if button.get_name() == "slide_timeout":
                button.set_sensitive(self.config["slide_auto"])
            button.set_value(float(self.config[button.get_name()]))
            
        for combo in self.combos:
            items = RAW_IMPORT
            active = self.config[combo.get_name()]
            for item in items:
                combo.append_text(str(item))
            combo.set_active(list(items).index(active))


    def save_settings(self):
        for button in self.buttons_file:
            self.config[button.get_name()] = os.path.abspath(button.get_filename())
        
        for button in self.buttons_check:
            self.config[button.get_name()] = button.get_active()
            
        for button in self.buttons_spin:
            self.config[button.get_name()] = int(button.get_value())
            
        for combo in self.combos:
            item = combo.get_active_text()
            self.config[combo.get_name()] = item
            
        self.config.save()
        
    def cb_slideauto(self, button):
        for button2 in self.buttons_spin:
            if button2.get_name() == "slide_timeout":
                break
        button2.set_sensitive(button.get_active())

class IntiSettings(SunDialog):
    def __init__(self, conf):
        SunDialog.__init__(self)

        self.vbox.set_border_width(7)
        self.vbox.set_spacing(5)

        self.set_title(_("IntiPunku preferences"))
        self.add_buttons(gtk.STOCK_SAVE, gtk.RESPONSE_ACCEPT, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
        self.set_property("skip-taskbar-hint", True)
        self.set_size_request(500, 500)

        page = PageSettings(conf)
        page.set_border_width(10)
        self.vbox.pack_start(page)
        self.page = page
        self.show_all()

    def load_settings(self):
        self.page.load_settings()

    def save_settings(self):
        self.page.save_settings()



class DateDialog(SunDialog):
    def __init__(self, files, dindex):
        SunDialog.__init__(self)

        self.set_title(_("Change time/date"))
        self.set_icon_name("clock")
        self.add_buttons(gtk.STOCK_SAVE, gtk.RESPONSE_ACCEPT, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)

        date = list(dindex[files[-1]].timetuple())[:6]

        self.spinbuttons1 = [("hour",date[3],(0,23)),("min",date[4], (0,59))]        
        self.spinbuttons2 = [("year",date[0],(1900,3000)),("month",date[1], (1,12)),("day",date[2],(1,31))]
        self.spins = []
        hbox1 = gtk.HBox()
        hbox2 = gtk.HBox()
        hbox1.pack_start(gtk.Label("%s (%s/%s):" % (_("time").capitalize(),_("hour"),_("minutes")))) 
        hbox2.pack_start(gtk.Label("%s (%s/%s/%s):" % (_("date").capitalize(),_("year"),_("month"),_("day")))) 
        for spin,v,ra in self.spinbuttons1:
            mi,ma = ra
            adjustment = gtk.Adjustment(value=v, lower=mi, upper=ma, step_incr=1, page_incr=0, page_size=0)
            sp = gtk.SpinButton()
            sp.set_wrap(True)
            sp.set_adjustment(adjustment)
            sp.set_name(spin)
            self.spins.append(sp)
            hbox1.pack_start(sp, fill=False, expand=False)
        for spin,v,ra in self.spinbuttons2:
            mi,ma = ra
            adjustment = gtk.Adjustment(value=v, lower=mi, upper=ma, step_incr=1, page_incr=0, page_size=0)
            sp = gtk.SpinButton()
            sp.set_wrap(True)
            sp.set_adjustment(adjustment)
            sp.set_name(spin)
            self.spins.append(sp)
            hbox2.pack_start(sp, fill=False, expand=False)
        
        a1 = gtk.Alignment(xalign=0)
        a1.add(hbox1)
        a2 = gtk.Alignment(xalign=0)
        a2.add(hbox2)

        self.vbox.pack_start(a1)
        self.vbox.pack_start(a2)

        if len(files) > 1:
            hbox = gtk.HBox()
            self.vbox.pack_start(hbox)
            gim = gtk.image_new_from_stock(gtk.STOCK_DIALOG_WARNING, gtk.ICON_SIZE_MENU)
            hbox.pack_start(gim)
            hbox.pack_start(gtk.Label(_("The change in date/time will be applied to the other files")))
        else:
            pass

        self.show_all()

    def get_datetime(self):
        d = {}
        for sp in self.spins:
            d[sp.get_name()] = int(sp.get_value())
        return datetime(d["year"],d["month"],d["day"],d["hour"],d["min"])
        
                    
class SunSplash(gtk.Window):
    def __init__(self, title, version):
        gtk.Window.__init__(self)
        self.set_position(gtk.WIN_POS_CENTER_ALWAYS)
        self.set_decorated(False)
        self.set_icon_from_file(DATA + os.sep + "logo-48x48.png")
        self.set_title(_("IntiPunku is loading"))   
        self.set_default_size(300,225)
        self.set_property("skip-taskbar-hint", True)     
        self.set_keep_above(True)
        #self.set_app_paintable(True)
        
        self.show_progress = False

        vbox = gtk.VBox()
        vbox.set_border_width(1)

        self.gim = gtk.Image()
        pixbuf = gtk.gdk.pixbuf_new_from_file(DATA + os.sep + "splash" + str(round(VERSION,1)) + ".png")
        self.gim.set_from_pixbuf(pixbuf)
        
        #self.gim = gtk.Image()
        #self.im = DATADIR + os.sep + "logo.png"
        #self.im_fr = 1
        

        vbox.pack_start(self.gim, expand=True, fill=True)
        
        self.vbox = vbox
        self.add(vbox)
        
                   
       
        self.show_all()
        
        for i in range(2):
            while gtk.events_pending(): 
		        gtk.main_iteration() 
            time.sleep(1)

        
        
    def sunset(self):
        gobject.timeout_add(2000, self.destroy)
                    

class SunProgress(SunDialog):
    def __init__(self, message, cancel=False, icon="intipunku", win=None):
        SunDialog.__init__(self)

        self.cancel = False
      
        if win: self.set_transient_for(win)
        self.set_icon(utils.load_icon(icon,size=48))
        self.set_title(_("IntiPunku is working"))
        self.set_property("skip-taskbar-hint", True)
        
        if cancel:
            self.set_size_request(350,120)
        else:
            self.set_size_request(350,90)
        
        vbox = self.vbox#gtk.VBox()

        label = gtk.Label(str=message)
        
        progress = gtk.ProgressBar()
        self.progress = progress
        self.progress.set_pulse_step(0.05)
        hbox = gtk.HBox()
        hbox.pack_start(gtk.image_new_from_pixbuf(utils.load_icon(icon, size=48)), fill=False, expand=False)
        hbox.pack_start(label)
        vbox.pack_start(hbox, expand=True, fill=True)
        vbox.pack_start(progress, expand=False, fill=False)

        cbutton = gtk.Button(stock=gtk.STOCK_CANCEL)
        cbutton.connect("clicked", self.cancel_cb)

        self.connect("delete-event", self.cancel_cb)

        if cancel:  
            self.add_action_widget(cbutton, 2000)      
            #vbox.pack_start(cbutton, expand=False, fill=False)

        #self.add(vbox)

        self.show_all()   

    def start(self, n, text=""):
        if n == 0: n = 1
        self.step = 0.9999/float(n)
        self.fraction = 0  
        self.progress.set_fraction(self.fraction)
        self.progress.set_text(text)

        while gtk.events_pending(): 
		    gtk.main_iteration() 

    def set_text(self, text):
        self.progress.set_text(text)

    def plus(self):
        self.fraction += self.step
        self.progress.set_fraction(self.fraction)
        while gtk.events_pending(): 
		    gtk.main_iteration() 
		    
    def pulse(self):
        if self.fraction == self.fraction:
            self.progress.pulse()
            while gtk.events_pending(): 
    		      gtk.main_iteration() 
    	    gobject.timeout_add(100, self.pulse)
		

    def cancel_cb(self, widget, *args):
        print "Progress cancelled"
        self.set_text(_("Canceling, please be patient..."))
        self.cancel = True
        self.cancel = True
        gobject.timeout_add(10000, self.destroy)


class PicasaWebDialog(SunDialog):
    def __init__(self, albums, i):
        SunDialog.__init__(self)

        self.set_property("skip-taskbar-hint", True)
        self.set_title(_("Upload to Picasaweb albums"))
        self.set_icon_name("picasaweb")
        self.add_buttons(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK)

        hbox1 = gtk.HBox()
        hbox1.set_spacing(10)
        entry_album = gtk.combo_box_entry_new_text()
        for album in albums:
            entry_album.append_text(album)
        entry_album.set_active(i)
        hbox1.pack_start(gtk.Label(_("Album")+": "), expand=False, fill=False)
        hbox1.pack_start(entry_album)        
        self.entry_album = entry_album

        hbox2 = gtk.HBox()
        hbox2.set_spacing(10)
        entry_size = gtk.combo_box_new_text()
        entries = [_("Original"),"1600x1200","1024x768","800x600","640x480"]
        for entry in entries: 
            entry_size.append_text(entry)
        entry_size.set_active(0)
        hbox2.pack_start(gtk.Label(_("Resolution")+": "), expand=False, fill=False)
        hbox2.pack_start(entry_size)
        self.entry_size = entry_size
        

        self.vbox.pack_start(hbox1)
        self.vbox.pack_start(hbox2)
        self.show_all()

    def get_album(self):
        return (self.entry_album.get_active(), self.entry_album.get_active_text())

    def get_size(self):
        return (self.entry_size.get_active_text())


class ImportDialog(SunDialog):
    def __init__(self):
        SunDialog.__init__(self)       
        self.set_size_request(760,500)
        self.add_buttons(gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
        self.set_title("Intipunku " + _("Import"))

    def set_page(self, page):
        self.vbox.pack_start(page)

        # add the otherstuff

class ChooseImportExportDialog(SunDialog):
    export = None
    def __init__(self):
        SunDialog.__init__(self)
        hbox = gtk.HBox()
        self.vbox.pack_start(hbox)
        self.add_buttons(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
        
        # Import side
        vbox_import = gtk.VBox()
        hbox.pack_start(vbox_import, fill=False, expand=False)
        
        label = gtk.Label()
        label.set_markup("<b>"+_("Import photos")+"</b>")
        vbox_import.pack_start(label, fill=False, expand=False)
        vbox_import.set_border_width(5)
        vbox_import.set_spacing(5)
        r = 10
        for page, label in page_labels:
            im = gtk.image_new_from_pixbuf(utils.load_icon(page, size=22))
            button = gtk.Button(label=_("Import from") + " " + label)
            button.set_image(im)
            button.connect("clicked", self.button_cb, r)
            r += 1
            vbox_import.pack_start(button, fill=False, expand=False)
            
        hbox.pack_start(gtk.VSeparator())
        
        # Export side
        vbox_export = gtk.VBox()
        hbox.pack_start(vbox_export, fill=False, expand=False)
        
        label = gtk.Label()
        label.set_markup("<b>"+_("Export photos")+"</b>")
        vbox_export.pack_start(label, fill=False, expand=False)
        vbox_export.set_border_width(5)
        vbox_export.set_spacing(5)
        for t, label in export_labels:
            im = gtk.image_new_from_pixbuf(utils.load_icon(t, size=22))
            button = gtk.Button(label=_("Export to") + " " + label)
            button.set_image(im)
            button.set_name(t)
            button.connect("clicked", self.button_cb, gtk.RESPONSE_OK)
            vbox_export.pack_start(button, fill=False, expand=False)
            
        self.show_all()

    def button_cb(self, widget, r):
        self.response(r)
        if r == gtk.RESPONSE_OK: self.export = widget
        
    def get_export(self):
        return self.export
        
        

class ChooseImportDialog(SunDialog):
    def __init__(self):
        SunDialog.__init__(self)
        self.set_title(_("Please choose from which source you would like to import photos."))
        self.vbox.pack_start(gtk.Label(_("Please choose from which source you would like to import photos.")))
        self.vbox.set_border_width(5)
        self.vbox.set_spacing(5)
        r = 10
        for page, label in page_labels:
            im = gtk.image_new_from_pixbuf(utils.load_icon(page, size=22))
            button = gtk.Button(label=_("Import from") + " " + label)
            button.set_image(im)
            button.connect("clicked", self.button_cb, r)
            r += 1
            self.vbox.pack_start(button, fill=False, expand=False)
       
        self.add_buttons(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
        self.show_all()

    def button_cb(self, widget, r):
        self.response(r)

class ChooseExportDialog(SunDialog):
    export = None
    def __init__(self, callback):
        SunDialog.__init__(self)
        
        self.vbox.pack_start(gtk.Label(_("Please choose to which destination you would like to export photos.")))
        self.vbox.set_border_width(5)
        self.vbox.set_spacing(5)
        for t, label in export_labels:
            im = gtk.image_new_from_pixbuf(utils.load_icon(t, size=22))
            button = gtk.Button(label=_("Export to") + " " + label)
            button.set_image(im)
            button.set_name(t)
            button.connect("clicked", self.button_cb, gtk.RESPONSE_OK)
            self.vbox.pack_start(button)

        self.add_buttons(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
        self.show_all()

    def button_cb(self, widget, r):
        self.export = widget
        self.response(r)

    def get_export(self):
        return self.export
        
        
class MessageInputDialog(gtk.MessageDialog):
    def __init__(self, markup, entry_text=""):
        gtk.MessageDialog.__init__(self)
        self.add_buttons(gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
        self.set_modal(True)
        self.set_property("message-type", gtk.MESSAGE_INFO)
        self.set_markup(markup)           
        vbox = self.vbox.get_children()[0].get_children()[1]
        self.entry = gtk.Entry()
        self.entry.set_text(entry_text)
        self.entry.select_region(0, -1)
        vbox.pack_start(self.entry, fill=True, expand=False)
        self.show_all()

    def get_text(self):
        return self.entry.get_text()

class LoginDialog(SunDialog):
    def __init__(self, login=None, password=None, message=""):
        SunDialog.__init__(self)
        self.add_buttons(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                 gtk.STOCK_OK, gtk.RESPONSE_OK)
        self.set_title(_("Enter login"))

        self.set_default_response(gtk.RESPONSE_OK)

        self.vbox.pack_start(gtk.Label(message), fill=False, expand=False)

        hbox = gtk.HBox(False, 8)
        hbox.set_border_width(8)
        self.vbox.pack_start(hbox, False, False, 0)

        stock = gtk.image_new_from_stock(gtk.STOCK_DIALOG_AUTHENTICATION,
                                         gtk.ICON_SIZE_DIALOG)
        hbox.pack_start(stock, False, False, 0)

        table = gtk.Table(2, 2)
        table.set_row_spacings(4)
        table.set_col_spacings(4)
        hbox.pack_start(table, True, True, 0)

        label = gtk.Label("_Login")
        label.set_use_underline(True)
        table.attach(label, 0, 1, 0, 1)
        self.login_entry = gtk.Entry()
        self.login_entry.set_activates_default(True)
        if login is not None:
            self.login_entry.set_text(login)
        table.attach(self.login_entry, 1, 2, 0, 1)
        label.set_mnemonic_widget(self.login_entry)

        label = gtk.Label("_Password")
        label.set_use_underline(True)
        table.attach(label, 0, 1, 1, 2)
        self.password_entry = gtk.Entry()
        self.password_entry.set_visibility(False)
        self.password_entry.set_activates_default(True)
        if password is not None:
            self.password_entry.set_text(password)
        table.attach(self.password_entry, 1, 2, 1, 2)
        label.set_mnemonic_widget(self.password_entry)
        self.show_all()

    def get_login(self):
        return self.login_entry.get_text()

    def get_password(self):
        return self.password_entry.get_text()


class HTMLExportDialog(SunDialog):
    def __init__(self, template_path, title):
        SunDialog.__init__(self)

        self.add_buttons(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                 gtk.STOCK_OK, gtk.RESPONSE_OK)

        self.set_size_request(500,500)
        self.set_icon_name("html")
        self.set_title(_("Creating Intipunku HTML album"))
        mvbox = gtk.VBox()
        self.vbox.pack_start(mvbox)
        mvbox.set_border_width(7)
        mvbox.set_spacing(10)

        # Export dir:
        label = gtk.Label(_("Please choose a folder to export the album") + ":")
        button = gtk.FileChooserButton(_("Select folder"))
        button.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
        self.folder = button
        mvbox.pack_start(label,fill=False, expand=False)
        mvbox.pack_start(button,fill=False, expand=False)

        # Title
        hbox = gtk.HBox()
        mvbox.pack_start(hbox, fill=False, expand=False)
        label = gtk.Label(_("Title")+":")
        entry = gtk.Entry()
        self.titlex = entry
        entry.set_text(title)
        hbox.pack_start(label,fill=False, expand=False)
        hbox.pack_start(entry)
        
        # Styles
        self.templates = []
        label2 = gtk.Label(_("Please choose style") + ":")
        button = None
        self.styles = []
        for f in os.listdir(template_path):
            name, ext = os.path.splitext(f)
            if ext == ".html" and os.path.exists(template_path + os.sep + name + "_preview.png"):
                hbox_s = gtk.HBox()
                button = gtk.RadioButton(group=button, label=name)
                gim = gtk.image_new_from_file(template_path + os.sep + name + "_preview.png")
                hbox_s.pack_start(button, fill=False, expand=False)
                hbox_s.pack_end(gim, fill=False, expand=False)
                button.set_name(os.path.basename(name))
                self.styles.append(button)
                mvbox.pack_start(hbox_s, fill=False, expand=False)
        if len(self.styles) == 0:
            print "No styles have been found, quiting"
            self.response(gtk.RESPONSE_CANCEL)

        # Size
        sizes = [(320,240,_("very small")),(640,480,_("small")),(800,600,_("medium")),(1024,768,_("large")),("?","?",_("Original"))]
        entry_size = gtk.combo_box_new_text()
        self.size = entry_size
        for w,h,l in sizes:
            entry = "%sx%s // %s"  % (w,h,l) 
            entry_size.prepend_text(entry)
        entry_size.set_active(3)
        hbox = gtk.HBox()
        mvbox.pack_start(hbox, fill=False, expand=False)
        hbox.pack_start(gtk.Label(_("Resolution") + ":"), fill=False, expand=False)
        hbox.pack_start(entry_size)

        self.show_all()

    def get_data(self):
        title = self.titlex.get_text()
        folder = self.folder.get_filename()
        for b in self.styles:
            if b.get_active():
                style = b.get_name()

        w,h = self.size.get_active_text().split("//")[0].strip().split("x")
        size = (int(w),int(h))
        return title, folder, style, size

class FolderExportDialog(SunDialog):
    def __init__(self, title):
        SunDialog.__init__(self)

        self.add_buttons(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                 gtk.STOCK_OK, gtk.RESPONSE_OK)

        self.set_size_request(500,250)
        self.set_icon_name("folder")
        self.set_title(_("Creating Intipunku HTML album").replace("HTML", ""))
        mvbox = gtk.VBox()
        self.vbox.pack_start(mvbox)
        mvbox.set_border_width(7)
        mvbox.set_spacing(10)

        # Export dir:
        label = gtk.Label(_("Please choose a folder to export the album") + ":")
        button = gtk.FileChooserButton(_("Select folder"))
        button.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
        self.folder = button
        mvbox.pack_start(label,fill=False, expand=False)
        mvbox.pack_start(button,fill=False, expand=False)

        # Title
        hbox = gtk.HBox()
        mvbox.pack_start(hbox, fill=False, expand=False)
        label = gtk.Label(_("Title")+":")
        entry = gtk.Entry()
        self.titlex = entry
        entry.set_text(title)
        hbox.pack_start(label,fill=False, expand=False)
        hbox.pack_start(entry)

        # Size
        sizes = [(320,240,_("very small")),(640,480,_("small")),(800,600,_("medium")),(1024,768,_("large")),("?","?",_("Original"))]
        entry_size = gtk.combo_box_new_text()
        self.size = entry_size
        for w,h,l in sizes:
            entry = "%sx%s // %s"  % (w,h,l) 
            entry_size.prepend_text(entry)
        entry_size.set_active(3)
        hbox = gtk.HBox()
        mvbox.pack_start(hbox, fill=False, expand=False)
        hbox.pack_start(gtk.Label(_("Resolution") + ":"), fill=False, expand=False)
        hbox.pack_start(entry_size)

        self.show_all()

    def get_data(self):
        title = self.titlex.get_text()
        folder = self.folder.get_filename()
        w,h = self.size.get_active_text().split("//")[0].strip().split("x")
        size = (int(w),int(h))
        return title, folder, size

class AboutDialog(gtk.AboutDialog):
    def __init__(self, title, version, authors, translators, license):
        gtk.AboutDialog.__init__(self)

        self.set_icon_from_file(DATADIR + os.sep + "logo-48x48.png")
        self.set_name(title)
        self.set_destroy_with_parent(True)
        self.set_website("http://intipunku.berlios.de")
        self.set_copyright("Mark Baas <mark.baas@gmail.com>")
        self.set_authors([authors])
        self.set_translator_credits(translators)
        self.set_logo(gtk.gdk.pixbuf_new_from_file(DATADIR + os.sep + "logo.png"))
        self.set_version(version)
        f = open(license)
        self.set_license(f.read())
        f.close()
        
if not WIN:    
    import gtkunixprint
    class PrintDialog(gtkunixprint.PrintUnixDialog):
        def __init__(self, title=""):
            gtkunixprint.PrintUnixDialog.__init__(self)
            
            self.set_title("Intipunku " + _("print"))
            self.set_icon_from_file(DATADIR + os.sep + "logo-48x48.png")
            self.set_destroy_with_parent(True)
            self.set_modal(True)
            self.set_property("skip-taskbar-hint", True)
            self.set_position(gtk.WIN_POS_CENTER_ALWAYS)
            
            vbox_i = self.vbox.get_children()[0].get_children()[0].get_nth_page(0)
            vbox_i2 = gtk.VBox()
            vbox_i2.set_spacing(10)
            vbox_i.pack_start(vbox_i2, fill=False, expand=False)
            

            label = gtk.Label()
            label.set_markup("<b>"+_("Intipunku specific settings")+"</b>")
            al = gtk.Alignment()
            al.add(label)
            
            vbox_i2.pack_start(al)

            # Title
            hbox = gtk.HBox()
            hbox.set_spacing(5)
            vbox_i2.pack_start(hbox, fill=False, expand=False)
            hbox.pack_start(gtk.Label(_("Title")+":"), fill=False, expand=False)
            self.entry = gtk.Entry()
            self.entry.set_text(title)
            hbox.pack_start(self.entry)
            
            
            # Photos per page
            hbox = gtk.HBox()
            hbox.set_spacing(5)
            vbox_i2.pack_start(hbox, fill=False, expand=False)
            hbox.pack_start(gtk.Label(_("Photos per page") + ":"), fill=False, expand=False)
            self.combo = gtk.combo_box_new_text()
            hbox.pack_start(self.combo)
            for c,r in PRINT_LAYOUT:
                n = c*r
                s = str(n) + " - " + str(c) + "x" + str(r)
                self.combo.append_text(s)
            self.combo.set_active(2)
                
            self.show_all()
            self.vbox.get_children()[0].get_children()[-1].destroy()
                
        def get_data(self):
            title = self.entry.get_text()
            n = self.combo.get_active()
            return title, n  
                      
elif WIN:         
    class PrintDialog(SunDialog):
        def __init__(self, title=""):
            SunDialog.__init__(self)
            self.add_buttons(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                     gtk.STOCK_OK, gtk.RESPONSE_OK)

            self.set_size_request(400,200)
                     
            # Title
            hbox = gtk.HBox()
            hbox.set_border_width(5)
            hbox.set_spacing(10)
            self.vbox.pack_start(hbox, fill=False, expand=False)
            hbox.pack_start(gtk.Label(_("Title")+":"), fill=False, expand=False)
            self.entry = gtk.Entry()
            self.entry.set_text(title)
            hbox.pack_start(self.entry)
                      
            # Photos per page
            hbox = gtk.HBox()
            hbox.set_border_width(5)
            hbox.set_spacing(10)
            self.vbox.pack_start(hbox, fill=False, expand=False)
            hbox.pack_start(gtk.Label(_("Photos per page") + ":"), fill=False, expand=False)
            self.combo = gtk.combo_box_new_text()
            hbox.pack_start(self.combo)
            for c,r in PRINT_LAYOUT:
                n = c*r
                s = str(n) + " - " + str(c) + "x" + str(r)
                self.combo.append_text(s)
            self.combo.set_active(2)
                
            self.show_all()
                
                
        def get_data(self):
            title = self.entry.get_text()
            n = self.combo.get_active()
            return title, n

        def get_settings(self):
            return None

class FlashExportDialog(SunDialog):
    def __init__(self, album=""):
        SunDialog.__init__(self)
        self.add_buttons(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                 gtk.STOCK_OK, gtk.RESPONSE_OK)

        self.set_size_request(400,200)
        self.set_title(_("Flash slideshow"))
        self.set_icon_name("slideswf")
        
        mvbox = gtk.VBox()
        self.vbox.pack_start(mvbox)
        mvbox.set_border_width(7)
        mvbox.set_spacing(10)
        
        # Slide title
        label = gtk.Label(_("Title of slideshow"))
        entry = gtk.Entry()
        entry.set_text(album)
        self.entry = entry
        mvbox.pack_start(label, fill=False, expand=False)
        mvbox.pack_start(entry, fill=False, expand=False)
        
        # Export dir:
        label = gtk.Label(_("Please choose a folder to save flash slideshow") + ":")
        button = gtk.FileChooserButton(_("Select folder"))
        button.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
        self.folder = button
        mvbox.pack_start(label,fill=False, expand=False)
        mvbox.pack_start(button,fill=False, expand=False)
        
        self.show_all()
        
    def get_output_info(self):
        title = self.entry.get_text()
        folder = self.folder.get_filename()
        return (folder, title)
        
class FlickrDialog(SunDialog):
    def __init__(self, album):
        SunDialog.__init__(self)
        
        self.add_buttons(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                 gtk.STOCK_OK, gtk.RESPONSE_OK)
                 
        self.set_size_request(400,200)
        self.set_title(_("Export to Flickr web album"))
        self.set_icon_name("flickr")
        self.vbox.set_border_width(7)
        self.vbox.set_spacing(10)
        
        self.vbox.pack_start(gtk.Label(_("Album name") + ":"))
        self.entry = gtk.Entry()
        self.entry.set_text(album)
        self.vbox.pack_start(self.entry)
        
        checks = [("public", _("Photos will be visible to anyone")),
                      ("family", _("Photos will be visible to family")),
                      ("friends", _("Photos will be visible to friends")),
                     ]
        self.checkboxes = {}
        
        for check,desc in checks:
            checkb = gtk.CheckButton(label=desc)
            checkb.set_active(True)
            self.checkboxes[check] = checkb
            self.vbox.pack_start(checkb)
            
        self.show_all()
        
    def get_data(self):
        album = self.entry.get_text()
        public = int(self.checkboxes["public"].get_active())
        family = int(self.checkboxes["family"].get_active())
        friends = int(self.checkboxes["friends"].get_active())

        return (album, public, family, friends) 
        
class NotifyProgressWindow(gtk.Window):
    def __init__(self, files):
        gtk.Window.__init__(self)
        self.set_decorated(False)
        self.set_size_request(200,100)
        self.set_property("skip-taskbar-hint", True)
        self.set_keep_above(True)
        
        swidth = gtk.gdk.screen_width()
        sheight = gtk.gdk.screen_height()
        self.move(swidth-200-10,sheight-75-30)
        
        vbox = gtk.VBox()
        vbox.set_border_width(5)
        self.add(vbox)
        
        hbox = gtk.HBox()
        vbox.pack_start(hbox)
        
        self.progress = gtk.ProgressBar()
        vbox.pack_end(self.progress, True, True)
        
        self.image = gtk.Image()
        hbox.pack_start(self.image, False, False)
        
        self.label = gtk.Label()
        hbox.pack_end(self.label, True, True)
       
        self.fraction = 0
        self.n = files
        self.n_items = 0
        
        self.timer = 0
        #gobject.timeout_add(1000, self.active)
       
    def update(self, n, f, msg="", ):
        if n != self.n: # we started over again
            self.n_items = 0
        self.show()
        self.n = n
        self.n_items += 1
        self.timer = 0
        self.fraction = float(self.n_items) / n
        self.progress.set_fraction(self.fraction)
        self.label.set_text(msg)
        self.progress.set_text("%s/%s" % (self.n_items, n))
        pixbuf = gtk.gdk.pixbuf_new_from_file(f)
        pixbuf = pixbuf.scale_simple(80,60, gtk.gdk.INTERP_BILINEAR)
        self.image.set_from_pixbuf(pixbuf) 
        if int(self.fraction*n) == n:
            gobject.timeout_add(1000, self.destroy)
            return None
        return self
        
    def active(self):
        self.timer += 1
        print self.timer
        if self.timer == 4:
            self.hide()
        return True
        
        
class NotifyWindow(gtk.Window):
    def __init__(self, f, msg, win=None):
        gtk.Window.__init__(self)
        self.set_decorated(False)
        if not win == None:
            self.set_transient_for(win)
        self.set_size_request(200,75)
        self.set_property("skip-taskbar-hint", True)
        self.set_keep_above(True)
        
        swidth = gtk.gdk.screen_width()
        sheight = gtk.gdk.screen_height()
        self.move(swidth-200-10,sheight-75-30)
        
        ebox = gtk.EventBox()
        ebox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("#000000"))
        ebox2 = gtk.EventBox()
        ebox2.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("#424242"))
        ebox2.set_border_width(1)
        ebox.add(ebox2)
  
        hbox = gtk.HBox()
        vbox = gtk.VBox()
        hbox.set_border_width(4)
        
        self.label_msg = gtk.Label()
        self.label_msg.set_markup("<span foreground=\"white\"><b>%s</b></span>" % (msg))
        pixbuf = gtk.gdk.pixbuf_new_from_file(f)
        pixbuf = pixbuf.scale_simple(80,60, gtk.gdk.INTERP_BILINEAR)
        self.image = gtk.image_new_from_pixbuf(pixbuf)
        
        hbox.pack_end(self.image, fill=False, expand=False)
        vbox.pack_start(gtk.image_new_from_pixbuf(utils.load_icon("intipunku")))
        vbox.pack_start(self.label_msg)
        hbox.pack_start(vbox)
        
        ebox2.add(hbox)
        self.add(ebox)   
        
        self.timer = 0
        
        gobject.timeout_add(1000, self.timer_add)
    
    def timer_add(self):
        self.timer += 1
        if self.timer > 2: self.destroy()
        else: return True
    
    def update(self, f, msg):
        self.timer = 0
        self.label_msg.set_markup("<span foreground=\"white\"><b>%s</b></span>" % (msg))
        pixbuf = gtk.gdk.pixbuf_new_from_file(f)
        pixbuf = pixbuf.scale_simple(80,60, gtk.gdk.INTERP_BILINEAR)
        self.image.set_from_pixbuf(pixbuf) 
        self.show()
        self.activate_focus()
        
class EmailDialog(SunDialog):
    def __init__(self, email):
        SunDialog.__init__(self)
        
        self.add_buttons(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                 gtk.STOCK_OK, gtk.RESPONSE_OK)
                 
        self.set_size_request(400,250)
        self.set_title(_("Send photos by email"))
        table = gtk.Table(rows=5, columns=2)
        self.vbox.pack_start(table, fill=False, expand=False)
        table.set_border_width(7)
        self.set_icon_name("photomail")
        

        entry_recipient = gtk.Entry()
        table.attach(gtk.Label(_("Recipient")+": "),0,1,1,2)
        table.attach(entry_recipient,1,2,1,2)
        self.entry_recipient = entry_recipient       
        
        entry_msg = gtk.Entry()
        entry_msg.set_text(_("Dear recipient, Your friend wanted to send you some photos. --Intipunku (auto-generated message)"))
        table.attach(gtk.Label(_("Message")+": "), 0,1,2,3)
        table.attach(entry_msg, 1,2,2,3)
        self.entry_msg = entry_msg          
        
        entry_size = gtk.combo_box_new_text()
        entries = ["320x240", "640x480", "800x600", "1024x768", "1600x1200",_("Original")]
        for entry in entries: 
            entry_size.append_text(entry)
        entry_size.set_active(0)
        table.attach(gtk.Label(_("Resolution")+": "), 0,1,3,4)
        table.attach(entry_size, 1,2,3,4)
        self.entry_size = entry_size

        entry_compress = gtk.combo_box_new_text()
        entries = [_("None"), "zip", "tar.bz2", "tar.gz"]
        for entry in entries: 
            entry_compress.append_text(entry)
        entry_compress.set_active(0)
        table.attach(gtk.Label(_("Packed in")+": "), 0,1,4,5)
        table.attach(entry_compress, 1,2,4,5)
        self.entry_compress = entry_compress
        
        entry_transport = gtk.combo_box_new_text()
        entries = ["Gmail", _("Default mail client")]
        for entry in entries: 
            entry_transport.append_text(entry)
        entry_transport.set_active(1)
        table.attach(gtk.Label(_("Send by")+": "), 0,1,5,6)
        table.attach(entry_transport, 1,2,5,6)
        self.entry_transport = entry_transport
        self.show_all()
        
    def get_data(self):
        recipient = self.entry_recipient.get_text()
        msg = self.entry_msg.get_text()
        size = self.entry_size.get_active_text()
        if size == _("Original"): size = None
        else: size = tuple(size.split("x"))
        compression = self.entry_compress.get_active_text()
        if compression == _("None"): compression = None
        transport = self.entry_transport.get_active_text()
        if transport == _('Default mail client'): transport = "default"
        transport = transport.lower()
        return (recipient, msg, size, compression, transport)
        
class CollageDialog(SunDialog):
    def __init__(self):
        SunDialog.__init__(self)

        self.add_buttons(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                 gtk.STOCK_OK, gtk.RESPONSE_OK)
                 
        self.set_size_request(400,350)
        self.set_title(_("Create collage"))
        
        
        vbox = gtk.VBox()
        self.vbox.pack_start(vbox)
        desc = gtk.Label()
        desc.set_line_wrap(True)
        desc.set_text(_("Creating collages is a random process. You can repeat this process until you are happy with the result."))
        
        styles = [("pile",_("Pile")), ("popart",_("Pop Art"))]
        button = None
        self.buttons = []
        for style, name in styles:
            hbox = gtk.HBox()
            button = gtk.RadioButton(group=button, label=name)
            button.set_name(style)
            self.buttons.append(button)
            hbox.pack_start(button)
            preview = gtk.image_new_from_file(DATADIR + os.sep + "collage_" + style + ".png")
            hbox.pack_start(preview)
            vbox.pack_start(hbox)
        
        vbox.pack_start(desc)
            
        self.show_all()
        
    def get_data(self):
        for button in self.buttons:
            if button.get_active():
                return button.get_name()
        
class TagDialog(SunDialog):
    def __init__(self, tagindex, selection):
        SunDialog.__init__(self)

        self.add_buttons(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                 gtk.STOCK_OK, gtk.RESPONSE_OK)
                 
        self.set_size_request(400,350)
        self.set_title(_("Modify tags"))
        self.set_icon_name("tag2")
        
        self.tags_modified = {}
        
        vbox = gtk.VBox()
        vbox.set_border_width(10)
        self.vbox.pack_start(vbox)
        
        # get all tags
        seltags = []
        for f in selection:
            for t in tagindex[f]:
                if not t in seltags:
                    seltags.append(t)
        self.alltags = {}
        for f in tagindex:
            for t in tagindex[f]:
                if t not in seltags:
                    v = False
                else:
                    v = True
                if self.alltags.has_key(t):
                    pass
#                    v2 = self.alltags[t]
#                    if f in selection: print v, v2
#                    if v2 != v: # Mixed results, so should inconsistent
#                        self.alltags[t] = None
                else:
                    self.alltags[t] = v
                    
        
        for t in self.alltags:
            for f in selection:
                if t in tagindex[f] and self.alltags[t] == False:
                    self.alltags[t] = None
                elif not t in tagindex[f] and self.alltags[t] == True:
                    self.alltags[t] = None    
                    
        self.alltags_s = list(self.alltags)
        self.alltags_s.sort()

        
        # New tag frame
        frame_new = gtk.Frame(label=_("New tag"))
        vbox.pack_start(frame_new, fill=False, expand=False)
        hbox = gtk.HBox()
        frame_new.add(hbox)
        self.entry = gtk.Entry()
        button_add = gtk.Button(stock=gtk.STOCK_ADD)
        button_add.connect("clicked", self.add_tag)
        hbox.pack_start(self.entry)
        hbox.pack_end(button_add, fill=False, expand=False)
        
            
        # add/remove frame
        frame_add = gtk.Frame(label=_("Add/remove tags"))

        vbox2 = gtk.VBox()
        self.vbox_tags = gtk.VBox()
        vbox2.pack_start(self.vbox_tags)
        frame_add.add(vbox2)
        vbox.pack_start(frame_add)
        
        buttonbox = gtk.HButtonBox()
        vbox2.pack_end(buttonbox, fill=False, expand=False)
        button1 = gtk.Button(label=_("Select all"))
        button1.connect("clicked", self.select_all, True)
        button2 = gtk.Button(label=_("Unselect all"))
        button2.connect("clicked", self.select_all, False)
        buttonbox.pack_start(button1, fill=False, expand=False)
        buttonbox.pack_start(button2, fill=False, expand=False)
        
        self.load_tags()
                       
        self.show_all()
        
        
    def add_tag(self, widget):
        tag = self.entry.get_text()
        if not self.alltags.has_key(tag):
            self.alltags[tag] = False
            self.alltags_s = list(self.alltags)
            self.alltags_s.sort()
            self.load_tags()
            
    
    def load_tags(self):
        # clean
        for c in self.vbox_tags.get_children():
            c.destroy()
        
        # load
        for t in self.alltags_s:
            v = self.alltags[t]
            button = gtk.CheckButton(label=t)
            button.show()
            if v == None:
                button.set_inconsistent(True)
            else:
                button.set_active(v)
            button.set_name(t)
            button.connect("toggled", self.toggled)
            self.vbox_tags.pack_start(button, fill=False, expand=False)
            
    def toggled(self, button):
        if button.get_inconsistent(): button.set_inconsistent(False)
        self.tags_modified[button.get_name()] = button.get_active()
            
    def get_data(self):
        return self.tags_modified
        
    def select_all(self, button, v):
        for c in self.vbox_tags.get_children():
            if c.get_inconsistent(): c.set_inconsistent(False)
            self.tags_modified[c.get_name()] = v
            c.set_active(v)
                
class ComponentDialog(SunDialog):
    def __init__(self, opts):
        SunDialog.__init__(self)

        self.add_buttons(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                 gtk.STOCK_OK, gtk.RESPONSE_OK)
                 
        self.set_size_request(400,350)
        self.set_title(_("Add components"))    
        
        self.modified = {}
        
        vbox = gtk.VBox()
        vbox.set_border_width(10)
        self.vbox.pack_start(vbox)
        
        desc = gtk.Label(_("Please select which components you would like to install. From this interface it is not possible to uninstall components."))
        desc.set_line_wrap(True)
        vbox.pack_start(desc)
        
        for c in opts:
            button = gtk.CheckButton(label=opts[c][2])
            if opts[c][1]:
                button.set_active(True)
                button.set_sensitive(False)
            button.connect("toggled", self.on_toggle)
            button.set_name(c)
            vbox.pack_start(button)
        self.show_all()
        
    def on_toggle(self, widget):
        self.modified[widget.get_name()] = widget.get_active()
        
    def get_data(self):
        return self.modified            
        
        
