#!/usr/bin/env python
#
#   This file is part of IntiPunku
#   Copyright (C) 2007 Mark Baas <mark.baas123@gmail.com>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

from inti import utils
import string
from inti.constants import *
import gobject

# Localization
import gettext
_ = gettext.gettext

if not utils.inpath("exiv2"):
    import sys
    print _("\nCannot run Intipunku, you have to install exiv2!\n")
    sys.exit()

import os, time, tempfile, datetime
import subprocess

class Image(dict):
    def __init__(self, image):
        self.image = image
        self.orig = {}
        self.tagtypes = {}
        self.read = False
        self.ext = os.path.splitext(self.image)[1].lower()
        if self.ext in EXT_EXIV2: self.ext_supported = True
        else: self.ext_supported = False
        
    def readMetadata(self):
        """
        Executing exiv2 and receive all tags
        """
        if self.ext_supported:
            metadata = self._readMetadata(self.image)
            for tag in metadata:
                value = metadata[tag]
                self[tag] = value
                self.orig[tag] = value
            self.read = True
        else:
            pass#print "exiv2: Cannot read metadata of " + self.ext
        
    def _readMetadata(self, image):
        metadata = {}
        output = ""
        t = time.time()
        #fdi, fdo, fde = os.popen3("exiv2 -P kycv \"%s\"" % (image))
        p = subprocess.Popen(args=["exiv2","-P","kycv",image], stdout=subprocess.PIPE)
        output += p.stdout.read()
        #fdi, fdo, fde = os.popen3("exiv2 -p i \"%s\"" % (image))
        p = subprocess.Popen(args=["exiv2","-p","i",image], stdout=subprocess.PIPE)
        output += p.stdout.read()
        for line in output.split("\n"):         
            if len(line.split()) < 4: continue # An abnormal line
            value = string.join(line.split()[3:])
            tag = line.split()[0]
            tagtype = line.split()[1]
            self.tagtypes[tag] = tagtype
            # Here should be implemented if the value should be treated specially
            if tag == "Exif.Image.DateTime" or tag == "Exif.Photo.DateTimeOriginal":
                try: 
                    value = value.replace("-", ":")
                    y,m,d,h,mi,s = time.strptime(value[:value.find(".")], "%Y:%m:%d %H:%M:%S")[:6]
                    metadata[tag] = datetime.datetime(y,m,d,h,mi,s) 
                except StandardError, err: print "exiv2:",err; metadata[tag] = value
            elif tag == "Iptc.Application2.Keywords":
                if not metadata.has_key(tag): metadata[tag] = []
                metadata[tag].append(value)
            else:
                metadata[tag] = value
        
        return metadata    
         
    def writeMetadata(self):
        if self.ext_supported:
            self._writeMetadata(self.orig, self.image)
        else:
            pass#print "exiv2: Cannot write metadata of " + self.ext
        
    def _addText(self, orig, text, key, value, alwaysinsert=False):
        # This is not really good, but probably works almost always, worst case it just puts Undefined
        if not self.tagtypes.has_key(key): 
            if key.startswith("Exif"): self.tagtypes[key] = "Ascii"
            if key.startswith("Iptc"): self.tagtypes[key] = "String"
        
        if not orig.has_key(key) or alwaysinsert:
            #print "inserting tag %s (%s)" % (key, self.tagtypes[key])
            text += "add %s %s '%s'\n" % (key, self.tagtypes[key], value)
        elif value != orig[key]:           
            #print "writing tag %s = %s (%s)" % (key, self.tagtypes[key], value)
            text += "set %s %s '%s'\n" % (key, self.tagtypes[key], value)    
        return text
    
    def _writeMetadata(self, orig, image):
        if not self.ext_supported: return
        if not self.read: raise StandardError, "You have to read metadata first"
        text = ""
        for key in self:
            value = self[key]
            # Here exception how to write the value
            if "DateTime" in key:
                #print "exiv2:",key, value
                value = str(value).replace("-", ":")
                text = self._addText(orig, text, key, value)
            elif key == "Iptc.Application2.Keywords":
                # first delete all
                text += "del " + key + "\n"
                # add again
                for value2 in value:
                    text = self._addText(orig, text, key, value2, True) 
            else:
                text = self._addText(orig, text, key, value)

        if text != "":
            temp = tempfile.mkstemp(prefix="intiexiv-")
            tempf = temp[1]
            
            f = open(tempf, "w")
            f.write(text)
            f.close()
            os.close(temp[0])
            
            #print "exiv2 -m \"%s\" \"%s\"" % (tempf, image)
            #fdi, fdo, fde = os.popen3("exiv2 -m \"%s\" \"%s\"" % (tempf, image)) 
            p = subprocess.Popen(args=["exiv2","-m",tempf,image], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            print p.stderr.read()
            os.remove(tempf)

    def copyMetadataTo(self, image):
        orig = self._readMetadata(image)
        self._writeMetadata(orig, image)
        
        
        
if __name__ == "__main__":
    import sys
    image = Image(sys.argv[1])
    image.readMetadata()
    print image["Exif.Image.DateTime"]
    image["Exif.Image.DateTime"] = "2010:12:09 14:47:00"
    image.writeMetadata()
    #image.copyMetadataTo(sys.argv[2])
