#!/usr/bin/env python
#
#   This file is part of IntiPunku
#   Copyright (C) 2007 Mark Baas <mark.baas123@gmail.com>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

# This module is created to add more windows compatibility to pygtk
# and to solve various bugs. So in this way i did not have to change
# so much code that works fine under Linux.

from gtk._gtk import *
import gobject
import os, os.path
import win32gui
from win32com.shell import shell, shellcon

GtkButton = Button
GtkWindow = Window
GtkFileChooserButton = FileChooser
GtkEventBox = EventBox
MESSAGE_ALERT = MESSAGE_QUESTION

# Fullscreen doesn't work in windows, damn taskbar, in this way it does.      
class Window(GtkWindow):
    def __init__(self, *args, **kwargs):
        GtkWindow.__init__(self, *args, **kwargs)
        
    def fullscreen(self):
        self.set_decorated(False)
        self.maximize()
        
        
class FileChooserButton(Button):
    __gsignals__ = {
        'changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
        }
    def __init__(self, title="Select a folder"):
        Button.__init__(self)
        self.title = title
        pidl = shell.SHGetFolderLocation (0, shellcon.CSIDL_DESKTOP, 0, 0)
        self.filename = shell.SHGetPathFromIDList(pidl)
        hbox = HBox()
        self.add(hbox)
        hbox.pack_start(image_new_from_icon_name("folder", 24), fill=False, expand=False)
        self.label = Label(title)
        hbox.pack_start(self.label)
        
        self.connect("show", self.show_cb)
        self.connect("clicked", self.clicked_cb)
        
    def show_cb(self, *args):
        print self.get_allocation().width
     
    def clicked_cb(self, widget):
        pidl_now = shell.SHGetFolderLocation (0, shellcon.CSIDL_DESKTOP, 0, 0)
        pidl, name, list = shell.SHBrowseForFolder(win32gui.GetDesktopWindow(),
                                                   pidl_now,self.title, 0, None, None)
        try:
            path = shell.SHGetPathFromIDList(pidl)
            self.filename = os.path.abspath(path)
            self.set_label(self.filename)
            self.emit("changed")
        except:
            pass # Guess i cannot change folder
        
    def set_current_folder(self, path):
        self.set_label(path)
        self.filename = path
        
    def get_filename(self):
        return self.filename
        
    def set_action(self, *args):
        pass
        
    def set_label(self, label):
        self.label.set_text(unicode(label, "latin-1").encode("utf-8"))
        
    def get_children(self):
        return [None, self]
