#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   This file is part of IntiPunku
#   Copyright (C) 2007 Mark Baas <mark.baas123@gmail.com>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os, sys
import inti.intigtk as gtk
import gobject
import utils
import tools
import metadata

# Load/check optional components
def load_gdata():
    try:
        import gdata.photos
        import gdata.photos.service
        global gdata
        return True
    except:
        return False
        
def load_gphoto():
    try:
        import gphoto2
        global gphoto2
        return True
    except:
        return False
    
GDATA_ENABLED = load_gdata()
GPHOTO_ENABLED = load_gphoto()

import tempfile
from distutils import dir_util, file_util
import urllib
import datetime, time
from PIL import Image
import StringIO
from inti import metadata

from inti import utils
from inti import web
from inti import dialogs
from browser import SunAlbum
import tools as tools
from inti.constants import *

# Facebook disable in this release
#from inti import facebook

# Localization
import gettext
_ = gettext.gettext

class ImportPage(gtk.VBox):

    options = {"import_original" : _("Keep original names"),
                    "import_delete" : _("Delete original files"),
                    
            }

    def __init__(self, win, sunalbums, config, callback=None):
        
        gtk.VBox.__init__(self)
        self.set_border_width(5)
        self.win = win
        self.sunalbums = sunalbums
        self.callback = callback
        self.config = config
        self.quit_load = False

        self.imindex = {}
        self.dindex = {}
        self.tagindex = {}
        self.tmpd = tempfile.mkdtemp(prefix="intipunku-import")
        # album/device
        hbox = gtk.HBox()
        hbox.set_spacing(5)
        self.image = gtk.image_new_from_pixbuf(utils.load_icon(self.icon,size=48))
        self.gtk_dev_label = gtk.Label(self.dev_label)
        self.combo = gtk.combo_box_new_text()
        hbox.pack_start(self.image, fill=False, expand=False)
        hbox.pack_start(self.gtk_dev_label, fill=False, expand=False)
        hbox.pack_start(self.combo)
        
        self.pack_start(hbox, fill=False, expand=False)

        self.browser = SunAlbum(self.imindex, self.dindex, win, config, callback=callback, thumbsize=100)
        self.pack_start(self.browser)
        
        # Select/unselect buttons
        hbox = gtk.HButtonBox()
        hbox.set_layout(gtk.BUTTONBOX_START)

        agroup = gtk.AccelGroup()
        self.win.add_accel_group(agroup)
        button = gtk.ToggleButton(_("Select all"))
        button.connect("toggled", self.select_cb)
        button.add_accelerator("activate", agroup, ord("a"), gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
        hbox.pack_start(button)

        self.pack_start(hbox, fill=False, expand=False)


        # Options
        for option in self.options:
            check = gtk.CheckButton(self.options[option])
            check.set_active(self.config[option])
            check.connect("toggled", self.checkb_cb, option)
            self.pack_start(check, expand=False)

        # To album?
        self.combo2 = gtk.combo_box_entry_new_text()
        for album in self.sunalbums:
            self.combo2.append_text(album)
        self.combo2.insert_text(0, _("Select or insert album"))

        hbox = gtk.HBox()
        hbox.pack_start(gtk.Label(_("Import to album") + ":"), fill=False, expand=False)
        hbox.pack_start(self.combo2)
        self.pack_start(hbox, fill=False, expand=False)


        # connect events
        win.connect("delete-event", self.cb_delete_event)
        win.connect("destroy", self.cb_delete_event)
        self.combo.connect("changed", self.cb_combo_changed)

        self.init()
        self.loadAlbumList()
        
        self.show_all()

    def init(self):
        pass
    
    def Progress(self, msg):
        # This progress window needs to be set transient
        win = self.get_parent().get_parent()
        progress = dialogs.SunProgress(msg, cancel=True, icon=self.icon, win=win)
        return progress

    def _loadAlbum(self, album, vt=0):
        self.combo2.remove_text(0)
        self.combo2.insert_text(0, album)
        self.combo2.set_active(0)
        self.browser.imindex = self.imindex
        self.browser.dindex = self.dindex
        self.browser.tagindex = self.tagindex
        self.browser.update(album, vt=vt)
        
    def cb_delete_event(self, *args):
        self.quit_load = True
        dir_util.remove_tree(self.tmpd)
        self.win.destroy()

    def cb_combo_changed(self, combo):
        if combo.get_active() == -1: return # Album is empty
        self.quit_load = True
        album_id = self.albums[combo.get_active()]
        self.loadAlbum(album_id)

    def LoadAlbumList(self):
        pass

    def LoadAlbum(self, album_id):
        pass

    def doImport(self):
        self.quit_load = True
        #print "importing"
        # get selection
        selection = []
        for box in self.browser.box_selection:
            selection.append(box.get_children()[0].get_name())

        # get to album?
        toalbum = self.combo2.get_active_text()
        if toalbum.strip() == "":
            toalbum = os.path.basename(os.path.dirname(selection[0]))

        albumpath = self.config["photo_folder"] + os.sep + toalbum
        if not os.path.exists(albumpath):
            dir_util.mkpath(albumpath)
        progress = self.Progress(_("Importing photos, this may take a while..."))
        progress.start(len(selection), _("Importing files"))
        for f in selection:
            if progress.cancel: break
            progress.set_text(_("Importing") + " %s" % os.path.basename(f))
            dest,ext = os.path.splitext(os.path.basename(f))
            ext = ext.lower()
            if self.config["import_original"]:
                if os.path.exists(albumpath + os.sep + dest + ext):
                    dialog = gtk.MessageDialog(type=gtk.MESSAGE_QUESTION, buttons=gtk.BUTTONS_OK_CANCEL)
                    dialog.add_buttons(_("Rename"), -2000)

                    dialog.set_icon_name("dialog-question")
                    dialog.set_markup(_("%s already exists, are you sure you want to overwrite this file?\nRenaming will append '_' to the file name." % os.path.basename(f)))
                    r = dialog.run()
                    if r == gtk.RESPONSE_CANCEL:
                        dialog.destroy()
                        continue
                    elif r == -2000:
                        while os.path.exists(albumpath + os.sep + dest + ext):
                            dest += "_"                  
                    dialog.destroy()
            else:
                dest = "00001"
                while os.path.exists(albumpath + os.sep + dest + ext):
                    dest = "%05d" % (int(dest)+1)

            dest_path = albumpath + os.sep + dest + ext
            self.copyFile(f, dest_path)
            print "copying ", f, dest_path
            if self.config["import_delete"]:
                self.deleteFile(f)
            progress.plus()
        progress.destroy()

    def checkb_cb(self, check, name):
        self.options[name] = check.get_active()
        self.config[name] = check.get_active()
        self.config.save()

    def select_cb(self, widget):
        
        if widget.get_active() == True:
            self.browser.select_box_all()
        else:
            self.browser.unselect_box_all()

    def deleteFile(self, f):
        pass


class PicasaPage(ImportPage):
    
    albums = {}
    title = "PicasaWeb"
    dev_label = _("Album") + ":"
    icon = "picasaweb"
    photos = []
    username = ""
    options = {"import_original" : _("Keep original names"),}
    
    def init(self):
    
        self.client = gdata.photos.service.PhotosService()
        hbox = self.get_children()[0]
        hbox.remove(self.combo)
        hbox.remove(self.gtk_dev_label)
        vbox = gtk.VBox()
        hbox.pack_start(vbox)        

        hbox2 = gtk.HBox()
        hbox2.set_spacing(4)
        hbox1 = gtk.HBox()
        hbox1.set_spacing(4)
        vbox.pack_start(hbox1, expand=False, fill=False)
        vbox.pack_start(hbox2)

        self.entry_user = gtk.Entry()
        hbox1.pack_start(gtk.Label(_("Username") + ":"), expand=False, fill=False)
        hbox1.pack_start(self.entry_user, expand=True, fill=True)
        submit = gtk.Button(label=_("Connect"))
        submit.connect("clicked", self.entry_cb)
        hbox1.pack_start(submit, fill=False, expand=False)
        hbox2.pack_start(self.gtk_dev_label, expand=False, fill=False)
        hbox2.pack_start(self.combo, expand=True, fill=True)


        #self.username, password = web.get_picasa_login(self.config)
        #if self.username == None:
        #    self.callback("sunset", self.win)
        #    return

        self.entry_user.set_text(self.username)
        

    def loadAlbumList(self):
        self.combo.get_model().clear()

#        try:
#            feed = self.client.GetUserFeed(user=self.username, kind='album')
#        except gdata.photos.service.GooglePhotosException, err:
#            dialog = gtk.MessageDialog(type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK)
#            dialog.set_icon_name("dialog-error")
#            dialog.set_markup(_("Invalid login, please try again"))
#            r = dialog.run()
#            dialog.destroy()
#            return []
        feed = self.client.GetUserFeed(user=self.username, kind='album')
        n = 0
        for a in feed.entry:
            self.albums[n] = a.link[0].href
            self.combo.append_text(a.title.text)
            n += 1
        
        return self.albums

    def loadAlbum(self, album_id):
        feed = self.client.GetFeed(album_id)
        progress = self.Progress(_("Loading thumbnails"))
        progress.start(len(feed.entry), _("Downloading thumbs"))
        album = feed.title.text
        for entry in feed.entry:      
            if progress.cancel: break
            fname = entry.media.content[0].url
            if self.imindex.has_key(fname):
                continue
            thumb_url = entry.media.thumbnail[1].url
           
            # get date     
            try:
                date = datetime.datetime(*time.localtime(float(entry.exif.time.text)/1000)[:6])
            except:
                date = datetime.datetime(*time.strptime(entry.published.text[:-5], "%Y-%m-%dT%H:%M:%S")[:6])
            progress.set_text(_("Downloading thumb") + ": " + os.path.basename(fname))

            # get thumb
            while 1:
                thumb_path = self.tmpd + os.sep + "SOLTHUMB_" + utils.random_string() + ".png"
                if not os.path.exists(thumb_path):
                    break   
            try: # Do do not let bad jpegs or bad links destroy the process, skip'em
                f = urllib.urlopen(thumb_url) 
                im = Image.open(StringIO.StringIO(f.read()))
                im = tools.shadow(im)
                im.save(thumb_path, "PNG")        
            except StandardError, err:
                print fname, err
                continue    

            # setup indexes
            self.imindex[fname] = thumb_path
            self.dindex[fname] = date
            self.tagindex[fname] = [album]

            progress.plus()
        progress.destroy()
        self._loadAlbum(album, vt=1)

    def copyFile(self, s, d):
        f = urllib.urlopen(s) 
        f2 = open(d, "w")
        f2.write(f.read())
        f2.close()
        f.close()
        # write date correct
        metadata.set_date(d, self.dindex[s])

    def entry_cb(self, widget):
        self.username = self.entry_user.get_text()
        self.loadAlbumList()

class CameraPage(ImportPage):
    title = _("Camera")
    dev_label = _("Device") + ":"
    photos = {}
    albums = {}
    icon = "camera"
    options = {"import_original" : _("Keep original names"),
                    "import_delete" : _("Delete original files") + " (" + _("Probably has no effect" + ")"),
            }

    def loadCamera(self):
        try:
            # maybe gvfs is block the camera, try to kill it off
            if not WIN and not MAC and not KDE:
                os.system("gvfs-mount -u ~/.gvfs/gphoto2*")
            self.cam = gphoto2.camera()
            self.cam.init()
            return True
        except:
            return False         

    def loadAlbumList(self):       

        if not self.loadCamera():
            utils.report_error(_("Cannot load the camera; either:\n\n1) the camera is not connected, then close this window, connect the camera and try again\n\n2) it is being used by another program, then close this window, close the program the is using the camera and try again."))
            self.callback("sunset", self.win)
            return
        
        album = self.cam.abilities.model
        self.combo.append_text(album)
        self.albums[0] = album
        self.photos[album] = []
        self.combo.append_text(album)
        self.albums[0] = album
        self.photos[album] = []  
        #make list of photos
        allfiles = []
        allfolders = ["/"]
        for folder in allfolders:
            print folder
            list = self.cam.list_folders_in_folder(folder)
            for i in range(0,list.count()):
                if not folder.endswith("/"): sep = "/"
                else: sep = ""
                allfolders.append(folder + sep + list[i][0])
            print folder
            list = self.cam.list_files_in_folder(folder)
            for i in range(0,list.count()):                
                photo = (folder, list[i][0])     
                self.photos[album].append(photo)
        print "got all photos"
           
        gobject.idle_add(self.combo.set_active, 0)
        
        
    def getMeta(self, destpath, thumb_path):
        try: # Do do not let bad jpegs destroy the process, skip'em
            im = tools.create_thumb(destpath, (80,80))
            im.save(thumb_path, "PNG")
            date,tags = metadata.get_datetags(destpath)
        except StandardError, err:
            print err
                           
        # indexes
        self.imindex[destpath] = thumb_path
        self.dindex[destpath] = date
        

    def loadAlbum(self, album):
        progress = self.Progress(_("Loading thumbnails"))
        progress.start(len(self.photos[album]), _("Downloading photos"))
 
                    
        for srcdir, srcfile in self.photos[album]:
            if progress.cancel: break
 
            progress.set_text(_("Downloading photo") + ": " + srcfile)
            # get photo and make thumb
            while 1:
                thumb_path = self.tmpd + os.sep + "SOLTHUMB_" + utils.random_string() + ".png"
                if not os.path.exists(thumb_path):
                    break  
                    
            destpath = self.tmpd + os.sep + srcfile
            attempt = 0
            while 1:
                try:
                    self.cam.download_file_to(srcdir, srcfile, destpath)
                    break
                except Exception, err:
                    print err
                    utils.report_error(_("Connection to the camera seemed to be lost, please reconnect the camera"))
                    self.loadCamera()
                    attempt += 1
                if attempt > 3: 
                    self.callback("sunset", self.win)
            gobject.idle_add(self.getMeta, destpath, thumb_path)
            self.tagindex[destpath] = album
            
            progress.plus()
        
        progress.destroy()
        self._loadAlbum(album, vt=1)    
        
    def copyFile(self, s, d):
        
        file_util.copy_file(s, d)

    def deleteFile(self, f):
        return
        album = self.albums[self.combo.get_active()]
        for n, photo in self.photos[album]:
            if f == photo:
                break
        #os.popen("gphoto2 -d %s" % n)
        print n

class LocationPage(ImportPage):
    title = _("Location")
    dev_label = _("Folder") + ":"
    album = None
    icon = "stock_folder"
    options = {"import_original" : _("Keep original names"),
                "import_delete" : _("Delete original files"),
                "import_subfolders": _("Search in subfolders"),
                
        }
    quit = False

    def loadAlbumList(self):
        # override the combobox and replace it with fileselector
        hbox = self.combo.get_parent()
        self.combo.destroy()
        self.fcb = gtk.FileChooserButton(_("Select folder"))
        self.fcb.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER) 
        hbox.pack_start(self.fcb)
        self.fcb.show()      
        button = gtk.Button(stock=gtk.STOCK_FIND)
        hbox.pack_start(button, fill=False, expand=False)
        button.connect("clicked", self.loadAlbum)

    def loadAlbum(self, *args):    
        album = self.fcb.get_filename()
        progress = self.Progress(_("Loading thumbnails"))
        
        files = []
        if self.options["import_subfolders"]:
            for a,b,c in os.walk(album):
                for d in c:
                    if os.path.splitext(d)[1].lower() in EXT:
                        files.append(os.path.join(a,d))
        else:
            for im in os.listdir(album):
                path = album + os.sep + im
                if os.path.splitext(path)[1].lower() in EXT:
                    files.append(path)
        progress.start(len(files), _("Creating thumbs")) 

        for path in files:    
            title,ext = os.path.splitext(path)

            if not ext.lower() in EXT:
                continue 
                
            if progress.cancel: break
            progress.set_text(_("Creating thumb") + ": " + os.path.basename(path))
            
            while 1:
                thumb_path = self.tmpd + os.sep + "SOLTHUMB_" + utils.random_string() + ".png"
                if not os.path.exists(thumb_path):
                    break   
            try: # Do do not let bad jpegs destroy the process, skip'em
                # create thumbs
                im = tools.create_thumb(path, (80,80))
                im.save(thumb_path, "PNG") 
   
                # get date       
                date, tags = metadata.get_datetags(path)
            except StandardError, err:
                print err
                continue

            #indexes
            self.imindex[path] = thumb_path
            self.dindex[path] = date
            self.tagindex[path] = album
            
            progress.plus()

        progress.destroy()

        self._loadAlbum(os.path.basename(album), vt=1)

    def copyFile(self, s, d):
        file_util.copy_file(s, d)

    def deleteFile(self, f):
        os.remove(f)
"""
class FacebookPage(ImportPage):
    client = facebook.Facebook(API_KEY, API_SECRET)
    albums = []
    title = "Facebook"
    dev_label = _("Album") + ":"
    icon = "facebook"
    photos = []
    username = ""
    friends = []
    
    def init(self):
        hbox = self.get_children()[0]
        hbox.remove(self.combo)
        hbox.remove(self.gtk_dev_label)
        vbox = gtk.VBox()
        hbox.pack_start(vbox)        

        hbox2 = gtk.HBox()
        hbox2.set_spacing(4)
        hbox1 = gtk.HBox()
        hbox1.set_spacing(4)
        vbox.pack_start(hbox1, expand=False, fill=False)
        vbox.pack_start(hbox2)

        self.entry_user = gtk.combo_box_new_text()
        hbox1.pack_start(gtk.Label(_("Username/Friend") + ":"), expand=False, fill=False)
        hbox1.pack_start(self.entry_user, expand=True, fill=True)
        submit = gtk.Button(label=_("Submit"), stock=gtk.STOCK_OK)
        submit.connect("clicked", self.entry_cb)
        hbox1.pack_start(submit, fill=False, expand=False)
        hbox2.pack_start(self.gtk_dev_label, expand=False, fill=False)
        hbox2.pack_start(self.combo, expand=True, fill=True)

        ##
        self.login()     
        
        self.entry_user.insert_text(0,_("Myself"))
        gobject.idle_add(self.loadFriendList)
           
        
    def login(self):
        self.client.auth.createToken()
        login = web.get_facebook_login(self.client, self.config.confdir)
        

    def loadFriendList(self):
        progress = dialogs.SunProgress(_("Loading friendlist, \nif you have many friend this might time a while"), cancel=True, icon=self.icon)
        friends = self.client.friends.get()
        progress.start(len(friends), "")
        for uid in friends:
            name = self.client.users.getInfo(uids=[uid])[0]['name']
            self.friends.append((uid, name))
            self.entry_user.append_text(name)
            progress.plus()
        progress.destroy()
        
    def loadAlbumList(self):
        self.combo.get_model().clear()

        infos = self.client.users.getInfo(uids=[self.username])
        for info in infos:
            for album in self.client.photos.getAlbums(uid=info['uid']):
                self.albums.append((album['aid'], album['name']))
                self.combo.append_text(album['name'])
        
        return self.albums

    def loadAlbum(self, album_id):

        progress = dialogs.SunProgress(_("Loading thumbnails"), cancel=True, icon=self.icon)
        progress.start(len(self.albums), _("Downloading thumbs"))

        aid, album = album_id

        for photo in self.client.photos.get(aid=aid):
            print photo
            thumb_url = photo['src']
            fname = photo['src_big']
            timestamp = int(photo['created'])
            date = datetime.datetime.fromtimestamp(timestamp)
            
            progress.set_text(_("Downloading thumb") + ": " + os.path.basename(fname))


            # get thumb
            while 1:
                thumb_path = self.tmpd + os.sep + "SOLTHUMB_" + utils.random_string() + ".jpg"
                if not os.path.exists(thumb_path):
                    break   
            try: # Do do not let bad jpegs or bad links destroy the process, skip'em
                f = urllib.urlopen(thumb_url) 
                im = Image.open(StringIO.StringIO(f.read()))
                im = tools.shadow(im)
                im.save(thumb_path)        
            except StandardError, err:
                print fname, err
                continue    

            # setup indexes
            self.imindex[fname] = thumb_path
            self.dindex[fname] = date
            self.tagindex[fname] = [album]

            progress.plus()
        progress.destroy()
        self._loadAlbum(album, vt=1)

    def copyFile(self, s, d):
        f = urllib.urlopen(s) 
        f2 = open(d, "w")
        f2.write(f.read())
        f2.close()

    def entry_cb(self, widget):
        n = self.entry_user.get_active()
        uid, name = self.friends[n-1]
        self.username = uid
        self.loadAlbumList()
"""


_pages = {"camera": CameraPage,"picasaweb": PicasaPage, "folder": LocationPage, "scanner": None}
pages = {}
for page, label in page_labels:
    pages[page] = _pages[page]
    
def cb(*args):
    print args



