#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   This file is part of IntiPunku
#   Copyright (C) 2007 Mark Baas <mark.baas123@gmail.com>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os, glob, sys, os.path

from PIL import Image, ImageDraw, ImageFont
import tools
import inti.intigtk as gtk
import pango, cairo
from constants import *
import dialogs

# Localization
import gettext
_ = gettext.gettext

M = 40

if WIN:
    import win32print
    import tempfile
    M = 300


def draw_text(cr, (x,y), text, size):
    # Some black text
    cr.set_source_rgba(0,0,0,1)
    cr.select_font_face("Arial", cairo.FONT_SLANT_NORMAL, cairo.FONT_WEIGHT_NORMAL)
    cr.set_font_size(size)

    cr.move_to(x,y)
    # We stroke and fill to make sure thinner parts are visible.
    cr.text_path(text)
    cr.set_line_width(0.5)
    cr.stroke_preserve()
    cr.fill()

def draw_page(op, context, page_nr, images, layout, title, progress, filename, date2):
    cr = context.get_cairo_context()
    width = int(context.get_width())
    height = int(context.get_height())
    columns = layout[0]
    rows = layout[1]
    onpage = layout[0]*layout[1]
    images = images[(page_nr+1)*onpage-onpage:(page_nr+1)*onpage]
    ind_w = int(float(width)/columns)
    ind_h = int(float(height)/rows)

    if page_nr == 0:
        # Some black text
        cr.set_source_rgba(0,0,0,1)
        cr.select_font_face("Arial", cairo.FONT_SLANT_NORMAL, cairo.FONT_WEIGHT_NORMAL)
        cr.set_font_size(M+0.25*M)
        # We need to adjust by the text's offsets to center it.
        x_bearing, y_bearing, widtht, heightt = cr.text_extents(title)[:4]
        x = int((context.get_width()/2)-(widtht/2))
        draw_text(cr, (x,-y_bearing), title, M+0.25*M)
    else:
        heightt = 0    
        
    # Pictures
    n = 0
    r = 1
    x, y = (M/2, heightt+M/2)
    h_max = 0
    w_max = 0

    for date,image in images:
        progress.plus()
        # define margins
        if n == columns: xmargin = 0
        else: xmargin = M

        if r == rows: ymargin = 0
        else: ymargin = M

        # open and thumbnail image
        im = Image.open(image)
        im.thumbnail((ind_w-M, ind_h-M))
               
        # get size data
        w, h = im.size
        if h > h_max: h_max = h
        if w > w_max: w_max = w
        
        # convert to pixbuf and past on cr
        pixbuf = tools.pil2pixbuf(im)
        cr.set_source_pixbuf(pixbuf, x+int(float(w_max-w)/2), y)
        cr.paint()
        
        if filename:
            # add filename label
            draw_text(cr, (x+int(float(w_max-w)/2), y+h+M/4), os.path.basename(image), M/4)
        if date2:
            # add date label
            draw_text(cr, (x+int(float(w_max-w)/2), y+h+2*(M/4)), str(date), M/4)
        
        # numbering
        x += w_max + xmargin    
        n += 1
        if n == columns:
            n = 0
            r += 1
            x = M/2
            y += h_max + ymargin
            
    layout = context.create_pango_layout()
    cr.show_layout(layout)

def intiprint(files, title="", win=None, settings=None ,layout=(2,3), filename=True, date=True):
    files.sort()
    print_op = gtk.PrintOperation()
    onpage = layout[0]*layout[1]
    pages = len(files)/float(onpage)

    if pages > int(pages): pages = int(pages)+1
    else: pages = int(pages)

    print_op.set_n_pages(pages)
    progress = dialogs.SunProgress(_("Printing"), icon="gtk-print")
    print_op.connect("draw_page", draw_page, files, layout, title, progress, filename,date)
    
    if settings != None:
        print_op.set_print_settings(settings)
    progress.start(len(files))
    try:
        res = print_op.run(gtk.PRINT_OPERATION_ACTION_PRINT)
    except:
        res = None
    progress.destroy()
    return res



