#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   This file is part of IntiPunku with contribution and  inspiration of sane-pygtk
#   Copyright (C) 2009 Mark Baas <mark.baas123@gmail.com>
#   Copyright (C) 2008 Yaacov Zamir <kzamir_a_walla.co.il>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#   

import os, os.path
import inti.intigtk as gtk
from inti import utils
from inti import dialogs
import gtkimageview
import types
from distutils import dir_util

# Localization
import gettext
_ = gettext.gettext

def load_sane():
    try:
        import sane
        global sane
        return True
    except:
        return False
        
SANE_ENABLED = load_sane()

# set global functions
def get_scanner_option (scanner, name):
  options = scanner.get_options()
  
  for option in options:
    if option[1] == name:
       return option
  return False
  
class IntiSane(dialogs.SunDialog):
    def __init__(self, albums, conf):
        dialogs.SunDialog.__init__(self)
        
        self.set_default_size(600,400)
        self.set_title(_("Intipunku Scanner"))
        self.config = conf
        
        hbox = gtk.HBox()
        self.vbox.pack_start(hbox)
        self.add_buttons(_("Scan"), gtk.RESPONSE_OK, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
        
        
        frame_settings = gtk.Frame(label=_("Settings"))
        hbox.pack_start(frame_settings, False, False)
        table_settings = gtk.Table()
        vbox_settings = gtk.VBox()
        vbox_settings.pack_start(table_settings, False, False)
        frame_settings.add(vbox_settings)
        # entries
        self.entry_scanner = gtk.combo_box_new_text()
        self.entry_mode = gtk.combo_box_new_text()
        self.entry_res = gtk.HScale()#gtk.SpinButton(gtk.Adjustment(value=0, lower=0, upper=1000, step_incr=1, page_incr=10, page_size=1))
        self.entry_brightness = gtk.HScale()#gtk.SpinButton(gtk.Adjustment(value=0, lower=0, upper=100, step_incr=1, page_incr=10, page_size=1))
        self.entry_contrast = gtk.HScale()#gtk.SpinButton(gtk.Adjustment(value=0, lower=0, upper=100, step_incr=1, page_incr=10, page_size=1))
        self.entry_album = gtk.combo_box_entry_new_text()
        self.entry_name = gtk.Entry()
        self.entry_name.set_text(_("Scanned photo"))
        settings = ( (_("Scanner"), self.entry_scanner),
                     (_("Mode") , self.entry_mode),
                     (_("Resolution") , self.entry_res),
                     (_("Brightness") , self.entry_brightness),
                     (_("Contrast") , self.entry_contrast),
                     (_("Photo name")+"*", self.entry_name),
                     (_("Scan to album"), self.entry_album),
                     )
        r = 0
        for label, setting in settings:
            table_settings.attach(gtk.Label(label+":"), 0, 1, r, r+1, xoptions=gtk.SHRINK, yoptions=gtk.SHRINK)
            table_settings.attach(setting, 1, 2, r, r+1, yoptions=gtk.SHRINK)
            r += 1
            
        bbox = gtk.HButtonBox()    
        
        reset_button = gtk.Button(label=_("Reset"))
        reset_button.set_image(gtk.image_new_from_pixbuf(utils.load_icon("undo")))
        bbox.pack_start(reset_button, False, False)
        
        label = gtk.Label()
        label.set_markup("<span size='8500'>* "+_("Photo name without extension, the extension defaults to .jpg")+"</span>")
        vbox_settings.pack_start(label)
            
        # preview
        vbox_preview = gtk.VBox()
        hbox.pack_start(vbox_preview)
        vbox_preview.pack_start(gtk.Label(_("Preview")), False, False)
        self.draw = gtkimageview.ImageView()
        self.stool = gtkimageview.ImageToolSelector(self.draw)
        self.draw.set_tool(self.stool)
        scroll = gtkimageview.ImageScrollWin(self.draw)
        vbox_preview.pack_start(scroll)
        preview_button = gtk.Button(label=_("Preview"))
        preview_button.set_image(gtk.image_new_from_pixbuf(utils.load_icon("camera")))
        bbox.pack_start(preview_button, False, False)
        
        self.vbox.pack_start(bbox, False, False)
        
        self.modes_t = { "Color" : _("Color"), "Gray" : _("Grayscale"), "Lineart" : _("Black and white")}
        
        # Connections
        preview_button.connect("clicked", self.button_preview_clicked_cb)
        self.entry_scanner.connect("changed", self.combobox_scanner_changed_cb)
        
        # Load albums
        for album in albums:
            self.entry_album.append_text(album)
        self.entry_album.insert_text(0, _("Select or insert album"))
        self.entry_album.set_active(0)
        
        # get scanners in system
        sane.init()
        self.sane_devices = sane.get_devices()
        sane.exit()
        
        # set the scanners combobox
        for device in self.sane_devices:
            self.entry_scanner.append_text(device[2])
        
        # get the current scanner
        self.entry_scanner.set_active(0)
        
        self.show_all()
        
    def button_preview_clicked_cb(self, obj):
        "button_preview_clicked_cb activated"
        
        # open scanner
        scanner_number = self.entry_scanner.get_active()
        sane.init()
        scanner = sane.open(self.sane_devices[scanner_number][0])
        # options
        options = get_scanner_option (scanner, 'mode')
        if options:
          scanner.mode = options[8][self.entry_mode.get_active()]
        options = get_scanner_option (scanner, 'resolution')
        if options:
          try:
              scanner.resolution = int(options[8][0])
          except AttributeError:
              print "intisane: Can't set scan resolution"
        try:
            scanner.brightness = int(self.spinbutton_brightness.get_value())
        except AttributeError:
              print "intisane: Can't set scan brightness"
        try:
              scanner.contrast = int(self.spinbutton_contrast.get_value())
        except AttributeError:
              print "intisane: Can't set scan contrast"
        try:
              scanner.preview = 1
        except AttributeError:
              print "intisane: Can't set scan preview"
        
        # geometry
        try:
          scanner.tl_x = 0
          scanner.tl_y = 0
          options = get_scanner_option (scanner, 'br-x')
          scanner.br_x = options[8][1]
          options = get_scanner_option (scanner, 'br-y')
          scanner.br_y = options[8][1]
        except AttributeError:
          print "intisane: Can't set scan geometry"
        
        #scan
        try:
            pil_image = scanner.scan()
                        
            w, h = pil_image.size
            data = pil_image.tostring()
        
            if pil_image.mode == 'L':
                rgb_data = ''
                for c in data:
                        rgb_data += c + c + c
                data = rgb_data
        
            if pil_image.mode == 'RGB' or pil_image.mode == 'L':
                self.pixbuf = gtk.gdk.pixbuf_new_from_data(data, gtk.gdk.COLORSPACE_RGB, False, 8, w, h, w * 3)
                #self.preview_pixbuf = self.pixbuf.scale_simple(int(w * factor), int(h * factor), gtk.gdk.INTERP_NEAREST)
                
            self.draw.set_pixbuf(self.pixbuf)
            
        except sane._sane.error, err:
            print "intisane: Can't scan"
            utils.report_error(_("Can't scan preview image,\n%s,\nCheck scanner settings") % err)
        
        # close scanner
        scanner.close()
        sane.exit()
        
    def combobox_scanner_changed_cb(self, obj):
        "combobox_scanner_changed_cb activated"
        
        scanner_number = obj.get_active()
        
        
        # reset settings options
        if scanner_number < len(self.sane_devices):
            try:
                sane.init()
                scanner = sane.open(self.sane_devices[scanner_number][0])
                
                # if scanner exist fill in all the settings
                
                # mode
                options = get_scanner_option (scanner, 'mode')
                i = 0
                # clear mode combobox
                model = self.entry_mode.get_model()
                self.entry_mode.set_model(None)
                model.clear()
                # add new entreis
                for mode in options[8]:
                    model.append([self.modes_t[mode]])
                self.entry_mode.set_model(model)
                mode_number = 2
                self.entry_mode.set_active(int(mode_number))
                
                # resolution
                options = get_scanner_option (scanner, 'resolution')
                if options:
                    if type(options[8]) == types.ListType:
                        self.entry_res.set_range(options[8][0], options[8][-1])
                    else:
                        self.entry_res.set_range(options[8][0], options[8][1])
                
                # brightness
                options = get_scanner_option (scanner, 'brightness')
                if options:
                    if type(options[8]) == types.ListType:
                        self.entry_brightness.set_range(options[8][0], options[8][-1])
                    else:
                        self.entry_brightness.set_range(options[8][0], options[8][1])
                
                # contrast
                options = get_scanner_option (scanner, 'contrast')
                if options:
                    if type(options[8]) == types.ListType:
                        self.entry_contrast.set_range(options[8][0], options[8][-1])
                    else:
                        self.entry_contrast.set_range(options[8][0], options[8][1])
                
                # scan window dimentions
                #options = get_scanner_option (scanner, 'br-x')
                #self.config.set('window', 'width', str(options[8][1]))
                #options = get_scanner_option (scanner, 'br-y')
                #self.config.set('window', 'height', str(options[8][1]))
                
                scanner.close()
                sane.exit()
                
                # set values
                self.entry_mode.set_active(self.config["scan_mode"])
                self.entry_res.set_value(self.config["scan_res"])
                self.entry_brightness.set_value(self.config["scan_brightness"])
                self.entry_contrast.set_value(self.config["scan_contrast"])
        
            except RuntimeError:
                print "intisane: Can't open scanner"
        else:
            print "intisane: Scanner number is out of range"
                
    def combobox_mode_changed_cb(self, obj):
        "combobox_mode_changed_cb activated"
        mode_number = obj.get_active()
        self.config.set('settings', 'mode', str(mode_number))
    
    def spinbutton_contrast_changed_cb(self, obj, val = None):
        "spinbutton_contrast_changed_cb activated"
        contrast = obj.get_value()
        self.config.set('settings', 'contrast', str(contrast))

    def spinbutton_brightness_changed_cb(self, obj, val = None):
        "spinbutton_brightness_changed_cb activated"
        brightness = obj.get_value()
        self.config.set('settings', 'brightness', str(brightness))

    def spinbutton_resolution_changed_cb(self, obj, val = None):
        "spinbutton_resolution_changed_cb activated"
        resolution = obj.get_value()
        self.config.set('settings', 'resolution', str(resolution)) 
    
    def doImport(self):
        im = self.scan()
        album = self.entry_album.get_active_text().strip()
        if album == _("Select or insert album").strip():
            album = _("Scanned")
            
        albumpath = self.config["photo_folder"] + os.sep + album
        im_name = self.entry_name.get_text().strip()
        dir_util.mkpath(albumpath)
        im.save(albumpath + os.sep + im_name + ".jpg", "JPEG")
        
        # save settings
        
        self.config["scan_mode"] = self.entry_mode.get_active()
        self.config["scan_res"] = self.entry_res.get_value()
        self.config["scan_brightness"] = self.entry_brightness.get_value()
        self.config["scan_contrast"] = self.entry_contrast.get_value()
        self.config.save()
        
    
    def scan(self):
        "button_scan_clicked_cb activated"
        # open scanner
        sane.init()
        scanner_number = self.entry_scanner.get_active()
        scanner = sane.open(self.sane_devices[scanner_number][0])
        
        # options
        options = get_scanner_option (scanner, 'mode')
        if options:
            scanner.mode = options[8][self.entry_mode.get_active()]
        try:
            scanner.resolution = int(self.entry_res.get_value())
        except AttributeError:
            print "intisane: Can't set scan resolution"
        try:
            scanner.brightness = int(self.entry_brightness.get_value())
        except AttributeError:
            print "intisane: Can't set scan brightness"
        try:
            scanner.contrast = int(self.entry_contrast.get_value())
        except AttributeError:
            print "intisane: Can't set scan contrast"
        try:
              scanner.preview = 0
        except AttributeError:
              print "intisane: Can't set scan preview"
        
        # geometry
        try:
            # Get selection
            rect = self.stool.get_selection()
            x,y,w,h = rect.x, rect.y, rect.width, rect.height
            x2 = x+w
            y2 = y+h
            
            # Get options from scanner
            options = get_scanner_option (scanner, 'br-x')
            x_max = options[8][1]
            options = get_scanner_option (scanner, 'br-y')
            y_max = options[8][1]
            
            # Get size of pixbuf
            pixbuf = self.draw.get_pixbuf()
            x_p = pixbuf.get_width()
            y_p = pixbuf.get_height()
            
            
            
            scanner.tl_x = (x*x_max)/x_p
            scanner.tl_y = (y*y_max)/y_p
            scanner.br_x = (x2*x_max)/x_p
            scanner.br_y = (y2*y_max)/y_p
        except AttributeError, err:
            print "intisane: Can't set scan geometry: ", err
        
        #scan
        try:
            pil_image = scanner.scan()
            #pil_image.save(self.entry_filename.get_text())
            
        except sane._sane.error, err:
            print "intisane: Can't scan"
            utils.report_error(_("Can't scan,\n%s,\nCheck scanner settings") % err)
            pil_image = None
        
        # close scanner
        scanner.close()
        sane.exit()
        
        return pil_image
          
        
#if __name__ == "__main__":

    #win = IntiSane()
    #r = win.run()
    #im = win.scan()
    #win.destroy()
    #gtk.main()
        
