#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   This file is part of IntiPunku
#   Copyright (C) 2007 Mark Baas <mark.baas123@gmail.com>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pyexiv2 as exiv2
import os, time, datetime
from inti.constants import *

def get_tags(taim):
    tags = []
    if type(taim['Iptc.Application2.Keywords']) == type(""): 
        if taim['Iptc.Application2.Keywords'].strip() != "":
            tags = [taim['Iptc.Application2.Keywords']]
    else: 
        for key in taim['Iptc.Application2.Keywords']:
            if key.strip() != "":
                tags.append(key)
    print tags
    return tags

def get_datetimestamp(path):
    statinfo = os.stat(path)
    date = datetime.datetime.fromtimestamp(statinfo.st_mtime)
    return date

def get_datetags(path):
    if os.path.splitext(path)[1] in EXT_EXIV2:
        image = exiv2.Image(path)
        image.readMetadata()
        if "Exif.Image.DateTime" in image.exifKeys():
            datetimetag = "Exif.Image.DateTime"
        else: datetimetag = "Exif.Photo.DateTimeOriginal"
        if datetimetag in image.exifKeys():
            idatetime = image[datetimetag]
            if type(idatetime) == type(str()):
                date = get_datetimestamp(path)
            else:
                date = idatetime
             
        else:
            date = get_datetimestamp(path)
        if 'Iptc.Application2.Keywords' in image.iptcKeys():
            tags = get_tags(image)
        else: tags = []
        
    else:
        date = get_datetimestamp(path)
        tags = []

    return date, tags

def set_date(path, dt):
    # exiv2 cannot write (correctly) other than jpeg and tiff, so elsewise we modify modification date
    ext = os.path.splitext(path)[1].lower()
    if ext in EXT_EXIV2:
        image = exiv2.Image(path)
        image.readMetadata()
        image["Exif.Image.DateTime"] = dt
        image["Exif.Image.Software"] = "IntiPunku"
        image.writeMetadata()
    else:
        timestamp = time.mktime(dt.timetuple())
        os.utime(path, (-1, timestamp))    

def has_exifdatetime(path):
    image = exiv2.Image(path)
    image.readMetadata()
    keys = image.exifKeys()
    if "Exif.Image.DateTime" in keys or "Exif.Photo.DateTimeOriginal" in keys:
        return True
    return False
    
    
def copy_metadata(path1, path2=None):
    # path2 is none, cache the keys
    if not os.path.splitext(path1)[1] in EXT_EXIV2: # only timestamp
        return get_datetimestamp(path1)
            
    image = exiv2.Image(path1)
    image.readMetadata()
    if path2 == None:
        image["Exif.Image.Software"] = "IntiPunku"
        if not "Exif.Image.DateTime" in image.exifKeys(): 
            date = get_datetags(path1)[0]
            print "no date tag, adding"
            image["Exif.Image.DateTime"] = date
        return image
    else:
        image2 = exiv2.Image(path2)
        image2.readMetadata()
        image.copyMetadataTo(image2)
        image2.writeMetadata()
        return None
        
def paste_metadata(image, path):
    if not os.path.splitext(path)[1] in EXT_EXIV2: 
        set_date(path, image)
        return
    # We have to re-read metadata!
    image2 = exiv2.Image(path)
    image2.readMetadata()
    image.copyMetadataTo(image2)
    image2.writeMetadata()
    
    


def write_tags(path, tags):
    image = exiv2.Image(path)
    image.readMetadata()
    image["Iptc.Application2.Keywords"] = tags
    image["Iptc.Application2.Program"] = "Intipunku"
    image["Iptc.Application2.ProgramVersion"] = str(VERSION)
    image.writeMetadata()

def set_orientation(path, i):
    image = exiv2.Image(path)
    image.readMetadata()
    image["Exif.Image.Orientation"] = i
    image["Iptc.Application2.Program"] = "Intipunku"
    image["Iptc.Application2.ProgramVersion"] = str(VERSION)
    image.writeMetadata()
    
def get_orientation(path):
    image = exiv2.Image(path)
    image.readMetadata()
    if "Exif.Image.Orientation" in image.exifKeys():
        return int(image["Exif.Image.Orientation"])
    else:
        return 1
        
def reload_tags(tagindex):
    n = len(tagindex)
    for im in tagindex:
        while gtk.events_pending(): 
            gtk.main_iteration() 
        image = exiv2.Image(im)
        image.readMetadata()
        if 'Iptc.Application2.Keywords' in image.iptcKeys():
            get_tags(image)
        else: tags = []
        tagindex[im] = tags

    return tagindex
