#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   This file is part of IntiPunku
#   Copyright (C) 2007 Mark Baas <mark.baas123@gmail.com>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
# Optional components are pano, gdata, gphoto2
# There should be made wrappers for this

import inti.intigtk as gtk
import os, os.path
import tempfile
import utils
import dialogs
# Localization
import gettext, locale
_ = gettext.gettext

optionals = {
    "gdata" : [
            { 
                "apt" : "python-gdata python-gconf python-gnome2-desktop",
                "yum" : "python-gdata gnome-python2-gnomekeyring gnome-python2-gconf",
            },
            ],
    "gphoto" : [
            {
                "apt" : "python-gphoto",
                "yum" : "",
            },
            ],
    "pano" : [
            {
                "apt" : "hugin-tools enblend panomatic",
                "yum" : "hugin-base enblend panomatic",
            },
            ],
    "flickr" : [
            {
                "apt" : "python-flickrapi python-gconf python-gnome2-desktop",
                "yum" : "python-flickrapi gnome-python2-gconf gnome-python2-gnomekeyring",
            },
            ],
    "ming" : [
            {
                "apt" : "python-ming",
            },
            ],
    "video" : [
            {
                "apt" : "python-vlc",
            }
            ],
    "email" : [
            {
                "apt" : "",
            }
            ],
    "exiv2" : [
            {
                "apt" : "python-pyexiv2",
            }
            ],
    "pygtkimageview" : [
            {
                "apt" : "python-gtkimageview",
            }
            ],   
    "scanner" : [
            {
                "apt" : "python-imaging-sane",
            }
            ],
    "raw" : [
            {
                "apt" : "dcraw",
            }
            ],
    "jpegtran" : [
            {
                "apt" : "libjpeg-progs",
            }
            ],
}

if os.path.exists("/etc/debian_version"): 
    system = "apt"
elif os.path.exists("/etc/redhat-release"): 
    system = "yum"
else: system = None

# Unsupported package manager
def unsupported(module):
  
    dialog = gtk.MessageDialog(flags=gtk.DIALOG_MODAL, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK)
    dialog.set_markup(_("Sorry your package manager is unsupported.\nPlease install the follow package manually '%s'" % (module)))
    dialog.set_title(_("Package manager unsupported"))
    r = dialog.run()
    dialog.destroy()  
    
    
def check_repo():
	if system == "apt":
	    repo = ""
	    f = open("/etc/apt/sources.list")
	    repo += f.read()
	    f.close()
	    for r in os.listdir("/etc/apt/sources.list.d"):
	        f = open("/etc/apt/sources.list.d/"+r)
	        repo += f.read()
	        f.close()
	
	    # check distro, default = hardy
	    release = os.popen("lsb_release -cs").read().strip()
	    mint = { "Gloria" : "jaunty",
	             "Felicia" : "intrepid",
	             "Elyssa" : "hardy",
	            }
	    if mint.has_key(release): release = mint[release]
	    aptline = "deb http://ppa.launchpad.net/intipunku/intipunku-%s/ubuntu %s main" % (release, release)
	
	    if not aptline in repo:       
	        prog = "f = open(\"/etc/apt/sources.list.d/intipunku.list\", \"w\")\nf.write(\"%s\")\nf.close()" % (aptline)
	        f = tempfile.mkstemp(prefix="intiapt-")[1]
	        fd = open(f, "w")
	        fd.write(prog)
	        fd.close()
	        m = _("<b>Intipunku</b> needs to add the intipunku repository to the APT sources")     
	        os.system("gksudo -m \"%s\" python %s" % (m, f))
	        os.remove(f)
	        
	        # import gpg key
	        os.system("gksudo -m \"%s\" apt-key adv --keyserver keyserver.ubuntu.com --recv-keys B89A4B25" % _("Intipunku needs to import the ppa key"))
	        
	        progress = dialogs.SunProgress(_("Updating apt database"), icon="deb")
	        progress.start(5)
	        progress.pulse()
	        line = "gksudo -m \"%s\" apt-get update" % _("<b>Intipunku</b> needs to update the apt database")
	        utils.sunthread(os.system, args=(line,))
	        progress.destroy()
	    return True
        
#def check_repo():
#    if system == "apt":
#        f = open("/etc/apt/sources.list")
#        lines = f.readlines()
#        f.close()
#        repo = "deb http://ppa.launchpad.net/intipunku/ubuntu hardy main"
#        for l in lines:
#            if l.startswith(repo):
#                return True
#        if os.path.exists("/etc/apt/source.list.d/intipunku.list"): return True
#        # Repo no added
#        dialog = gtk.MessageDialog(flags=gtk.DIALOG_MODAL, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK)
#        dialog.set_markup(_("Sorry the intipunku apt repository is not added to you /etc/apt/sources.list\n\nPlease add the following lines to /etc/apt/sources.list:\n\n<b>%s</b>" % (repo)))
#        dialog.set_title(_("APT repo not installed"))
#        r = dialog.run()
#        dialog.destroy()  
            

# Install module and reload
#def install(module):
#    if cmd == None:
#        
#    if not check_repo(): return False
#    
#    
#    package = optionals[module][0][system]
#    cmd2 = cmd.replace("PACKAGE",package)
#    progress = utils.SunProgress(_("Installing packages please wait ..."), icon="package")
#    progress.start(1)
#    progress.pulse()
#    utils.sunthread(os.system, (cmd2,))    
#    progress.destroy()
#    return True
    
def install(modules):
    if not check_repo(): return False
    
    if system == "apt":
        packlist = ""
        for module in modules:
            for pack in optionals[module][0][system].split():
                packlist += "%s    install\n" % (pack)
        packlist = packlist.strip()
        f = tempfile.mkstemp(prefix="intipacklist-")[1]
        fd = open(f, "w")
        fd.write(packlist)
        fd.close()
        f2 = tempfile.mkstemp(prefix="intiapt-")[1]
        fd = open(f2, "w")
        fd.write("synaptic --hide-main-window --non-interactive --set-selections < %s" % (f))
        fd.close()
        print f2
        if utils.inpath("gksudo"): sudo = "gksudo -m"
        elif utils.inpath("kdesudo"): sudo = "kdesudo --comment"
        os.system("%s \"%s\" sh %s" % (sudo, _("<b>Intipunku</b> needs your permission to install necessary packages"), f2))
        #os.remove(f)
        #os.remove(f2)
    else:
        module = ""
        for m in modules:
            module += m + " "
        unsupported(module.strip())
        return False

# Check whether to install module
def question(c):
    dialog = gtk.MessageDialog(flags=gtk.DIALOG_MODAL, type=gtk.MESSAGE_QUESTION, buttons=gtk.BUTTONS_OK_CANCEL)
    dialog.set_markup(_("This component depends on external resources, which are not installed.\nWould you like to install dependencies to enable this feature?"))
    dialog.set_title(_("Component disabled!"))
    r = dialog.run()
    dialog.destroy()
    if r == -5:
        return True
    return False
    


def check(c, callback, constant, q=True):
    while not constant:
        if q:
            r = question(c)       
        else:
            r = True
        if r:
            r2 = install([c])
            if r2 == False:
                return False
            constant = callback()
        else:
            return False
    return True 
    
    



            



