#!/usr/bin/env python
#
#   This file is part of IntiPunku
#   Copyright (C) 2007 Mark Baas <mark.baas123@gmail.com>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import inti.intigtk as gtk, gobject, time, math, pango, os.path
from PIL import Image
from inti import utils, tools
from inti.constants import *

# Localization
import gettext
_ = gettext.gettext

class GtkSlide(gtk.DrawingArea):
    def __init__(self, win):
        gtk.DrawingArea.__init__(self)
        
             
        self.connect("expose-event", self.draw_expose)
        self.connect("configure-event", self.draw_configure)
        self.current = None
        self.pixmap = None
        self.eventarea = None
        self.pixbuf = None
        self.anim_count = -1
        self.win = win
        #gobject.timeout_add(10, self.anim)
        
    def draw_configure(self, widget, event):
        while gtk.events_pending(): gtk.main_iteration()
        w = self.get_allocation().width
        h = self.get_allocation().height
        
        if w < 10 or h < 10:
            # We cannot put a picture here
            return
        
        self.pwidth = w
        self.pheight = h
        self.pixmap = gtk.gdk.Pixmap(self.window, w, h)
        
        # Draw black background
        cr = self.pixmap.cairo_create()
        cr.set_source_rgb(0, 0, 0)
        cr.rectangle(0, 0, 1, 1)
        cr.fill()
        cr.paint()
        
        # reload pixbuf
        if self.pixbuf: self.load(self.current, anim=False)
        
            
    
    def load(self, f, anim=True):

        while self.anim_count != -1:
            return
            
        # load pil to handle orientation
        im = Image.open(f)
        im = tools.handle_orientation(im, f)
        
        # Load new pixbuf
        if self.current != f:
            if WIN: f = unicode(f, "latin-1")
            self.pixbuf = tools.pil2pixbuf(im)
            self.pixbufo = tools.pil2pixbuf(im)
        else:
            self.pixbuf = self.pixbufo
         
        # remove pil to clean memory   
        del im
        
        self.current = f
                    
        # Resize to screen width
        w = self.pixbuf.get_width()
        h = self.pixbuf.get_height()
        
        if self.pwidth*h/w < self.pheight: 
            hn = self.pwidth*h/w
            wn = self.pwidth
        elif self.pheight*w/h < self.pwidth:
            wn = self.pheight*w/h
            hn = self.pheight
        
        self.pixbuf = self.pixbuf.scale_simple(wn, hn, gtk.gdk.INTERP_BILINEAR)

        # Calculate margins
        self.margin_x = (self.pwidth-wn)/2
        self.margin_y = (self.pheight-hn)/2

        # Start transition
        if anim:
            self.anim_count = 0
            while self.anim_count <= 20:
                self.anim()
                
                time.sleep(0.05)
            self.anim_count = -1     
        else:
            self.draw_pixmap(alpha=False)
            self.emit("expose-event", None) 
                
        
    def anim(self):
        if self.anim_count > -1:
            
            if self.anim_count >= 20:              
                self.draw_pixmap(alpha=False)
                self.emit("expose-event", None)
                
            else:
                self.draw_pixmap()
                self.emit("expose-event", None)
            self.anim_count += 1
        return True
        
    def draw_pixmap(self, alpha=True):
        cr = self.pixmap.cairo_create()
        
        # Black bars
        w = self.pixbuf.get_width()
        h = self.pixbuf.get_height()
        
        if alpha: cr.set_source_rgba(0, 0, 0, 0.2)
        else: cr.set_source_rgb(0, 0, 0)
                
        cr.rectangle(0,0, self.margin_x, h)
        cr.fill()
        
        cr.rectangle(w+self.margin_x,0, w, h)
        cr.fill()
              
        # Draw the image
        cr.set_source_pixbuf(self.pixbuf,self.margin_x, self.margin_y) 
        if alpha:
            cr.paint_with_alpha(0.2)
        else:
            cr.paint()

    def draw_expose(self, widget, event):
        if self.pixmap:
            if event:
                self.eventarea = event.area
            
            x , y, width, height = self.eventarea
            widget.window.draw_drawable(widget.get_style().fg_gc[gtk.STATE_NORMAL], self.pixmap, x, y, x, y, width, height)
            
    def sunset(self):
        del self.pixmap
        del self.pixbufo
        del self.pixbuf
        self.destroy()

class Slidewindow(gtk.Window):
    def __init__(self, files, config=None, select=0):
        gtk.Window.__init__(self)
              
        # Set up VBox
        self.vbox = gtk.VBox()
        self.add(self.vbox)
        
        # Add slider
        self.draw = GtkSlide(self)
        self.vbox.pack_start(self.draw)
        
        # Add Control Bar
        hbox = gtk.HBox()
        hbox.set_border_width(2)
        
        self.vbox.pack_end(hbox, fill=False, expand=False)
        
        # Buttons
        hbuttons = gtk.HButtonBox()
        hbox.pack_start(hbuttons, expand=False, fill=False)
        self.controls = hbox
        
        b_forward = gtk.Button()
        b_forward.connect("clicked", self.next_image)
        b_forward.set_relief(gtk.RELIEF_NONE)
        b_forward.set_image(gtk.image_new_from_stock(gtk.STOCK_GO_FORWARD, gtk.ICON_SIZE_MENU))
       
        b_back = gtk.Button()
        b_back.connect("clicked", self.prev_image)
        b_back.set_relief(gtk.RELIEF_NONE)
        b_back.set_image(gtk.image_new_from_stock(gtk.STOCK_GO_BACK, gtk.ICON_SIZE_MENU))
        
        b_first = gtk.Button()
        b_first.connect("clicked", self.first_image)
        b_first.set_relief(gtk.RELIEF_NONE)
        b_first.set_image(gtk.image_new_from_stock(gtk.STOCK_GOTO_FIRST, gtk.ICON_SIZE_MENU))
        
        b_last = gtk.Button()
        b_last.connect("clicked", self.last_image)
        b_last.set_relief(gtk.RELIEF_NONE)
        b_last.set_image(gtk.image_new_from_stock(gtk.STOCK_GOTO_LAST, gtk.ICON_SIZE_MENU))
        
        hbox.pack_start(b_first, False, False)
        hbox.pack_start(b_back, False, False)
        hbox.pack_start(b_forward, False, False)
        hbox.pack_start(b_last, False, False)
        separator = gtk.VSeparator()
        separator.set_size_request(20,10)
        hbox.pack_start(separator, False, False)
        
        # Settings widgets
        autonext = gtk.CheckButton(label=_("Change photos every")+" ")
        autonext.connect("toggled", self.autonext_changed)
        autointerval = gtk.SpinButton(climb_rate=1)
        autointerval.set_increments(1, 10)
        autointerval.set_range(2, 1000)
        autointerval.connect("value-changed", self.autointerval_changed)
        autohide = gtk.CheckButton(label=_("Hide controls"))
        autohide.connect("toggled", self.autohide_changed)
        b_leave = gtk.Button(stock=gtk.STOCK_LEAVE_FULLSCREEN)
        b_leave.connect("clicked", self.sunset)
        
        hbox.pack_start(autonext, False, False)
        hbox.pack_start(autointerval, False, False)
        hbox.pack_start(gtk.Label(" "+_("seconds")), False, False)
        separator = gtk.VSeparator()
        separator.set_size_request(20,10)
        hbox.pack_start(separator, False, False)
        hbox.pack_start(autohide, False, False)

        self.label = gtk.Label()
        self.label.set_ellipsize(pango.ELLIPSIZE_MIDDLE)
        hbox.pack_start(self.label)

        hbox.pack_end(b_leave, False, False)
             
        self.files = files
        self.sunset = False
        self.config = config
        
        self.autobuttons = [autointerval]
        for b in self.autobuttons:
            b.set_sensitive(autonext.get_active())
                
        # settings
        if config:
            self.autonext = self.config["slide_auto"]
            self.autotimeout = self.config["slide_timeout"]
            self.autohide = self.config["slide_hide"]
        else:
            self.autohide = True
            self.autonext = False
            self.autotimeout = 5
        
        autonext.set_active(self.autonext)
        autointerval.set_value(self.autotimeout)
        autohide.set_active(self.autohide)
        
        

            
        self.autotimer = 0
        self.motiontimer = 0
        
        self.add_events(gtk.gdk.POINTER_MOTION_MASK | gtk.gdk.SCROLL_MASK)
        self.connect("motion-notify-event", self.motion)
        
        # Load first image
        gobject.timeout_add(2000, self.load, files[select])
        
        # Timer 
        gobject.timeout_add(1000, self.timer)
        
    def load(self, f):
        total = len(self.files)
        n = self.files.index(f)+1
        bf = os.path.basename(f)
        self.label.set_markup("<b>(%s/%s)</b> %s" % (n,total,bf))
        
        self.current = f
        while gtk.events_pending(): gtk.main_iteration()
        #print "sliding:", f
        self.draw.load(f)
        
    def next_image(self, widget=None):
        n = self.files.index(self.current)+1
        if n > len(self.files)-1:
            n = 0
        self.load(self.files[n])
        
    def prev_image(self, widget=None):
        n = self.files.index(self.current)-1
        if n < 0:
            n = len(self.files)-1
        self.load(self.files[n])
        
    def last_image(self, widget=None):
        self.load(self.files[-1])

    def first_image(self, widget=None):
        self.load(self.files[0])
               
    def autointerval_changed(self, widget):
        self.autotimeout = widget.get_value()
        
    def autonext_changed(self, widget):
        self.autonext = widget.get_active()
        
        for b in self.autobuttons:
            b.set_sensitive(widget.get_active())
        
    def autohide_changed(self, widget):
        self.autohide = widget.get_active()
        self.motiontimer = 0 
    
    def mouse_hide(self):
        pix_data = """/* XPM */
        static char * invisible_xpm[] = {
        "1 1 1 1",
        "       c None",
        " "};"""
        color = gtk.gdk.Color()
        pix = gtk.gdk.pixmap_create_from_data(None, pix_data, 1, 1, 1, color, color)
        invisible = gtk.gdk.Cursor(pix, pix, color, color, 0, 0)

        self.draw.window.set_cursor(invisible)
        
    def mouse_show(self):
        self.draw.window.set_cursor(None)
 
    def timer(self):
        if self.sunset: return False
        
        if self.autonext: 
            self.autotimer += 1
            if self.autotimer == self.autotimeout:
                self.next_image()
                self.autotimer = 0
        if self.autohide:        
            self.motiontimer += 1
        if self.motiontimer == 5:          
            self.controls.hide()
            self.mouse_hide()
            #self.draw.draw_pixmap()
            self.draw.emit("expose-event", None)
            
            
        return True
        
    def motion(self, *args):
        if self.motiontimer > 5:
            self.controls.show()
            self.mouse_show()
        self.autotimer = 0  
        self.motiontimer = 0
    
    def sunset(self, widget=None):
        if self.config:
            self.config["slide_auto"] = self.autonext
            self.config["slide_timeout"] = int(self.autotimeout)
            self.config["slide_hide"] = self.autohide
            self.config.save()
        self.draw.sunset()
        self.sunset = True
        self.destroy()
        
       
        
        




if __name__ == "__main__":
    import sys, os
    images = []
    for f in os.listdir(sys.argv[1]):
        if f.endswith(".jpg"):
            images.append(sys.argv[1]+os.sep+f)
    images.sort()
    win = Slidewindow(images)
    win.show_all()
    #win.fullscreen()
   
    gtk.main()
