#!/usr/bin/env python
#
#   This file is part of IntiPunku
#   Copyright (C) 2007 Mark Baas <mark.baas123@gmail.com>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys, os

def load_ming():
    try:
        import ming
        global ming
        return True
    except:
        return False

MING_ENABLED = load_ming()
from inti import tools
import tempfile

def easeInQuad(t, b, c, d):
    t/=float(d)
    return c * t * t + b

def easeOutQuad(t, b, c, d):
    t/=float(d)
    return -c * (t) * (t-2) + b
    
def create_swfslide(imgs, output):
    # Resize all jpegs
    jpegs = []
    for img in imgs:
        f = tempfile.mkstemp(prefix="intiswf-")[1]+".jpg"
        tools.resize_image(img, f, (640,480))
        jpegs.append(f)

    # some typical movie variables
    ming.Ming_setScale(20.0000000);
    ming.Ming_useSWFVersion(6);
    movie = ming.SWFMovie();
    movie.setRate(31); 
    movie.setDimension(580,420);

    # basic actionscript control of playback using mouse
    strAction="""
    if(!init){
        init=true;
        stopped=false;
        controls = {
	        onMouseDown: function () {
		        if(!stopped){
			        stop();
			        stopped=true;
		        }else{
			        play();
			        stopped=false;
		        }
	        }
        };
        Mouse.addListener(controls);
    }
    """
    movie.add(ming.SWFAction(strAction))
    
    jpegs.sort()

    # add the jpgs to the movie with basic fade in/out
    movie.nextFrame();
    for jpeg in jpegs:
        img = ming.SWFBitmap(jpeg)
        pic = movie.add(img)
        pic.moveTo((275-img.getWidth()/2),(200-img.getHeight()/2))
        transition = 20
        cnt = 1
        startpos = 0
        offset = 1
        duration = transition
        while cnt <= duration:
            cnt += 1
            inc = easeInQuad(cnt, startpos, offset, duration)
            
            pic.multColor(1,1,1,inc)
            movie.nextFrame()
        j = 1
        while j <= transition*2:
            j += 1
            movie.nextFrame()
        cnt = 1
        startpos = 1
        offset = -1
        duration = transition
        while cnt <= duration:
            cnt += 1
            inc = easeOutQuad(cnt, startpos, offset, duration)
            pic.multColor(1,1,1,inc)
            movie.nextFrame()
        movie.remove(pic)
    movie.nextFrame()
    
    # save swf
    movie.save(output)
    
    # remove tempory files
    for jpeg in jpegs:
        os.remove(jpeg)
    print "swfexport: done"
    
        
if __name__ == "__main__":
    image1 = sys.argv[1]
    image2 = sys.argv[2]
    create_swfslide([image1, image2], "flash.swf")
    
