#!/usr/bin/env python
#
#   This file is part of IntiPunku
#   Copyright (C) 2007 Mark Baas <mark.baas123@gmail.com>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import inti.intigtk as gtk

# Localization
import gettext
_ = gettext.gettext

class SunTray(gtk.ScrolledWindow):
    def __init__(self, win):
        gtk.ScrolledWindow.__init__(self)

        self.table = gtk.Table()
        self.add_with_viewport(self.table)
        self.win = win

        self.selection = []
        self.ypos = 0
        self.xpos = 0
        self.win.connect("size-allocate", self.checksize_cb)
        self.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        self.show_all()

    def add(self, im):
        pixbuf = gtk.gdk.pixbuf_new_from_file(im)
        pixbuf = pixbuf.scale_simple(25,25, gtk.gdk.INTERP_NEAREST)
        gim = gtk.Image()
        gim.set_from_pixbuf(pixbuf)
        gim.set_name(im)
        gim.show()
        self.selection.append(gim)
        xpos = self.xpos
        ypos = self.ypos
        self.table.attach(gim,xpos,xpos+1,ypos,ypos+1)
        self.xpos += 1
        if self.xpos == 5:
            self.xpos = 0
            self.ypos += 1

    def remove(self, im):
        selection = []
        for gim in self.selection:
            if gim.get_name() != im:
                selection.append(gim.get_name())
        self.clear()
        for im in selection:
            self.add(im)
                
    def clear(self):
        for gim in self.selection:
            gim.destroy()
        self.selection = []
        self.xpos = 0
        self.ypos = 0

    def checksize_cb(self, *args):
        win_height = self.win.get_allocation().height
        win_width = self.win.get_allocation().width
        self.set_size_request(int(float(win_width)/6),int(float(win_height)/9))
        return True
