#! /usr/bin/python

"""VLC Widget classes.

This module provides two helper classes, to ease the embedding of a
VLC component inside a pygtk application.

VLCWidget is a simple VLC widget.

DecoratedVLCWidget provides simple player controls.

$Id$
"""

import gtk
import sys
import vlc
import gobject
from PIL import Image
import StringIO
import time
import tempfile
import os

from gettext import gettext as _

class VLCWidget(gtk.DrawingArea):
    """Simple VLC widget.

    Its player can be controlled through the 'player' attribute, which
    is a MediaControl instance.
    """
    def __init__(self, *p):
        gtk.DrawingArea.__init__(self)
        self.player=vlc.MediaControl(*p)
        def handle_embed(*p):
            if sys.platform == 'win32':
                xidattr='handle'
            else:
                xidattr='xid'
            self.player.set_visual(getattr(self.window, xidattr))
            return True
        self.connect("map-event", handle_embed)
        self.set_size_request(10, 10)
#        
#        gobject.timeout_add(5000, self.capture)
#        
#    def capture(self):
#        drawable = self.window
#        w, h = drawable.get_size()
#        pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, False, 8, w, h)
#        pixbuf.get_from_drawable(drawable, drawable.get_colormap(), 0, 0, 0, 0, w, h)
#        pixbuf.save("thumb.jpg", "jpeg", {"quality":"100"})



class DecoratedVLCWidget(gtk.VBox):
    """Decorated VLC widget.

    VLC widget decorated with a player control toolbar.

    Its player can be controlled through the 'player' attribute, which
    is a MediaControl instance.
    """
    def __init__(self, *p):
        gtk.VBox.__init__(self)
        self._vlc_widget=VLCWidget(*p)
        self.player=self._vlc_widget.player
        self.pack_start(self._vlc_widget, expand=True)
        self._toolbar = self.get_player_control_toolbar()
        self.pack_start(self._toolbar, expand=False)
        self.player.sound_set_volume(50)

    def get_player_control_toolbar(self):
        """Return a player control toolbar
        """
        tb=gtk.Toolbar()
        tb.set_style(gtk.TOOLBAR_ICONS)

        def on_play(b):
            self.player.start(0)
            return True

        def on_stop(b):
            self.player.stop(0)
            return True

        def on_pause(b):
            self.player.pause(0)
            return True

        tb_list = (
            (_("Play"), _("Play"), gtk.STOCK_MEDIA_PLAY,
             on_play),
            (_("Pause"), _("Pause"), gtk.STOCK_MEDIA_PAUSE,
             on_pause),
            (_("Stop"), _("Stop"), gtk.STOCK_MEDIA_STOP,
             on_stop),
            )

        for text, tooltip, stock, callback in tb_list:
            b=gtk.ToolButton(stock)
            b.connect("clicked", callback)
            tb.insert(b, -1)
        tb.show_all()
        return tb

class VideoPlayer:
    """Example video player.
    """
    def __init__(self):
        self.vlc = DecoratedVLCWidget()

    def main(self, fname):
        self.vlc.player.set_mrl(fname)
        self.popup()
        gtk.main()

    def popup(self):
        w=gtk.Window()
        w.add(self.vlc)
        w.show_all()
        w.connect("destroy", gtk.main_quit)
        return w

class Thumbnailer:
    def __init__(self, fname):
        self.f = tempfile.gettempdir() + os.sep + "intiv"
        self.movie = fname
        
    def get_thumb(self):
        #print "cvlc -V image --start-time 4 --stop-time 5 --image-out-format png --image-out-ratio 24 --image-out-prefix %s --image-out-replace %s --no-audio vlc://quit" % (self.f, self.movie)
        os.popen("cvlc -V image --start-time 4 --stop-time 5 --image-out-format png --image-out-ratio 24 --image-out-prefix \"%s\" --image-out-replace \"%s\" --no-audio vlc://quit 2> /dev/null" % (self.f, self.movie))
        
        if not os.path.exists(self.f + ".png"): 
         
            return None
        im = Image.open(self.f + ".png")

        return im
        
    def destroy(self):
        os.remove(self.f+".png")
        

# this code sucks, so we do it by commandline      
class Thumbnailer_sucked:
    def __init__(self, fname):
        print "class"
        self.vlc = DecoratedVLCWidget()
        self.f = fname
        print "blabla"
        
    def get_thumb(self):
        self.vlc.player.set_mrl(self.f)
        self.popup()
        self.im = None
        gobject.timeout_add(10, self.start)
        while self.im == None:
            while gtk.events_pending(): 
		        gtk.main_iteration() 
        return self.im
        
        
    def start(self):
        self.vlc.player.sound_set_volume(0)
        self.vlc.player.start()
        time.sleep(1)      
        
        png = self.vlc.player.snapshot(3)
        f = StringIO.StringIO(png["data"])       
        im = Image.open(f)
#        im.save("thumb.png")
        self.im = im.copy()
        f.close()
        print "saved"
        self.vlc.player.stop()
        self.vlc.destroy()
        

    def popup(self):
        w=gtk.Window()
        w.set_property("skip-taskbar-hint", True)
        w.add(self.vlc)
        w.show_all()
        w.set_keep_below(True)
        self.win = w
        w.connect("destroy", gtk.main_quit)
        return w

if __name__ == '__main__':
    import time
    if not sys.argv[1:]:
       print "You must provide a movie filename"
       sys.exit(1)
    p=Thumbnailer(sys.argv[1])
    print p.main()
    #p.main()
    #p = VideoPlayer()
    #p.main(sys.argv[1])
    gtk.main()
    
