#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   This file is part of IntiPunku
#   Copyright (C) 2007 Mark Baas <mark.baas123@gmail.com>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import gobject
import time
from datetime import datetime
import os, os.path
from inti import dialogs
from inti import utils
from inti import metadata
import sqlite3
from inti.constants import *
from inti import tools
from distutils import dir_util
from distutils import file_util

# Localization
import gettext, locale
_ = gettext.gettext

class sunindex(dict):
    def __init__(self, call, type):
        self.call = call
        self.type = type
        self.updated = time.time()
        gobject.timeout_add(1000, self.update)

    def update(self, index=None):
        if not index:
            try: index = self.call(self.type)
            except: return
        if self != index:
            for f in index:
                self[f] = index[f]
            todelete = []
            for f in self:
                if not index.has_key(f):
                    todelete.append(f)
            for f in todelete:
                del self[f]
            print "sunindex: updated %sindex" % self.type
            self.updated = time.time()
        return True

class SunWatcher:
    def __init__(self, albumdir, cdir, version):
        self.cachedir = cdir
        albumdir = os.path.abspath(albumdir)
              
        t1 = time.time()
        self.watcher_stop = False
        self.hold_thread = False
        self.skiplist = []
        self.fixlist = []

        # Starting splash
        splash = dialogs.SunSplash("IntiPunku",version)
        splash.show()

        
        # Checking self.cachedir
        idxfile = self.cachedir + os.sep + "index.db"
        if not os.path.exists(self.cachedir):
            dir_util.mkpath(self.cachedir)
        index = sunindex(self.get_index, "im")
        dindex = sunindex(self.get_index, "d")
        tagindex = sunindex(self.get_index, "tag")
        self.mindex = {}
        index_thumbs = []
        if os.path.exists(idxfile):             
            sqlconn = sqlite3.connect(idxfile)
            sqlc = sqlconn.cursor()
            sqlc.execute('SELECT * FROM inticache')
            for row in sqlc:
                if WIN: path = row[0].encode("latin-1")
                else: path = str(row[0])

                # To check the file, it should exist and the dirname of dirname (albumpath) should be albumpath
                if os.path.exists(path) and os.path.abspath(os.path.dirname(os.path.dirname(path))) == albumdir:
                    thumb = str(row[1])
                    
                    # Check if it is from old version
                    if os.path.basename(thumb).startswith("SOLTHUMB_"): continue
                    if not os.path.exists(thumb): continue
                    
                    dt = str(row[2]).split(";")
                    date = datetime(int(dt[0]),int(dt[1]),int(dt[2]),int(dt[3]),int(dt[4]),int(dt[5]))
                    tags = str(row[3])
                    mtime = str(row[4])
                    self.mindex[path] = float(mtime)
                    index[path] = thumb
                    dindex[path] = date
                    if tags.strip() != "":
                        tagindex[path] = tags.split(";")
                    else: tagindex[path] = []
                    index_thumbs.append(thumb)
                
        # Checking for unindex thumbs
        removethumbs = []
        for thumb in os.listdir(self.cachedir):
            if thumb.endswith(".png"):
                thumb_path = self.cachedir + os.sep + thumb
                if not thumb_path in index_thumbs:
                    removethumbs.append(self.cachedir + os.sep + thumb)

        if len(removethumbs) > 0:
            for thumb in removethumbs:   
                os.remove(thumb)              
                

        # Checking albumdir
        albums = []
        total_n_files = 0
        for a in os.listdir(albumdir): 
            if utils.isvaliddir(albumdir + os.sep + a):
                albums.append(a)
                for b in os.listdir(albumdir + os.sep + a):
                    total_n_files += 1
        
        # Notice that we no looking scan for new files here. the thread now is responsible for this.

        self.imindex = index
        self.dindex = dindex
        self.tagindex = tagindex
        self.idxfile = idxfile
        self.albumdir = albumdir

        self.update()
        splash.sunset()


    def get_index(self, t):
        if t == "im":
            return self.imindex
        elif t == "d":
            return self.dindex
        elif t == "tag":
            return self.tagindex

    def rename_folder(self, folder1, folder2):
        for im in list(self.mindex):
            if os.path.dirname(im) == folder1:
                im2 = os.path.abspath(folder2 + os.sep + os.path.basename(im))
                self.mindex[im2] = self.mindex[im]
                del self.mindex[im]

    def update(self):#, imindex, dindex, tagindex):
        # When renaming folders, you first have to run rename_folder to update mindex
        imindex = self.imindex
        dindex = self.dindex
        tagindex = self.tagindex
        sqlconn = sqlite3.connect(self.idxfile)
        sqlc = sqlconn.cursor()   
        sqlc.execute("create table if not exists inticache (path text, thumb text, date text, tags text, mtime text)")     
        sqlc.execute("delete from inticache")
        for path in imindex:
            thumb = imindex[path]
            date = ""
            for a in dindex[path].timetuple()[:6]:
                date += str(a) + ";"
            date = date.strip(";")
            tags = ""
            mtime = self.mindex[path]
            if tagindex.has_key(path):
                for tag in tagindex[path]:
                    tags += tag + ";"
                tags = tags.strip(";")
            
            if WIN: 
                path = unicode(path, "latin-1")

            sqlc.execute("insert into inticache values (?, ?, ?, ?, ?)", (unicode(path), unicode(thumb), unicode(date), unicode(tags), unicode(mtime)))
        sqlconn.commit()
        print "inticache: database saved"

    def create_thumb(self, path):
        thumb_path = self.cachedir + os.sep + "SOLTHUMB2_" + utils.random_string() + ".png"
        while os.path.exists(thumb_path):
            thumb_path = self.cachedir + os.sep + "SOLTHUMB2_" + utils.random_string() + ".png"
        im = tools.create_thumb(path)
        
        im.save(thumb_path, "PNG")
        return thumb_path


    def get_datetags(self, path):
        date, tags = metadata.get_datetags(path)
        return date, tags

    def set_date(self, path, dt):
        self.dindex[path] = dt
        metadata.set_date(path, dt)
        self.update()

    def watch(self, callback):
        Thread(target=self.watch_thread, args=(callback,)).start()
        return True

    def watch_thread(self, callback):
        while not self.watcher_stop:
            while self.hold_thread:
                time.sleep(5)
            time.sleep(2)
            
            checked = []
            modified = []
            calls = []
            break_l = False
            n_toprocess = 0
            for album in os.listdir(self.albumdir):
                if not utils.isvaliddir(self.albumdir + os.sep + album):
                    #print "error in validdir", self.albumdir + os.sep + album; break
                    continue
                for o in os.listdir(self.albumdir + os.sep + album):
                    f = self.albumdir + os.sep + album + os.sep + o
                    if len(os.listdir(self.albumdir + os.sep + album)) > 1 and o.endswith("intiplaceholder.png"):
                        os.remove(f)
                        continue
                    
                    
                    title,ext = os.path.splitext(f)
                    
                    # If raw convert to tiff
                    raw_format = callback("import_raw", None)
                    if raw_format:
	                    if ext.lower() in EXT_RAW and not os.path.exists(title+"."+raw_format):
	                        if raw_format != None and utils.inpath("dcraw"):                      
	                            tools.convert_raw(f, raw_format)
	                            f = title+"."+raw_format
	                            print "watcher: converted raw to %s" % (os.path.basename(f))
                            
                    if not ext.lower() in EXT:
                        continue
                    if not utils.isvalidfile(f):
                        continue
                    if f in self.skiplist:
                        print "watcher: bad file: moving %s to folder %s" % (os.path.basename(os.path.dirname(f))+os.sep+os.path.basename(f),_("Bad files"))
                        dir_util.mkpath(os.path.realpath(os.path.dirname(f)) + os.sep + _("Bad files"))
                        file_util.move_file(f, os.path.realpath(os.path.dirname(f)) + os.sep + _("Bad files") + os.sep + os.path.basename(f))
                        continue
                                              
           
                    # mtime should be rounded on 1, otherwise on windows the mtimes not correct.
                    mtime = round(os.stat(f).st_mtime, 1)
                    
                    if break_l:
                        checked.append(f)    
                        continue
                    
                    if self.mindex.has_key(f): # exists
                        if self.mindex[f] != mtime: # but modified
                            # Check for tags, image changes then update
                            modified.append(f)
                            calls.append(("update",f))
                            print "watcher update" , f
                            
                    else: # doesnt exist => newly added
                        # Check for tags, image changes then add
                        calls.append(("add",f))
                        print "watcher: add", f
                    if not break_l: 
                        self.mindex[f] = mtime
                    checked.append(f)
                    # For overload issues, I break when is over 50 items
                    if len(calls) > 50:
                        break_l = True
                        
                
            
            # Check for deleted files
            todelete = []
            for f in self.mindex:
                if not f in checked:
                    # Remove from lists
                    calls.append(("del",f))
                    print "watcher: del" , f
                    todelete.append(f)
            for f in todelete:
                del self.mindex[f]

            imindex = {}
            dindex = {}
            tagindex = {}
            forgetlist = []
            n_files = len(calls)-len(todelete)
            
            for f in self.mindex:
                
                # try 3 times: sometimes a file is being written
                # if not works then put file in skiplist
                t = 0      
                while t < 3:
                    
                    try:
                        if not self.imindex.has_key(f) or f in modified: # update/add
                            t1 = time.time()
                            imindex[f] = self.create_thumb(f)
                            
                            date, tags = self.get_datetags(f)
                            dindex[f] = date
                            tagindex[f] = tags
                            if f in modified: msg = _("Updated")
                            else: msg = _("Added")
                            #gobject.idle_add(callback, "notify", [imindex[f], msg])
                            gobject.idle_add(callback, "notify-update", [n_files, imindex[f], msg])
                        else:
                            imindex[f] = self.imindex[f]
                            tagindex[f] = self.tagindex[f]
                            dindex[f] = self.dindex[f]
                        break
                    except StandardError, err:
                        if time.time() - t1 > 15: 
                            print "watcher: %s has a serious problem, moving it out the image directory" % (f)
                            dir_util.mkpath(os.path.realpath(os.path.dirname(f)) + os.sep + _("Bad files"))
                            file_util.move_file(f, os.path.realpath(os.path.dirname(f)) + os.sep + _("Bad files") + os.sep + os.path.basename(f))
                            t = 3
                        t += 1
                        #print "watcher: error in %s, %s more tries" % (f, 4-t)
                        #if os.path.exists(imindex[f]): os.remove(imindex[f])
                        time.sleep(1)
                        if t == 3:
                           
                            error = "watcher: %s: %s" % (f, err)
                            if (time.time() - os.stat(f).st_ctime) < 200:
                                # creation date is less than 5 minutes, maybe it is still be written (huggeeee image file)
                                # going on, but not putting in skiplist
                                pass #print "watcher: skip %s" % (f)
                            elif type(err) == type(IOError()):
                                # It's creation date is more than 5 minutes ago, so we assume it is corrupt.
                                # Let the main process try to fix it
                                if f in self.fixlist: 
                                    self.skiplist.append(f)
                                elif not os.path.splitext(f)[1].lower() in EXT_VID: # not for videos
                                    callback("fix", f)
                                    
                                    self.fixlist.append(f)
                                    
                                    t -= 1
                                
                            else:                                
                                print error 
                                self.skiplist.append(f)
                                
                            
                            # Remove for indexes
                            for i in [imindex, dindex, tagindex]:
                                try: del i[f]
                                except: pass   
                            forgetlist.append(f)
                            break

            # be sure the notify window is not still waiting
            gobject.idle_add(callback, "notify-end", None)
            
            if self.imindex != imindex or self.dindex != dindex or self.tagindex != tagindex:
                self.imindex.update(imindex)
                self.dindex.update(dindex)
                self.tagindex.update(tagindex)
                self.update()
                
            

            call_add = []
            call_del = []
            call_update = []
            for call in calls:
                if call[1] in forgetlist: continue
                
                if call[0] == "add":
                    call_add.append(call[1])
                if call[0] == "del":
                    call_del.append(call[1])
                if call[0] == "update":
                    call_update.append(call[1])
                    
            if call_add != []:
                gobject.idle_add(callback, "add", call_add)
            if call_del != []:
                gobject.idle_add(callback, "del", call_del)
            if call_update != []:
                gobject.idle_add(callback, "update", call_update)
