#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   This file is part of IntiPunku
#   Copyright (C) 2007 Mark Baas <mark.baas123@gmail.com>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
from distutils import dir_util, file_util
import os
from inti import utils
from inti import tools
from inti import dialogs
import zipfile, tarfile
import datetime
from inti import metadata
from inti.constants import *
import socket

def load_gdata():
    try:
        import gdata.photos
        import gdata.photos.service
        import gnomekeyring
        global gdata, gnomekeyring
        return True
    except StandardError, err:
        print err
        return False
        
def load_flickr():
    try:
        import flickrapi
        global flickrapi
        return True
    except:
        return False
        
def load_email():
    if WIN:
        try:
            import simplemapi
            del simplemapi
            return True
        except: 
            return False
    else:
        return True
        #return utils.inpath("xdg-email")
        
GDATA_ENABLED = load_gdata()
FLICKR_ENABLED = load_flickr()
EMAIL_ENABLED = load_email()

import gobject, gtk



#import facebook, mozembed

import tempfile
from PIL import Image




# Localization
import gettext
_ = gettext.gettext


def get_picasa_login(config):
    keyring = gnomekeyring.get_default_keyring_sync()
    auth_token = int(config["google_token"])
    if auth_token > 0:
        try:
            secret = gnomekeyring.item_get_info_sync(keyring, auth_token).get_secret()
        except:
            #print err
            login = None
            password = None
            auth_token = 0
        else:
            login, password = secret.split('\n')
            return login, password
    else:
        login = None
        password = None

    if login != None and password != None:
        return login, password
    pws = gdata.photos.service.PhotosService()
    message = ""
    while 1:
        dialog = dialogs.LoginDialog(login, password, message)
        response = dialog.run()
        login = dialog.get_login()
        password = dialog.get_password()
        dialog.destroy()
        if response == gtk.RESPONSE_OK:
            if not login or not password:
                continue
            try:
                pws.ClientLogin(login, password)
            except gdata.service.BadAuthentication: 
                message = _("Wrong username/password, try again")
                continue

            auth_token = gnomekeyring.item_create_sync(
                keyring,
                gnomekeyring.ITEM_GENERIC_SECRET,
                "IntiPunku Google Login, login information",
                dict(appname="IntiPunku, keyring"),
                "\n".join((login, password)), True)
            config["google_token"] = auth_token
            config.save()
            return login, password
        elif response == gtk.RESPONSE_CANCEL:
            return None, None

class get_facebook_login:
    def __init__(self, fb, profiled):
        self.fb = fb
        url = fb.get_login_url(popup=True)
        gtkmozembed.set_profile_path(profiled, "intisurf")
        self.moz = gtkmozembed.MozEmbed()
        
                
        win = gtk.Window()
        win.set_icon(utils.load_icon("intipunku",size=48))
        win.set_title(_("Please log in to Facebook"))
        win.set_size_request(650,500)
        win.add(self.moz)
        self.moz.load_url(url)
        self.moz.connect("open-uri", self.event)
        win.show_all()
        self.win = win
        self.loop = gobject.MainLoop()
        self.loop.run()

    def event(self, moz, url):
        
        if url == "https://login.facebook.com/login.php?popup=1&canvas=1":
            gobject.timeout_add(3000, self.get_session)

    def get_session(self):
        try:
            session = self.fb.auth.getSession()
            uid = session['uid']
            self.loop.quit()
            self.win.destroy()
            
            
        except facebook.FacebookError, e:
            print e    

class SunHTML:
    def __init__(self, photos, template="white", title="IntiPunku Photo Page",logo=DATADIR+os.sep+"logo.png", size=(640,480)):



        # Bring up the dialog to get user preferences
        dialog = dialogs.HTMLExportDialog(DATADIR + os.sep + "templates_web", title)
        r = dialog.run()
        
        if r == gtk.RESPONSE_OK:
            title, folder, template, size = dialog.get_data()
            dialog.destroy()
        else:
            dialog.destroy()
            return
        self.webfolder = folder + os.sep + title
        progress = utils.SunProgress(_("Generating HTML page"), icon="html")
        n_actions = len(photos)*2 + 2
        progress.start(n_actions)

        n = 2
        webfolder = self.webfolder
        while os.path.exists(self.webfolder):
            self.webfolder = webfolder + str(n)
            n += 1

        dir_util.mkpath(self.webfolder + os.sep + "images")
        dir_util.mkpath(self.webfolder + os.sep + "thumbs")
        dir_util.mkpath(self.webfolder + os.sep + "targets")
        
        progress.set_text(_("Creating logos")) # Progress
        tools.resize_image(logo, self.webfolder + os.sep + "logo.png", (100,100), type="PNG")
        progress.plus() # Progress
        tools.resize_image(logo, self.webfolder + os.sep + "logo_small.png", (25,25), type="PNG")
        progress.plus() # Progress

        f = open(DATADIR + os.sep + "templates_web" + os.sep + template + ".html")
        html = f.read()
        f.close()
        html = "<meta http-equiv=Content-Type content=\"text/html; charset=UTF\">\n"+html
        html = html.replace("%title", unicode(title))

        html2 = ""
        for photo in photos:
            basename = os.path.basename(photo)
            progress.set_text(_("Processing %s") % basename) # Progress
            image_path = self.webfolder + os.sep + "images" + os.sep + basename
            thumb_path = self.webfolder + os.sep + "thumbs" + os.sep + basename
            tools.resize_image(photo, image_path, size)   
            progress.plus() # Progress
            tools.resize_image(photo, thumb_path, (200,200))
            progress.plus() # Progress
            html2 += "<a href=\"targets/%s.html\"><img border=\"0\" src=\"thumbs/%s\"></a>\n" % (basename, basename)

            # Create first,prev,next, last button on target page
            index = "<a href=\"../index.html\">%s</a>" % (_("Index"))
            first = "<a href=\"../targets/%s.html\">%s</a>" % (os.path.basename(photos[0]),_("First"))
            last = "<a href=\"../targets/%s.html\">%s</a>" % (os.path.basename(photos[-1]),_("Last"))
            if photos[-1] == photo:
                next = _("Next")
            else:
                next = "<a href=\"../targets/%s.html\">%s</a>" % (os.path.basename(photos[photos.index(photo)+1]),_("Next"))
            if photos[0] == photo:
                previous = _("Previous")
            else:
                previous = "<a href=\"../targets/%s.html\">%s</a>" % (os.path.basename(photos[photos.index(photo)-1]),_("Previous"))
            header = "%s|%s|%s|%s|%s" % (index, first, previous, next, last)
            html_t =html.replace("%navigate", header)
            self.create_target(image_path, html_t)
        html = html.replace("%navigate", "")
        html = html.replace("%pictures", html2)
        html = html.replace("%logo", "logo.png")
        html = html.replace("%lsmall", "logo_small.png")
        f = open(self.webfolder + os.sep + "index.html", "w")
        f.write(html)
        f.close()
        progress.destroy()


    def create_target(self, photo, html):
        html = html.replace("%logo", "../logo.png")
        html = html.replace("%lsmall", "../logo_small.png")
        target_html = html.replace("%pictures", "<img src=\"../images/%s\">" % os.path.basename(photo))
        f = open(self.webfolder + os.sep + "targets" + os.sep + os.path.basename(photo) + ".html", "w")
        f.write(target_html)
        f.close()

class SunFolder:
    def __init__(self, photos, title, size=(640,480)):

        # Bring up the dialog to get user preferences
        dialog = dialogs.FolderExportDialog(title)
        r = dialog.run()
        
        if r == gtk.RESPONSE_OK:
            title, folder, size = dialog.get_data()
            dialog.destroy()
        else:
            dialog.destroy()
            return
        self.webfolder = folder + os.sep + title
        progress = utils.SunProgress(_("Processing files"), icon="folder")
        n_actions = len(photos)
        progress.start(n_actions)

        n = 2
        webfolder = self.webfolder
        while os.path.exists(self.webfolder):
            self.webfolder = webfolder + str(n)
            n += 1
           
        dir_util.mkpath(self.webfolder)

        for photo in photos:
            basename = os.path.basename(photo)
            progress.set_text(_("Processing %s") % basename) # Progress
            image_path = self.webfolder + os.sep + basename
            tools.resize_image(photo, image_path, size)   
            progress.plus() # Progress

        progress.destroy()



class PicasaWeb:
    def __init__(self, files, calbum, config):
        username, password = get_picasa_login(config)
        if username == None:
            return
        
        
        progress = dialogs.SunProgress(_("Uploading photos"), cancel=True, icon="picasaweb")
        progress.start(1, _("Logging in to Google"))
        self.pws = gdata.photos.service.PhotosService()
        utils.sunthread(self.login, (username, password, progress))
        progress.plus()

        #Get all albums
        
        try: userfeed_albums = self.pws.GetUserFeed(kind='album')
        except socket.gaierror:
            utils.report_error(_("Connection with google had failed, please try again."))
            progress.destroy()
    
            return

        if progress.cancel: return

        albumsi = {} # to get the xml feed later
        albums = [] # to have everying in an order


        # Add the other albums
        n = 1
        for a in userfeed_albums.entry:
            albumsi[n] = a
            albums.append(a.title.text)
            n += 1

        # Prepare the current album - possible not created - in the list of albums, so we can process it later
        if not unicode(calbum) in albums:
            albums.insert(0, calbum)
            albumsi[0] = None
            i = 0
        else:
            i = albums.index(calbum)

        dialog = dialogs.PicasaWebDialog(albums, i)
        r = dialog.run()
        if r == gtk.RESPONSE_CANCEL:
            dialog.destroy()
            progress.destroy()
            return     

        # Handle album stuff
        i, album = dialog.get_album()
        if i == -1:
            albumf = None
        else:
            if not albumsi.has_key(0):
                i += 1
            albumf = albumsi[i]
       
        if albumf == None:
            albumf = self.pws.InsertAlbum(title=album, summary="")

        # Handle Image size stuff
        size = dialog.get_size()
        if size != _("Original"): size = size.split("x")
        tmpd = tempfile.mkdtemp(prefix="intipunku-")
            
        dialog.destroy()
        progress.start(len(files), text=_("Uploading photos"))
        for f in files:
            if progress.cancel:
                break
            if size != _("Original"):
                tmpf = tmpd + os.sep + os.path.basename(f)
                im = Image.open(f)
                imexiv = metadata.copy_metadata(f)
                im.thumbnail(size)
                im.save(tmpf)
                metadata.paste_metadata(imexiv,tmpf)
                f = tmpf
            progress.set_text(_("Uploading %s" % os.path.basename(f)))
            utils.sunthread(self.insertPhotoSimple, (albumf, f, progress))
                
            progress.plus()

        # clean up
        dir_util.remove_tree(tmpd)

        progress.destroy() 
    
    def insertPhotoSimple(self, albumf, f, progress):
        
        try: self.pws.InsertPhotoSimple(albumf, os.path.basename(f), "",f)
        except socket.gaierror:
            utils.report_error(_("Connection with google had failed, please try again."))
            

        
    def login(self, username, password, progress):
        
        try: self.pws.ClientLogin(username, password, progress)
        except socket.gaierror:
            utils.report_error(_("Connection with google had failed, please try again."))
            
   
            

class Flickr:
    def __init__(self, files, calbum, public=1, friend=1, family=1):
        flickr = flickrapi.FlickrAPI(FLICKR_API_KEY, FLICKR_API_SECRET)
        (token, frob) = flickr.get_token_part_one(perms='write')
        if not token:
            dialog = gtk.MessageDialog(flags=gtk.DIALOG_MODAL, type=gtk.MESSAGE_WARNING, buttons=gtk.BUTTONS_OK)
            dialog.set_markup(_("A webbrowser window has been opened. Please click OK when you have allowed intipunku to acces flickr."))
            dialog.set_title(_("Please wait..."))
            r = dialog.run()
            dialog.destroy()  
        flickr.get_token_part_two((token, frob))  
        progress = dialogs.SunProgress(_("Uploading photos"), cancel=True, icon="flickr")
        progress.start(len(files))
        for f in files:
            if progress.cancel: break
            flickr.upload(filename=f, callback=self.progress, tags='''"%s"''' % (calbum), is_public=public, is_friend=friend, is_family=family)
            progress.plus()
        progress.destroy()
        
    def progress(self, progress, done):
        if progress:
            while gtk.events_pending(): gtk.main_iteration() 
            
class Email:
    def __init__(self, files, config, recipient, message, size, compression, transport):
        tmpd = tempfile.mkdtemp(prefix="intiemail")
        
        progress = dialogs.SunProgress(_("Generating email and attachments"), cancel=False, icon="photomail")
        if compression: n = len(files)*2
        else: n = len(files)
        progress.start(n)
        
        # Resize/copy
        paths = []
        for f in files:
            if size:
                im = Image.open(f)
                im.thumbnail(size)
                im.save(os.path.join(tmpd,os.path.basename(f)))
            else:
                file_util.copy_file(f, os.path.join(tmpd,os.path.basename(f)))
            progress.plus()
            paths.append(os.path.join(tmpd,os.path.basename(f)))
        
        
        # Create the compressed archive
        dt = datetime.datetime.today()
        today = str(dt.year)+str(dt.month)+str(dt.day)
        if compression:
            d = os.getcwd()
            os.chdir(tmpd)
            if compression == "tar.bz2":
                fz = _("photos")+"_"+today+".tar.bz2"
                pack = tarfile.TarFile.open(fz, "w:bz2")
            elif compression == "tar.gz":
                fz = _("photos")+"_"+today+".tar.gz"
                pack = tarfile.TarFile.open(fz, "w:gz")
            elif compression == "zip":
                fz = _("photos")+"_"+today+".zip"
                pack = zipfile.ZipFile(fz, "w")
            
            for f in paths:
                if compression == "zip":
                    pack.write(os.path.basename(f))
                else:
                    pack.add(os.path.basename(f))
                progress.plus()
            
            pack.close()
            os.chdir(d)
            attachments = [os.path.join(tmpd,fz)]
        else:
            attachments = paths
           
        progress.pulse()
        # Fire up default mail client
        if transport == "default":
            if MAC:
                o = ""
                for attachment in attachments:
                    o += "%s " % attachment
                os.popen("send_mail.sh %s" % (o))
            elif WIN: 
                import simplemapi
                o = ""
                for attachment in attachments:
                    o += "%s;" % attachment
                try: simplemapi.SendMail(recipient, subject=_("Intipunku photo mailing"), body=message, attachfiles=o, preview=1)
                except: pass
                del simplemapi
            else:
                o = ""
                for attachment in attachments:
                    o += "--attach \"%s\" " % attachment
                
                xdg = os.path.join(DATADIR,"xdg-email")
                cmd = "sh %s --subject \"%s\" --body \"%s\" %s mailto:\"%s\"" % (xdg, _("Intipunku photo mailing"),_("Dear recipient,\n\nYour friend wants to send you some photos.\n\n --Intipunku (auto-generated message)"), o, recipient)
                print cmd
                os.popen(cmd)
            
            
        elif transport == "gmail":
            # Construct message
            from email.MIMEText import MIMEText
            from email.MIMEImage import MIMEImage
            from email.MIMEMultipart import MIMEMultipart
            from email.MIMEBase import MIMEBase
            from email import encoders
            import smtplib
            import mimetypes
            
            user, password = get_picasa_login(config)
            msg = MIMEMultipart()
            msg['From'] = user
            msg['To'] = recipient
            msg['Subject'] = _("Photos sent through Intipunku")
            msg.attach(MIMEText(message))

            for attachment in attachments:
                f = open(attachment)
                if compression:
                    m, s = "application", "x-compressed" 
                    att = MIMEBase(m,s)
                    att.set_payload(f.read())
                    encoders.encode_base64(att)
                else:
                    att = MIMEImage(f.read())
                att.add_header('Content-Disposition', 'attachment', filename=os.path.basename(attachment))
                f.close()
                msg.attach(att)
                
            print "email: opening smtp server"
            mailer = smtplib.SMTP("smtp.gmail.com", 587)
            mailer.ehlo()
            mailer.starttls()
            mailer.ehlo()
            user, password = get_picasa_login(config)
            mailer.login(user, password)
            print "email: logged in"
            try:
                mailer.sendmail("", recipient, msg.as_string())
            except smtplib.SMTPRecipientsRefused:
                utils.report_error(_("Sending email has failed, wrong email address!"))
            mailer.close()
            
            del MIMEText
            del MIMEImage
            del MIMEMultipart
            del MIMEBase
            del smtplib
            del mimetypes
            del encoders
            
            print "email: sent, server closed"
        progress.destroy() 
        dir_util.remove_tree(tmpd)
        
            
