#!/usr/bin/env python
#
#   This file is part of IntiPunku
#   Copyright (C) 2007 Mark Baas <mark.baas123@gmail.com>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys, os.path, os

__dir__ = os.path.realpath(os.path.dirname(__file__))
sys.path.append(__dir__)
DATADIR = __dir__ + os.sep + "data"
os.environ["INTIDATA"] = DATADIR

# Localization
import gettext
APP = 'intipunku'
if os.path.exists("po"):
    gettext.bindtextdomain(APP, "po")
gettext.textdomain(APP)
_ = gettext.gettext

from inti import intiimport
from inti import dialogs
from inti.config import SunConfig
from inti import utils
import inti.intigtk as gtk, gobject
from distutils import file_util, dir_util
from inti.constants import *

class IntiPunkuImport:
    def __init__(self, t):
        if "--config" in sys.argv:
            self.confdir = sys.argv[sys.argv.index("--config")+1]
        else:
            self.confdir = os.path.expanduser("~") + os.sep + ".config/intipunku"
        self.cachedir = self.confdir + os.sep + "cache"

        self.config = SunConfig(self.confdir)
        if not os.path.exists(self.config["photo_folder"]):
            dialog = IntiSettings(self.config)
            r = dialog.run()
            if r == gtk.RESPONSE_ACCEPT:
                dialog.save_settings()
            dialog.destroy()

        self.albumdir = self.config["photo_folder"]
        albums = []
        for album in os.listdir(self.albumdir):      
            if os.path.isdir(self.albumdir + os.sep + album):
                albums.append(album)
        
        self.win = gtk.Window()#dialogs.ImportDialog()
        self.win.set_size_request(700,500)
        self.win.set_icon(utils.load_icon("intipunku",size=48))
        self.win.connect("delete-event", self.sunset)

        self.page = intiimport.pages[t](self.win, albums, self.config, callback=self.cb_import)
        
        vbox = gtk.VBox()
        vbox.set_border_width(5)
        vbox.pack_start(self.page)
        vbox.pack_start(gtk.HSeparator(), fill=False, expand=False)
        hbox = gtk.HButtonBox()
        hbox.set_border_width(3)
        hbox.set_layout(gtk.BUTTONBOX_END)
        vbox.pack_start(hbox, fill=False, expand=False)
        button_ok = gtk.Button(stock=gtk.STOCK_OK)
        button_cn = gtk.Button(stock=gtk.STOCK_CANCEL) 
        button_ok.connect("clicked", self.run, gtk.RESPONSE_OK)
        button_cn.connect("clicked", self.run, gtk.RESPONSE_CANCEL)
        hbox.pack_start(button_ok)
        hbox.pack_start(button_cn)
        self.win.add(vbox)
        self.win.set_position(gtk.WIN_POS_CENTER)
        self.win.show_all()

    def run(self, widget, r):
        if r == gtk.RESPONSE_OK:
            self.page.doImport()
        self.sunset()

    def sunset(self, *args):
        self.win.destroy()
        try: gtk.main_quit()        
        except: sys.exit()
        
    def cb_import(self, name, *args):
        if name == "sunset":
            self.sunset()



if __name__ == "__main__":
    page = None
    for t in intiimport.pages:
        if "--"+t in sys.argv:
            page = t
            break
    if not page == None:
        app  = IntiPunkuImport(page)
        try: gtk.main()
        except KeyboardInterrupt, err:
            print err
            sys.exit()
    else:
        print _("Usage: ")
        for t in intiimport.pages:
            print "  intipunku-import --%s" % (t)
