#!/usr/bin/env python
#
#   This file is part of IntiPunku
#   Copyright (C) 2007 Mark Baas <mark.baas123@gmail.com>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys, os.path, os

__dir__ = os.path.realpath(os.path.dirname(__file__))
sys.path.append(__dir__)
DATADIR = __dir__ + os.sep + "data"
os.environ["INTIDATA"] = DATADIR

# Localization
import gettext
APP = 'intipunku'
if os.path.exists("po"):
    gettext.bindtextdomain(APP, "po")
gettext.textdomain(APP)
_ = gettext.gettext

from inti import intiimport
from inti import dialogs
from inti.config import SunConfig
from inti import utils
from inti import editor
from inti import tools
from inti import browser
import inti.intigtk as gtk, gobject
from distutils import file_util, dir_util
from inti.constants import *
import urllib
from inti import metadata

__title__ = _("IntiPunku Viewer")
__version__ = "0.3"
f = open(__dir__ + os.sep + "credits.txt")
__credits__ = f.read()
__authors__ = __credits__.split("Translators:")[0].strip()
__translators__ = __credits__.split("Translators:")[1].strip()

class IntiPunkuViewer:
    def __init__(self, f):
    
        self.photo = f

        # General start up initialization stuff
        if "--config" in sys.argv:
            self.confdir = sys.argv[sys.argv.index("--config")+1]
        else:
            self.confdir = os.path.expanduser("~") + os.sep + ".config/intipunku"
        self.cachedir = self.confdir + os.sep + "cache"

        self.config = SunConfig(self.confdir)
        if not os.path.exists(self.config["photo_folder"]):
            dialog = IntiSettings(self.config)
            r = dialog.run()
            if r == gtk.RESPONSE_ACCEPT:
                dialog.save_settings()
            dialog.destroy()

        self.albumdir = self.config["photo_folder"]
        albums = []
        for album in os.listdir(self.albumdir):      
            if os.path.isdir(self.albumdir + os.sep + album):
                albums.append(album)
        
        __version__ = self.config["version"]

        # Set up fake dindex and tagindex, so we can get the files date
        self.dindex = {}
        self.tagindex = {}
        date, tags = metadata.get_datetags(f)
        self.tagindex[f] = tags
        self.dindex[f] = date


        
        self.win = gtk.Window()#dialogs.ImportDialog()
        self.win.set_size_request(800,500)
        self.win.set_title(__title__)
        self.win.set_icon(utils.load_icon("intipunku",size=48))
        self.win.connect("delete-event", self.sunset)
        mvbox = gtk.VBox()

        """
        Menu Bar
        """
        imindex = self.get_index()
        
        minibrowser = browser.SunMiniBrowser(imindex, self.cb_editor, i_method=gtk.gdk.INTERP_NEAREST)

        menubar = self.create_menus()
        self.editor = editor.SunEditor(self.win, self.cb_editor, minibrowser=minibrowser)    
        minibrowser.im_load = list(imindex)
        minibrowser.init(editor=self.editor)
        self.mb = minibrowser
        mvbox.pack_start(menubar, expand=False, fill=False)
        mvbox.pack_start(self.editor, expand=True, fill=True)

        self.win.add(mvbox) 
        self.win.set_position(gtk.WIN_POS_CENTER)
        
        self.editor.load(f)  
        self.win.show_all() 


    def sunset(self, *args):
        if self.editor.draw.im_modified:
            self.editor.draw.save_to_file(path=self.photo)
        self.win.destroy()
        try: gtk.main_quit()        
        except: sys.exit()
        
    def cb_editor(self, name, *args):
        if name == "savedraw":
            if self.editor.draw.im_modified:
                self.editor.draw.save_to_file(path=self.photo)
        if name == "edit":
            imindex = self.get_index()
            self.mb.im_load = list(imindex)
            self.mb.imindex = imindex

            self.photo = args[0]
            date, tags = metadata.get_datetags(self.photo)
            self.tagindex[self.photo] = tags
            self.dindex[self.photo] = date

            self.editor.load(args[0])

    def create_menus(self):
        utils.register_icons()
        self.uimanager = gtk.UIManager()
        accelgroup = self.uimanager.get_accel_group()
        self.win.add_accel_group(accelgroup)

        # Rewrite the menu.xml so that only action for the viewer will be show
        f = open(DATADIR + os.sep + "menus.xml")
        lines = f.readlines()
        f.close()
        filters = ["settings", "slideshow", "pano", "import", "export", "rename_album", "edit", "Tags", "removetags", "removedisk", "alltags", "selectall"]
        xml = ""
        for l in lines:
            skip = False
            for filt in filters:
                s = "action=\"%s\"" % filt
                if s in l:
                    skip = True
                    break
            if skip == False:
                xml += l + "\n"
                           

        actiongroup = gtk.ActionGroup('menuactions')
        actions = [("intipunku", None, "IntiPunku", None, None, None),
                 ("actions", None, _("_Edit"), None, None, None),
                 ("quit", gtk.STOCK_QUIT, _("E_xit"), None, None, self.handle_item),
                 ("help", None, _("_Help"), None, None, None),
                 ("about", gtk.STOCK_ABOUT, None, None, None, self.handle_item),
                 ("wall", "wallpaper", _("Set as wallpaper"), None, None, self.handle_item),
                 ("rleft", "rotate-left", _("Rotate left"), "<control>Left", None, self.handle_item),
                 ("rright", "rotate-right", _("Rotate right"),"<control>Right", None, self.handle_item),
                 ("date", "clock", _("Change time/date"), None, None, self.handle_item),
                 ("tools", None, _("_Tools"), None, None, None),
                ]
        actiongroup.add_actions(actions)

        self.uimanager.insert_action_group(actiongroup, 0)
        self.uimanager.add_ui_from_string(xml)

        return self.uimanager.get_widget('/MenuBar')


    def handle_item(self, widget):
        name = widget.get_name()
        if name == "quit":
            self.sunset()
     
        if name == "about":
            dialog = dialogs.AboutDialog(__title__, __version__, __authors__, __translators__, __dir__ + os.sep + "license.txt")
            dialog.run()
            dialog.destroy()
        if name == "wall":
            tools.set_wallpaper(self.photo)
        if name == "rleft":
            self.editor.draw.rotate_left()
        if name == "rright":
            self.editor.draw.rotate_right()
        if name == "date":
            dialog = dialogs.DateDialog([self.photo], self.dindex)
            r = dialog.run()
            if r == -3:
                dt = dialog.get_datetime()
                tools.set_date(self.photo, dt)
                self.dindex[self.photo] = dt
                self.editor.update_label(self.photo)
            dialog.destroy()

    def get_index(self):
        imindex = {}
        for f in os.listdir(os.path.dirname(self.photo)):
            name, ext = os.path.splitext(f)
            if ext.lower() in EXT:
                imindex[os.path.dirname(self.photo) + os.sep + f] = os.path.dirname(self.photo) + os.sep + f
        return imindex


if __name__ == "__main__":
    if len(sys.argv) >= 2:
        f = urllib.unquote(sys.argv[1])
        if sys.platform == "win32":
            f = os.path.splitdrive(f)[1]         
        else:
            if f.startswith("file://"):
                f = f.split("//")[1]
            
        if not os.path.exists(f):
            sys.exit()
        f = os.path.realpath(f)
        print f
        app  = IntiPunkuViewer(f)
            

        try: gtk.main()
        except KeyboardInterrupt, err:
            print err
            sys.exit()

    else:
        print _("Usage: intipunku-viewer <path to file>")
