#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   This file is part of IntiPunku
#   Copyright (C) 2007 Mark Baas <mark.baas123@gmail.com>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import inti

from threading import Thread
import inti.intigtk as gtk
import gobject, time
gobject.threads_init()
import os, sys, os.path

# Localization
import gettext
APP = 'intipunku'
if os.path.exists("po"):
    gettext.bindtextdomain(APP, "po")
elif os.environ.has_key("LOCALEDIR"):
    gettext.bindtextdomain(APP, os.environ["LOCALEDIR"])

gettext.textdomain(APP)
_ = gettext.gettext

try:
    f = __file__
except:
    __file__ = sys.executable
__dir__ = os.path.realpath(os.path.dirname(__file__))
sys.path.append(__dir__)
DATADIR = __dir__ + os.sep + "data"
os.environ["INTIDATA"] = DATADIR

from inti.constants import *

if WIN:
    os.environ["PATH"] += VSEP + (__dir__ + os.sep + "bin")

from inti import utils
from inti import optional

# important dependencies are exiv2 and python-gtkimageview, check then or ask to install them
pkgs = []
try:
    import pyexiv2
    del pyexiv2
except:
    pkgs.append("exiv2")
try: 
    import gtkimageview
    del gtkimageview
except: 
    pkgs.append("pygtkimageview")
if pkgs != []: optional.install(pkgs)

    
from inti import watcher
from inti import config
from inti.editor import SunEditor
from inti.browser import SunBrowser, SunMiniBrowser
from inti import slide
from inti.web import SunHTML, PicasaWeb, Flickr, SunFolder
from inti import web
SunHTML = web.SunHTML
PicasaWeb = web.PicasaWeb
from inti import dialogs
from inti import tools
from inti import intiimport
from inti.tray import SunTray
from inti import intiprint

from inti import metadata
from inti import swfexport
from inti import intisane


from distutils import dir_util, file_util
import urllib, datetime

if MAC:
    from igemacintegration import *
    import gio,atk,pangocairo

__title__ = "IntiPunku"
__version__ = str(VERSION)
f = open(__dir__ + os.sep + "credits.txt")
__credits__ = f.read()
__authors__ = __credits__.split("Translators:")[0].strip()
__translators__ = __credits__.split("Translators:")[1].strip()
f.close()

def load_raw():
    return utils.inpath("dcraw")
RAW_ENABLED = load_raw()
def load_jpegtran():
    return utils.inpath("jpegtran")
JPEGTRAN_ENABLED = load_jpegtran()

# List of optionals (when adding, add here test callbacks and var and in optional.py the necessary packages
opts = {
"gdata" : (intiimport.load_gdata, intiimport.GDATA_ENABLED, _("Support for exporting fotos to Picasaweb albums")),
"gphoto" : (intiimport.load_gphoto, intiimport.GPHOTO_ENABLED, _("Support for importing fotos from a digital camera")),
"ming" : (swfexport.load_ming, swfexport.MING_ENABLED, _("Support for exporting fotos into a Flash slideshow")),
"pano" : (tools.load_pano, tools.PANO_ENABLED,_("Support for creating panoramas")),
"flickr" : (web.load_flickr, web.FLICKR_ENABLED, _("Support for exporting fotos to Flickr albums")),
"email" : (web.load_email, web.EMAIL_ENABLED, _("Support for sending emails")),
"video" : (load_video, VIDEO_ENABLED, _("Support playing video files")),
"scanner": (intisane.load_sane, intisane.SANE_ENABLED, _("Support for scanning photos")),
"raw" : (load_raw, RAW_ENABLED, _("Support for auto importing raw photos")),
"jpegtran" : (load_jpegtran, JPEGTRAN_ENABLED, _("Support for lossless rotations")),
}

class IntiPunku:
    def __init__(self):

        """
        All functions related to checking the environment should be put here
        """


        """
        Load configuration
        """

        if "--config" in sys.argv:
            self.confdir = sys.argv[sys.argv.index("--config")+1]
        else:
            self.confdir = CONFDIR
        self.cachedir = self.confdir + os.sep + "cache"

        self.config = config.SunConfig(self.confdir)
        if self.config["first_time"]:
            dialog = config.DialogFirst(self.config)
            r = dialog.run()
            dialog.destroy()      
            if r == gtk.RESPONSE_CANCEL:
                gobject.idle_add(gtk.main_quit)
                return
        self.config.init()
        # All fine, initialize the program
        self.init()         

    def init(self):

        # Proceed configuration

        if self.config["version"] == 0.1: # Rebuild cache
            try: dir_util.remove_tree(self.cachedir)
            except: pass
        self.config["version"] = __version__
        self.config.save()     

        # Set up watcher to watch activity in folder
                
        self.albumdir = self.config["photo_folder"]
        self.watcher = watcher.SunWatcher(self.albumdir, self.cachedir, __version__)
        self.imindex = self.watcher.imindex
        self.dindex = self.watcher.dindex
        self.tagindex = self.watcher.tagindex
        self.alltags = []
        for im in self.tagindex:
            for tag in self.tagindex[im]:
                if tag.strip() == "":
                    continue
                if not tag in self.alltags:
                    self.alltags.append(tag)

        # Load main window

        self.win = gtk.Window()
        self.win.set_default_size(600,400)
        self.win.maximize()
        self.win.set_icon_from_file(DATADIR + os.sep + "logo-48x48.png")
        self.win.connect("delete-event", self.sunset)
        self.win.set_title(_("IntiPunku - The Sun Gate Photo Manager"))
        self.win.show()

        # Setting up other widgets

        self.notebook = gtk.Notebook()
        self.notebook.set_show_tabs(False) 

        self.phototray = SunTray(self.win)
        self.imbrowser = SunBrowser(self.imbrowser_cb, self.imindex, self.dindex, self.tagindex, self.win, self.config)

        # Menu Bar
        
        menubar = self.create_menus()            

        mvbox = gtk.VBox()
        mvbox.pack_start(menubar, expand=False, fill=False)
        mvbox.pack_start(self.notebook, expand=True, fill=True)

        self.win.add(mvbox) 

        # Control panel
       
        hbox = gtk.HBox()
        
        vbox2 = gtk.VBox()
        vbox2.pack_start(self.phototray, fill=False, expand=False)

        hbox.pack_start(vbox2, fill=False, expand=False)
        
        #toolbar
        vboxt = gtk.VBox()
        vboxt.set_border_width(2)
        toolbar = self.uimanager.get_widget("/Toolbar")
        toolbar.set_style(gtk.TOOLBAR_BOTH)
        toolbar2 = self.uimanager.get_widget("/Toolbar2")
        toolbar2.set_style(gtk.TOOLBAR_BOTH)
        vboxt.pack_start(toolbar, fill=False, expand=False)
        self.imbrowser.viewer.titlespace.pack_end(toolbar2, fill=True, expand=True)
        hbox.pack_start(vboxt)

        vbox = gtk.VBox()
        vbox.pack_start(self.imbrowser, fill=True, expand=True)
        vbox.pack_end(hbox, fill=False, expand=False)
        vbox.set_spacing(5)
        vbox.set_border_width(5)

        self.minibrowser = SunMiniBrowser(self.imindex, callback=self.imbrowser_cb)
        self.editor = SunEditor(self.win, self.edit_cb, minibrowser=self.minibrowser)
        self.editor.connect("image-changed", self.general_cb, "image-changed")
        self.minibrowser.init(self.editor, self.imbrowser)
        self.minibrowser.album_changed_cb(None, self.imbrowser.viewer.box_load)
        self.notebook.append_page(vbox)
        self.notebook.append_page(self.editor)           

        self.win.show_all()
        if MAC: menubar.hide() #The menubar should be kept hidden
        
        self.notebook.set_current_page(0)      

        # Monitor files on change
        Thread(target=self.watcher.watch_thread, args=(self.watcher_cb,)).start()
        #gobject.timeout_add(10000, self.watcher.watch, self.watcher_cb)
        
        # Check for new versions
        utils.sunthread(self.check_version)
        
    def check_version(self):
        try:
            webversion = float(urllib.urlopen("http://intipunku.berlios.de/version").read())
            if webversion > VERSION and self.config["notify_updates"]:
                gobject.idle_add(utils.report_error,_("There is a new version (%s) available.\nYou can download this at <b>http://intipunku.berlios.de</b>" % (webversion))+"\n\n"+_("<i>This warning can be turned off in preferences</i>"))
        except:
            # No internet connection, not a problem
            pass
    
       
    def sunset(self, *args):
        self.watcher.watcher_stop = True
        gtk.main_quit()

    def general_cb(self, *args):
        if args[-1] == "image-changed":
            images = list(args[1:-1])
            self.imbrowser.set_selection(images)

    def imbrowser_cb(self, *args):
        action = args[0]
        if action == "button3":
            #gobject.timeout_add(100, self.imbrowser_menu)
            self.imbrowser_menu(args[1], args[2])
        if action == "phototray":
            action2 = args[1]
            if action2 == "add":
                self.phototray.add(args[2])
            elif action2 == "remove":
                self.phototray.remove(args[2])
            else:
                self.phototray.clear()
        if action == "album_popup":
            menu = self.uimanager.get_widget("/Album")
            menu.popup(None,None,None,3,0)

        if action == "rename_album":
            self.rename_album(args[1])

        if action == "edit":
            self.switch_edit(args[1])
        if action == "savedraw":
            self.editor.draw.save_to_file()
        if action == "reload_tags":
            self.reload_tags()

        if action == "album-changed":
        	self.imbrowser.emit("album-changed", self.imbrowser.viewer.box_load)
        	
        if action == "dragdest":
            dest_album, source = args[1:]
            #source = urllib.unquote(source.strip().split("://")[1]), we drag selections
            sources = self.imbrowser.get_selection()
            # Ask whether the file is to be moved/copied
            if len(sources) < 1: sources = [urllib.unquote(source.strip().split("://")[1])]
            if len(sources) > 1: q = _("Do you want to move or copy these files?")
            else: q = _("Do you want to move or copy this file?")
            
            question1 = utils.question(q, buttons=[(_("Move"), 40), (gtk.STOCK_COPY, 41)])
            question2_all = False
            for source in sources:
                dest = os.path.join(dest_album, os.path.basename(source))
                
                ext = os.path.splitext(source)[1].lower()
                # Check if is right extension
                if ext in EXT:
                    # Ask whether to overide the existing file
                    if os.path.exists(dest):
                        if question2_all == False:
                            question2, question2_all = utils.question(_("A foto with the same filename (%s) already exists in this folder, do you want to overwrite or rename this file?" % (os.path.basename(source))), buttons=[(_("Overwrite"), gtk.RESPONSE_OK), (_("Rename"), 40), (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)], widget=True)
                            
                        if question2 == gtk.RESPONSE_CANCEL: return
                        elif question2 == 40: 
                            name = os.path.splitext(os.path.basename(source))[0]
                            while os.path.exists(dest):
                                print dest
                                try: 
                                    name = "%05d" % (int(name)+1)
                                    dest = os.path.join(os.path.dirname(dest), name+ext)
                                except:
                                    dest = os.path.join(os.path.dirname(dest), os.path.splitext(os.path.basename(dest))[0]+"_"+ext)
                        #else: continue
                    
                    if question1 == 41:
                        print "copying", source, "to", dest
                        file_util.copy_file(source, dest)
                    elif question1 == 40:
                        print "moving", source, "to", dest
                        file_util.move_file(source, dest)
                        


    def edit_cb(self, *args):
        action = args[0]
        if action == "update-thumbs":
            print "This function is depreciated"
            self.imbrowser.viewer.update_thumb(args[1])
            self.imbrowser.viewer.update_date(args[1])
            self.minibrowser.update_thumb(args[1])
        if action == "sunset_editor":
            self.notebook.set_current_page(0)
        if action == "datetags":
            im = self.editor.imf
            date = self.dindex[im]
            tags = self.tagindex[im]
            return date,tags
        if action == "thumb":
            im = self.editor.imf
            return self.imindex[im]
        if action == "lossless":
            if not os.path.splitext(args[1])[1].lower() == ".jpg":
                return False
            return self.config["editor_lossless"]
        if action == "save":
            return self.config["editor_autosave"]
            
    def watcher_cb(self, action, flist):
        if action == "update":
            for f in flist:
                self.imbrowser.viewer.update_thumb(f)
                self.imbrowser.viewer.update_date(f)
                self.minibrowser.update_thumb(f)
        elif action == "add":
            self.imbrowser.refresh_albums(select=self.imbrowser.album)
            sort_alb = False
            for f in flist:
                alb = os.path.basename(os.path.dirname(f))
                if alb == self.imbrowser.album:
                    sort_alb = True
                    self.imbrowser.viewer.add_thumb_simple(f)
                    self.imbrowser.emit("album-changed", self.imbrowser.viewer.box_load)
            if sort_alb:
                self.imbrowser.viewer.sort_thumbs()
        elif action == "del":
            self.imbrowser.refresh_albums(select=self.imbrowser.album)
            sort_alb = False
            for f in flist:
                alb = os.path.basename(os.path.dirname(f))
                if alb == self.imbrowser.album:
                    sort_alb = True
                    self.imbrowser.viewer.remove_thumb_simple(f)
                    self.imbrowser.emit("album-changed", self.imbrowser.viewer.box_load)
                    
            if sort_alb:
                self.imbrowser.viewer.sort_thumbs()
        elif action == "notify-update":
            utils.notify(flist)
        elif action == "notify-end":
            utils.notify("end")
        elif action == "import_raw":
            if self.config["import_raw"] == False: return None
            else: return self.config["raw_format"] 
        elif action == "fix":
            f = flist
            try: 
                tools.fixbroken(f)
            except StandardError, err:
                print "tools: %s couldn't be fixed: %s" % (f, err)            

    def rename_album(self, new):
        path1 = self.albumdir + os.sep + self.imbrowser.album
        path2 = self.albumdir + os.sep + new
        if os.path.exists(path2):
            utils.report_error(_("Cannot change album name: There is already an album with this name"))
            return
        # Rename dir
        os.rename(path1, path2)
        # Change imindex
        todelete = []
        for im in list(self.imindex):
            c = os.path.basename(os.path.dirname(im))
            if c == self.imbrowser.album:
                basename = os.path.basename(im)
                nim = os.path.abspath(path2 + os.sep + basename)
                self.imindex[nim] = self.imindex[im]
                self.dindex[nim] = self.dindex[im]
                self.tagindex[nim] = self.tagindex[im]
                todelete.append(im)
                del self.imindex[im]
                del self.dindex[im]
                del self.tagindex[im]

        self.watcher.rename_folder(path1, path2)
        


    #########################
    # Popup menu ImBrowser
    # 

    def imbrowser_menu(self, event_time, unset_mod_cb):      
        popup = self.uimanager.get_widget("/Pictures") 
        # We need to connect also de release event to the menu, other you can release e.g. ctrl and the event the self.modifier keeps set
        popup.connect("key-release-event", unset_mod_cb)
        popup.popup(None,None,None,3,event_time)                    

    #
    #########################

    #########################
    # Setup menus


    def create_menus(self):
        utils.register_icons()
        self.uimanager = gtk.UIManager()
        accelgroup = self.uimanager.get_accel_group()
        self.win.add_accel_group(accelgroup)

        f = open(DATADIR + os.sep + "menus.xml")
        xml = f.read()
        f.close()
#        xmltags = ""
#        actionstags = []
#        for tag in self.alltags:
#            xmltags += "<menuitem action=\"%s\"/>\n" % tag
#            actionstags.append((tag, None, tag, None, None, self.handle_tags, False))
#        xml = xml.replace("<menuitem action=\"alltags\"/>", xmltags.strip())

        # Mac menu adjustment
        # Eliminate the main intipunku main
        if MAC:
            b = xml.find("<menu action=\"intipunku\">")
            e = xml[b:].find("</menu>")+b+len("</menu>")
            xml = xml[:b]+xml[e:]
            xml = xml.replace("<menu action=\"actions\">","<menu action=\"actions\">\n<menuitem action=\"settings\"/>\n<menuitem action=\"quit\"/>\n")

        actiongroup = gtk.ActionGroup('menuactions')
        actions = [("intipunku", None, "Intipunku", None, None, None),
                 ("settings", gtk.STOCK_PREFERENCES, _("_Preferences"), None, None, self.handle_item),
                 ("quit", gtk.STOCK_QUIT, _("E_xit"), None, None, self.handle_item),
                 ("help", None, _("_Help"), None, None, None),
                 ("about", gtk.STOCK_ABOUT, None, None, None, self.handle_item),
                 ("new_album", None, _("New album"), None, None, self.handle_item),
                 ("rename_album", None, _("Rename album"), None, None, self.handle_item),
                 ("edit", "gtk-edit", _("_Edit mode"), None, None, self.handle_item_gt0),
                 ("wall", "wallpaper", _("Set as wallpaper"), None, None, self.handle_item_gt0),
                 #("export", "export", _("Export photos"), None, None, self.handle_item_gt0),
                 ("actions", None, _("_Edit"), None, None, None),
                 ("selectall", "gtk-select-all", _("Select all"), "<control>a", None, self.handle_item),
                 ("rleft", "rotate-left", _("Rotate left"), "<control>Left", None, self.handle_item_gt0),
                 ("rright", "rotate-right", _("Rotate right"),"<control>Right", None, self.handle_item_gt0),
                 ("tags", "tag2", _("Edit tags"), None, None, self.handle_item_gt0),
                 ("date", "clock", _("Change time/date"), None, None, self.handle_item_gt0),
                 ("slideshow", "slideshow", _("Slideshow"), None, None, self.handle_item),
                 ("pano", "panorama", _("Panorama"), None, None, self.handle_item_gt1),
                 ("tools", None, _("_Tools"), None, None, None),
                 ("import", "import", _("Import or Export"), None, None, self.handle_item),
                 ("removedisk", gtk.STOCK_DELETE, _("Remove from disk"), None, None, self.handle_item_gt0),
                 ("print", "gtk-print", _("Print"), None, None, self.handle_item_gt0),
                 ("slideswf", "flash", _("Flash slideshow"), None, None, self.handle_item_gt1),
                 ("email", "photomail", _("Email"), None, None, self.handle_item_gt0),
                 ("collage", "collage", _("Collage"), None, None, self.handle_item_gt1),
                 ("components", gtk.STOCK_EXECUTE, _("Components"), None, None, self.handle_item),
                 
                ]
        actiongroup.add_actions(actions)
        #actiongroup.add_toggle_actions(actionstags)


        self.uimanager.insert_action_group(actiongroup, 0)
        self.uimanager.add_ui_from_string(xml)

        if MAC:
            macmenu = MacMenu()
            macmenu.set_menu_bar(self.uimanager.get_widget('/MenuBar'))
            quit_item = self.uimanager.get_widget('/MenuBar/actions/quit')
            macmenu.set_quit_menu_item(quit_item)
            group = macmenu.add_app_menu_group()
            group.add_app_menu_item(self.uimanager.get_widget('/MenuBar/actions/settings'), None)
            macdock = MacDock()
            macdock.connect('quit-activate', lambda d: self.sunset())

        return self.uimanager.get_widget('/MenuBar')

    def handle_item_gt0(self, widget):
        action = widget.get_name()

        if not len(self.imbrowser.get_selection()) > 0:
            utils.report_error(_("Your selection should exist of one or more photos"), "warning")
            return 

        if action == "date":
            dialog = dialogs.DateDialog(self.imbrowser.get_selection(), self.dindex)
            r = dialog.run()
            dialog.destroy()
            if r == -3:
                dt0 = dialog.get_datetime()
                
                dt1 = self.dindex[self.imbrowser.get_selection()[-1]]
                
                progress = dialogs.SunProgress(_("Changing time/date"),icon="clock")
                progress.start(len(self.imbrowser.get_selection()))
                for path in self.imbrowser.get_selection():
                    dt2 = self.dindex[path]          
                    dt3 = dt2 + (dt0-dt1)
                    progress.set_text(os.path.basename(path))
                    self.watcher.set_date(path, dt3)
                    self.imbrowser.viewer.update_date(path)
                    self.editor.update_label()
                    progress.plus()
                progress.destroy()
                self.imbrowser.refresh_albums(select=self.imbrowser.album)
                
        if action == "tags":
            dialog = dialogs.TagDialog(self.tagindex,self.imbrowser.get_selection())
            r = dialog.run()
            tags = dialog.get_data()
            dialog.destroy()
            if r == gtk.RESPONSE_OK:
                for t in tags:
                    v = tags[t]
                    for f in self.imbrowser.get_selection():
                        if v == True:
                            if t not in self.tagindex[f]:
                                self.tagindex[f].append(t)
                        elif v == False:
                            if t in self.tagindex[f]:
                                self.tagindex[f].remove(t)
                               
                progress = dialogs.SunProgress(_("Modifying tags"),icon="tag2")
                progress.start(len(self.imbrowser.get_selection()))
                for im in self.imbrowser.get_selection():
                    metadata.write_tags(im, self.tagindex[im])
                    progress.plus()
                progress.destroy()
                self.reload_tags()
                self.watcher.update()                

        if action == "edit":
            self.switch_edit(self.imbrowser.get_selection()[0])
        if action == "wall":
            tools.set_wallpaper(self.imbrowser.get_selection()[0])
        if action == "html":
            SunHTML(self.imbrowser.get_selection(), title=self.imbrowser.viewer.title)
        if action == "picasaweb":
            # gdata is a optional component check for existence
            if not self.check_opt("gdata"): return  
            PicasaWeb(self.imbrowser.get_selection(),self.imbrowser.album, self.config)
        if action == "flickr":
            # flickr is a optional component check for existence
            if not self.check_opt("flickr"): return
            
            dialog = dialogs.FlickrDialog(self.imbrowser.album)
            r = dialog.run()

            if r == gtk.RESPONSE_CANCEL:
                dialog.destroy(); return
            data = dialog.get_data()
            dialog.destroy()
            
            Flickr(self.imbrowser.get_selection(), *data)
            
        if action == "folder":
            SunFolder(self.imbrowser.get_selection(), title=self.imbrowser.viewer.title)
        
        if action == "slideswf":
            # should be gt1, so redirecting
            self.handle_item_gt1(widget)
            
        if action == "email":
            if not self.check_opt("email"): return
            dialog = dialogs.EmailDialog(self.config["email"])
            r = dialog.run()
            data = dialog.get_data()
            dialog.destroy()
            if r == gtk.RESPONSE_CANCEL:
                return
            web.Email(self.imbrowser.get_selection(), self.config, *data)
            
         
       
        if action == "rleft":
            progress = dialogs.SunProgress(_("Rotating photos"), icon="rotate-left")
            progress.start(len(self.imbrowser.get_selection()))
            for im in self.imbrowser.get_selection():            
                self.editor.draw.load(im)
                self.editor.draw.rotate_left()
                progress.plus()
            progress.destroy()
        if action == "rright":
            progress = dialogs.SunProgress(_("Rotating photos"), icon="rotate-right")
            progress.start(len(self.imbrowser.get_selection()))
            for im in self.imbrowser.get_selection():
                self.editor.draw.load(im)
                self.editor.draw.rotate_right()
                progress.plus()
            progress.destroy()

        if action == "export":
            dialog = dialogs.ChooseExportDialog(None)
            r = dialog.run()
            w = dialog.get_export()
            dialog.destroy()
            if r == gtk.RESPONSE_OK:
                self.handle_item_gt0(w)

        if action == "removedisk":
            fs = self.imbrowser.get_selection()
            
            question_all = False
            for f in fs:
                if question_all == False:
                    question, question_all = utils.question(_("Are you sure you want to delete this photo?"), buttons=[(gtk.STOCK_OK, gtk.RESPONSE_OK), (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)], widget=True)

                if question == gtk.RESPONSE_OK:
                    try: 
                        print "remove ", os.path.dirname(f) + os.sep + "Originals" + os.sep + os.path.basename(f)
                        os.remove(os.path.dirname(f) + os.sep + "Originals" + os.sep + os.path.basename(f))
                    except: pass
                    try: 
                        print "remove", f
                        os.remove(f)
                    except: pass
            
        if action == "print":
            dialog = dialogs.PrintDialog(title=self.imbrowser.viewer.title)
            r = dialog.run()
            title, n = dialog.get_data()
            settings = dialog.get_settings()
            dialog.destroy()
            if r == gtk.RESPONSE_OK:
                toprint = []
                for f in self.imbrowser.get_selection():
                    toprint.append((self.dindex[f],f))
                intiprint.intiprint(toprint, title, layout=PRINT_LAYOUT[n], settings=settings, win=self.win, filename=self.config["print_filename"], date=self.config["print_date"])   	
            

    def handle_item_gt1(self, widget):
        action = widget.get_name()
        
        if not len(self.imbrowser.get_selection()) > 1:
            utils.report_error(_("Your selection should exist of two or more photos"), "warning")
            return

        if action == "pano":
            # Check if autopano-(c), hugin-tools and is installed
            if not self.check_opt("pano"): return
            dialog = gtk.MessageDialog(flags=gtk.DIALOG_MODAL, type=gtk.MESSAGE_WARNING, buttons=gtk.BUTTONS_OK_CANCEL)
            dialog.set_markup(_("Intipunki is about to start panorama creation.\nCreating panoramas is quite a time consuming process, \nare you sure you want to continue?"))
            dialog.set_title(_("Intipunku panorama creation"))
            r = dialog.run()
            dialog.destroy()  
            if r == gtk.RESPONSE_CANCEL:
                return
            
            pano = tools.pano(self.imbrowser.get_selection())
            

        if action == "slideswf":
            # Ming is optional check
            if not self.check_opt("ming"): return
            dialog = dialogs.FlashExportDialog(album=self.imbrowser.album)
            r = dialog.run()       
            folder, title = dialog.get_output_info() 
            dialog.destroy()     
            if r == -5:
                swfexport.create_swfslide(self.imbrowser.get_selection(), folder + os.sep + title.replace(" ", "_").lower() + ".swf")
                
        if action == "collage":
            
            dialog = dialogs.CollageDialog()
            r = dialog.run()
            if r == gtk.RESPONSE_OK:
                style = dialog.get_data()
                folder = os.path.dirname(self.imbrowser.get_selection()[0])
                fs = self.imbrowser.get_selection()
                fs.sort()
                f = "collage_" + style + "_" + os.path.splitext(os.path.basename(fs[0]))[0] + "-" + os.path.splitext(os.path.basename(fs[-1]))[0]  + ".jpg"
                
                dest = os.path.join(folder, f)
                tools.collage(self.imbrowser.get_selection(), dest, mode=style)
            dialog.destroy()
            #
            

    def handle_item(self, widget):
        action = widget.get_name()

        if action == "settings":
            dialog = dialogs.IntiSettings(self.config)
            r = dialog.run()
            if r == gtk.RESPONSE_ACCEPT:
                dialog.save_settings()
                if self.config["default_import"]:
                    utils.set_default_import()
            dialog.destroy()
            
        if action == "components":
            dialog = dialogs.ComponentDialog(opts)
            r = dialog.run()
            o = dialog.get_data()
            dialog.destroy()
            packs = []
            checks = []
            if r == gtk.RESPONSE_OK:
                for c in o:
                    if o[c]: packs.append(c)
                optional.install(packs)
                for c in o:
                    cb,var,title = opts[c]
                    cb()
            

        if action == "quit":
            self.sunset()

        if action == "about":
            dialog = dialogs.AboutDialog(__title__, __version__, __authors__, __translators__, __dir__ + os.sep + "license.txt")
            dialog.run()
            dialog.destroy()
            
        if action == "new_album":
            dialog = dialogs.MessageInputDialog(markup=_("Please enter the name of the new album:"), entry_text="")
            r = dialog.run()
            dialog.destroy()
            if r == gtk.RESPONSE_CANCEL:
                return
            newfolder = dialog.get_text()
            emptyicon = utils.load_icon("empty").scale_simple(148,148,gtk.gdk.INTERP_HYPER)
            album = os.path.join(self.albumdir, newfolder)
            dir_util.mkpath(album)
            emptyicon.save(os.path.join(album, "intiplaceholder.png"), "png")
           
        if action == "rename_album":
            self.imbrowser.set_editing()
        if action == "selectall":
            self.imbrowser.select_all()
    
        if action == "import":
            dialog = dialogs.ChooseImportExportDialog()
            r = dialog.run()
            w = dialog.get_export()
            
            if r >= 10 and r <= 14: #import
                t, l = page_labels[r-10]
                # gdata is a optional component check for existence
                if t == "picasaweb":
                    if not self.check_opt("gdata"): return 
                # gphoto is a optional component check for existence
                if t == "camera":
                    if not self.check_opt("gphoto"): return 
                # scanner either
                if t == "scanner":
                    if not self.check_opt("scanner"): return
                    
                albums = self.imbrowser.collections
                
                # scanner doesnt follow the importpage idea
                if t == "scanner":
                    dialog2 = intisane.IntiSane(albums, self.config)
                    page = dialog2
                else:
                    dialog2 = dialogs.ImportDialog()
                    page = intiimport.pages[t](dialog2, albums, self.config, callback=self.handle_import) 
                    dialog2.set_page(page)
                    
                r = dialog2.run()
                if r == gtk.RESPONSE_OK:
                    page.doImport()
                dialog2.destroy()
            elif gtk.RESPONSE_OK: #export                            
                if r == gtk.RESPONSE_OK:
                    self.handle_item_gt0(w)
            dialog.destroy()
                

        if action == "slideshow":
            # If there is a selection, get the current selected
            all_ims = self.imbrowser.get_selection(select_all=True)
            all_ims2 = []
            for im in all_ims:
                if not os.path.splitext(im)[1].lower() in EXT_VID: all_ims2.append(im)
                
            if len(self.imbrowser.get_selection()) > 0:
                current_im = self.imbrowser.get_selection()[0] 
                if os.path.splitext(current_im)[1] in EXT_VID: c = 0          
                else: c = all_ims.index(current_im)
            else:
                c = 0
            win = slide.Slidewindow(all_ims2, config=self.config, select=c)
            win.fullscreen()
            win.show_all()
     

    def handle_import(self, name, *args):
        if name == "sunset":
            gobject.idle_add(args[0].destroy) 

    def reload_tags(self):
        #self.tagindex = metadata.reload_tags(self.tagindex)
        self.alltags = []
        for im in self.tagindex:
            for tag in self.tagindex[im]:
                if not tag in self.alltags:
                    self.alltags.append(tag)
        #self.update_index_childs()

    def switch_edit(self, im):
        self.editor.load(im)
        self.notebook.set_current_page(1)
        
        
    def check_opt(self, o, q=True):
        cb,var,title = opts[o]
        if not optional.check(o, cb, var, q=q): return False
        else:
            opts[o] = (cb, cb(), title)
            return True



if __name__ == "__main__":
    try:
        import psyco
        psyco.profile()
    except:
        pass#if not MAC: print "The psyco module does not seem to be installed. It is not necessary, however it can speed up performance."

    print "Intipunku initialising"
    app  = IntiPunku()

    try: gtk.main()
    except KeyboardInterrupt, err:
        print err
        sys.exit()
